/* File name     : camlib.c for HP-RT
 * Creation date : September 1993
 * Author        : Y.Yasu, Online group, Physics department, KEK
 * Modified date : June 1994, Y.Yasu
 *                 Adding DMA functions & KEK List Processing
 *               : July 1994, Y.Yasu
 *                 Bug fixed : mapping size was wrong.
 * Version 1.0   : August 1994, Y.Yasu
 *                 Clean up the code.
 * Version 1.01  : November 1994, Y.Yasu
 *                 Internal data buffer has been eliminated
 * Version 1.02  : May 1995, Y.Yasu
 *                 clean up Interrupt handling code
 */

#include <stdio.h>
#include <sys/file.h>
#include <sys/errno.h>
#include <sys/ioctl.h>
#include <shmmap.h>
#include <memlk.h>
#include "camlib.h"

extern errno;

/* Local Storage */
static int cc_sys_status;          /* temporally system status storage */
static int cc_status;              /* current camac status */
static int cc_path;
static int cc_retlen;
static struct CAMAC *cc_CAMAC;
static u_short cc_camac_qx;        /* camac q & x status : true = 1 */
static u_short cc_camac_err;       /* camac error code of */
static int cc_cur_branch;
static int cc_cur_crate;
static int cc_crate_pattern[CC_K_MAX_CRATE];
static int cc_signal_flag;         /* signal handling flag */
static int cc_event_count;
static int cc_busy = CC_K_READY;
static union swaps {
  long longs;
  short shorts[2];
} swaps;

static volatile caddr_t vme_vaddr;     /* virtual address */
static int vme_mapsize;
static int vme_offset;
static int vme_pagesize;
static int vme_fd;       /* file descriptor for VME_A16D16 device */

/* internal register for list processor */
static int cc_s_reg;               /* s register */
static int cc_t_reg;               /* t register */
static int cc_a_reg;               /* a register */
static int cc_event_count;         /* event counter */
/* list processing */
static u_short *cc_ptr_list;   /* current pointer to list_buffer */
static int cc_len_list;        /* actual length processed */
static u_short *cc_ptr_udata_s;/* first pointer to user's buffer address */
static u_short *cc_ptr_udata;  /* current pointer to user's buffer address */
static u_short *cc_ptr_udata_h;/* header pointer */
static int cc_len_udata_t;     /* total length of user's buffer */
static int cc_len_udata;       /* actual length of user's buffer */
static int cc_branch_flag;      /* flow control */
static u_short cc_list_area[CC_LIST_MAX_LENGTH]; /* list code area */

/* Internal routines of CAMAC list processing */
extern  cc_stop(),
        cc_read16(),  cc_read24(),  cc_read16_a(),  cc_read24_a(),
        cc_write16(), cc_write24(), cc_write16_a(), cc_write24_a(),
        cc_ndt(), cc_setcrate_a(), cc_setcrate(), cc_genz(),
        cc_genc(), cc_remi(), cc_seti(), cc_enbint(), cc_dsbint(),
        cc_qwait(),
        cc_qs16(), cc_qs24(), cc_qs16_a(), cc_qs24_a(),
        cc_as16(), cc_as24(), cc_as16_a(), cc_as24_a(),
        cc_igq16(), cc_igq24(), cc_igq16_a(), cc_igq24_a(),
        cc_move_as(), cc_move_at(), cc_move_sa(), cc_move_st(),
        cc_move_ta(), cc_move_ts(),
        cc_move32_a(), cc_move32_s(), cc_move32_t(), cc_move16_a_ptr(),
        cc_move32_a_ptr(), cc_move16_ptr(), cc_move32_ptr(),
        cc_move16_a_head(), cc_move16_head_a(), cc_move16_head(),
        cc_load_ptr(), cc_load_remain(), cc_load_timer(),
        cc_load_count(), cc_load_qx(),
        cc_head(), cc_exchg_at(), cc_swap_a(),
        cc_bis32_at(), cc_bis32_ta(), cc_bis32_a(), cc_bis32_t(),
        cc_bic32_at(), cc_bic32_ta(), cc_bic32_a(), cc_bic32_t(),
        cc_add32_at(), cc_add32_ta(), cc_add32_a(), cc_add32_t(),
        cc_sub32_at(), cc_sub32_ta(), cc_sub32_a(), cc_sub32_t(),
        cc_mul32_at(), cc_mul32_ta(), cc_mul32_a(), cc_mul32_t(),
        cc_div32_at(), cc_div32_ta(), cc_div32_a(), cc_div32_t(),
        cc_waitint(), cc_waitevent(),
        cc_br(), cc_beq(), cc_bne(), cc_bgt(), cc_bge(),
        cc_blt(), cc_ble(), cc_bbc(), cc_bbs(),
        kl_stop(),
        kl_read16(),  kl_read24(), kl_write16(), kl_write24(), kl_ndt(),
        kl_qs16(), kl_qs24(), kl_as16(), kl_as24(), kl_igq16(), kl_igq24();

int (*instruction_call)();
void *instruction[0x80] = {
/* 0 */                 cc_stop,

/* 1,2,3,4 */           cc_read16,  cc_read24,  cc_read16_a,  cc_read24_a,
/* 5,6,7,8 */           cc_write16, cc_write24, cc_write16_a, cc_write24_a,

/* 9,A,B,C */           cc_ndt, cc_setcrate_a, cc_setcrate, cc_genz,
/* D,E,F,10,11 */       cc_genc, cc_remi, cc_seti, cc_enbint, cc_dsbint,

/* 12 */                cc_qwait,

/* 13,14,15,16 */       cc_qs16, cc_qs24, cc_qs16_a, cc_qs24_a,

/* 17,18,19,1A */       cc_as16, cc_as24, cc_as16_a, cc_as24_a,

/* 1B,1C,1D,1E */       cc_igq16, cc_igq24, cc_igq16_a, cc_igq24_a,

/* 1F - 22 */           cc_stop, cc_stop, cc_stop, cc_stop,

/* 23,24,25,26 */       cc_move_as, cc_move_at, cc_move_sa, cc_move_st,
/* 27,28 */             cc_move_ta, cc_move_ts,

/* 29,2A,2B,2C */       cc_move32_a, cc_move32_s, cc_move32_t, cc_move16_a_ptr,
/* 2D,2E,2F */          cc_move32_a_ptr, cc_move16_ptr, cc_move32_ptr,
/* 30,31,32 */          cc_move16_a_head, cc_move16_head_a, cc_move16_head,

/* 33,34,35 */          cc_load_ptr, cc_load_remain, cc_load_timer,
/* 36,37 */             cc_load_count, cc_load_qx,

/* 38,39,3A */          cc_head, cc_exchg_at, cc_swap_a,

/* 3B,3C,3D,3E */       cc_bis32_at, cc_bis32_ta, cc_bis32_a, cc_bis32_t,

/* 3F,40,41,42 */       cc_bic32_at, cc_bic32_ta, cc_bic32_a, cc_bic32_t,

/* 43,44,45,46 */       cc_add32_at, cc_add32_ta, cc_add32_a, cc_add32_t,

/* 47,48,49,4A */       cc_sub32_at, cc_sub32_ta, cc_sub32_a, cc_sub32_t,

/* 4B,4C,4D,4E */       cc_mul32_at, cc_mul32_ta, cc_mul32_a, cc_mul32_t,

/* 4F,50,51,52 */       cc_div32_at, cc_div32_ta, cc_div32_a, cc_div32_t,

/* 53,54 */             cc_waitint, cc_waitevent,

/* 55,56,57,58,59 */    cc_br, cc_beq, cc_bne, cc_bgt, cc_bge,
/* 5A,5B,5C,5D */       cc_blt,cc_ble, cc_bbc, cc_bbs,
/* 5E,5F */             cc_stop, cc_stop,

};


/* CAM_Open, COPEN, CAMOPN */
int cam_open_( ) { return( COPEN( )); }
int camopn_( ) { return( COPEN( )); }
int copen_( ) { return( COPEN( )); }
int COPEN( ) {
    int i;

    if (cc_busy == CC_K_BUSY)
      return (EBUSY);
    else
      cc_busy = CC_K_BUSY;
    cc_path = open("/dev/cc", O_RDWR);
    if( cc_path == -1 ) {
      return ENODEV;
    }
    if ((vme_fd = open("/dev/vmeacc", O_RDWR)) == -1)
      return -1;
    /* if mmap fails, return value becomes -1 */
    vme_pagesize = getpagesize();
    vme_offset = (int)CC_CAMAC_ADDR&0xFFFF - VME_ACCBASE;
    vme_mapsize = sizeof(struct CAMAC) + vme_offset%vme_pagesize;
    vme_vaddr = (caddr_t)shmmap(vme_fd, 0, vme_mapsize,
				vme_offset/vme_pagesize*vme_pagesize,
				SHM_READ | SHM_WRITE );
    if (vme_vaddr == (volatile caddr_t) 0) {
        return -1;
    }
    cc_CAMAC = (struct CAMAC *)(vme_vaddr + vme_offset%vme_pagesize);
#ifdef DEBUG
    printf("address of CAMAC interface : %x\n", cc_CAMAC);
#endif
    cc_cur_branch = 0;
    cc_cur_crate = 0;
    cc_crate_pattern[0] = 0;
    cc_event_count = 0;
    for( i = 1; i < CC_K_MAX_CRATE; i++ )
        cc_crate_pattern[i] = cc_crate_pattern[i-1] + CC_CMDM_CRT;
    return 0;
}

/* CAM_Close, CAMCLS, CCLOSE */
int cam_close_( ) { return( CCLOSE( ) ); }
int camcls_( ) { return( CCLOSE( ) ); }
int cclose_( ) { return( CCLOSE( ) ); }
int CCLOSE( ) {

    cc_busy = CC_K_READY;
    shmunmap((void *)vme_vaddr, 0);
    if( (close(cc_path)) == -1 )
        return errno;
    return 0;
}

/* NAF */
int naf_( n, a, f )
int *n, *a, *f; {
    return (NAF( *n, *a, *f ));
}
int NAF( n, a, f )
int n, a, f; {
    return  ((((n<<4) + a) << 5) + f);
}

/* CRESET */
int creset_() { return( CRESET() ); }
int CRESET() {
    cc_init();
}

int cc_init() {
    register struct CAMAC *CAMAC = cc_CAMAC;

    CAMAC->CSR = CC_CSRM_RST;
    CAMAC->CSER = 0;
    CAMAC->DOCR = CC_DOCRK_INIT;
    CAMAC->SCCR = 0;
    CAMAC->MTCR = 0;
    CAMAC->LAMICR = CC_INTRK_IRQ4;
    CAMAC->DONICR = CC_INTRK_IRQ4;
    CAMAC->EMPICR = CC_INTRK_IRQ4;
    CAMAC->ABOICR = CC_INTRK_IRQ4;
    CAMAC->LAMIVR = CC_VECTRK_LAM;
    CAMAC->DONIVR = CC_VECTRK_DONE;
    CAMAC->EMPIVR = CC_VECTRK_EMPTY;
    CAMAC->ABOIVR = CC_VECTRK_ABORT;
    CAMAC->AMR = CC_AMRK_INIT;
    CAMAC->CMAR = CC_CMAK_SINGLEND;
    CAMAC->CMR = CC_CMDK_HALT;
    return 0;
}

/* CSETBR */
int csetbr_( branch ) int *branch; { return( CSETBR(*branch) ); }
int CSETBR( branch )
int branch; {
    return 0;
}

/* CSETCR */
int csetcr_( crate ) int *crate; { return( CSETCR(*crate) ); }
int CSETCR( crate )
int crate; {
    int status;

    if( (crate > (CC_K_MAX_CRATE-1)) || (crate < 0) )
        return EINVAL;
    cc_cur_crate = crate;
    return 0;
}

int cdmprg_(reg) struct reg2917s *reg; { return ( CDMPRG(reg) ); }
int CDMPRG( reg )
register struct reg2917s *reg; {
    register struct CAMAC *CAMAC = cc_CAMAC;

    reg->cser    = CAMAC->CSER;
    reg->docr    = CAMAC->DOCR;
    reg->sccr    = CAMAC->SCCR;
    reg->mtcr    = CAMAC->MTCR;
    reg->machi   = CAMAC->MACHI;
    reg->maclo   = CAMAC->MACLO;
    reg->lamicr  = ( CAMAC->LAMICR & 0xFF );
    reg->donicr  = ( CAMAC->DONICR & 0xFF );
    reg->empicr  = ( CAMAC->EMPICR & 0xFF );
    reg->aboicr  = ( CAMAC->ABOICR & 0xFF );
    reg->lamivr  = ( CAMAC->LAMIVR & 0xFF );
    reg->donivr  = ( CAMAC->DONIVR & 0xFF );
    reg->empivr  = ( CAMAC->EMPIVR & 0xFF );
    reg->aboivr  = ( CAMAC->ABOIVR & 0xFF );
    reg->cmar    = CAMAC->CMAR;
    reg->cwcr    = CAMAC->CWCR;
    reg->srr     = ( CAMAC->SRR & 0xFF );
    reg->csr     = CAMAC->CSR;
    return 0;
}

int cdmpcm_(cmd) struct camcmd *cmd; { return ( CDMPRG(cmd) ); }
int CDMPCM( cmd )
register struct camcmd *cmd; {
    register struct CAMAC *CAMAC = cc_CAMAC;

    CAMAC->CMAR = cmd->cmdadr;
    cmd->cmd = CAMAC->CMR;
    cmd->naf = CAMAC->CMR;
    cmd->wc = CAMAC->CMR;
    cmd->dummy = CAMAC->CMR;
    cmd->camhalt = CAMAC->CMR;
    return 0;
}

/* CGENZ */
int cgenz_() { return( CGENZ() ); }
int CGENZ() {
    int status, q, x = 0;
    short i=0;

    if( (status = CAMACW(NAFGEN(30,0,1),&i,&q,&x) ) != 0 )
        return status;
    i |= 1;
    return ( CAMACW(NAFGEN(30,0,17),&i,&q,&x) );
}

/* CGENC */
int cgenc_() { return( CGENC() ); }
int CGENC() {
    int status, q, x;
    short i=0;

    if( (status = CAMACW(NAFGEN(30,0,1),&i,&q,&x) ) != 0 )
        return status;
    i |= 2;
    return ( CAMACW(NAFGEN(30,0,17),&i,&q,&x) );
}

/* CSETI */
int cseti_() { return( CSETI() ); }
int CSETI() {
    int status, q, x;
    short i=0;

    if( (status = CAMACW(NAFGEN(30,0,1),&i,&q,&x)  ) != 0 )
	return status;
    i |= 4;
    return ( CAMACW(NAFGEN(30,0,17),&i,&q,&x) );
}

/* CREMI */
int cremi_() { return( CREMI() ); }
int CREMI() {
    int status, q, x;
    short i=0;

    if( (status = CAMACW(NAFGEN(30,0,1),&i,&q,&x) ) != 0 )
	return status;
    i &= ~4;
    return ( CAMACW(NAFGEN(30,0,17),&i,&q,&x) );
}

/* CAMACW */
int camacw_( naf, data, q, x )
u_short *data;
int *naf, *q, *x; {
    return CAMACW( *naf, data, q, x );
}
int CAMACW(naf, data, q, x)
u_short *data;
int naf, *q, *x; {
    int qx;

    camac_single(CC_K_MODE_WS16, naf, data, &qx);
    if( qx & CC_CSRM_NOQ )
        *q = 0;
    else
        *q = 1;
    if( qx & CC_CSRM_NOX )
        *x = 0;
    else
        *x = 1;
    return 0;
}

/* CAMACL, CAMAC */
int camacl_( naf, data, q, x )
int *naf, *data, *q, *x; {
    return CAMAC(*naf, data, q, x);
}
int camac_( naf, data, q, x )
int *naf, *data, *q, *x; {
    return CAMAC(*naf, data, q, x);
}
int CAMAC(naf, data, q, x)
int naf, *data, *q, *x; {
    int qx;

    camac_single( CC_K_MODE_WS24, naf, data, &qx);
    if( qx & CC_CSRM_NOQ )
        *q = 0;
    else
        *q = 1;
    if( qx & CC_CSRM_NOX )
        *x = 0;
    else
        *x = 1;
    return 0;
}

/* Cam_EnableLAM, CENLAM, CELAM */
int cam_enablelam_( mask )
int *mask; {
    return CENLAM( *mask );
}
int cenlam_( mask )
int *mask; {
    return CENLAM( *mask );
}
int celam_( mask )
int *mask; {
    return CENLAM( *mask );
}
int CENLAM(mask)
int mask; {
    int status, qx;
    int i= 0;

    if((status=camac_single(CC_CMDK_WS24, NAFGEN(30,13,17), &mask, &qx)) != 0)
        return status;
    if((status=camac_single( CC_CMDK_WS24, NAFGEN(30,0,1), &i, &qx)) != 0)
        return status;
    i |= 0x100;
    return (camac_single( CC_CMDK_WS24, NAFGEN(30,0,17), &i, &qx));
}

/* Cam_DisableLAM, CDSLAM, CDLAM */
int cam_disablelam_() {
    return CDSLAM();
}
int cdslam_() {
    return CDSLAM();
}
int cdlam_() {
    return CDSLAM();
}
int CDSLAM() {
    int mask = 0;
    int status, qx;
    int i= 0;

    i = 0;
    if((status=camac_single( CC_CMDK_WS24, NAFGEN(30,13,17), &i, &qx)) != 0 )
        return status;
    if((status=camac_single( CC_CMDK_WS24, NAFGEN(30,0,1), &i, &qx)) != 0 )
        return status;
    i  &= ~0x0100;
    return (camac_single( CC_CMDK_WS24, NAFGEN(30,0,17), &i, &qx));
}

/* Cam_WaitLAM, CWTLAM, CWLAM */
cam_waitlam_( timeout )
int *timeout; {
    return CWTLAM( *timeout );
}
int cwtlam_( timeout )
int *timeout; {
    return CWTLAM( *timeout );
}
int cwlam_( timeout )
int *timeout; {
    return CWTLAM( *timeout );
}
int CWTLAM(timeout)
int timeout; {
    int status;
    register struct CAMAC *CAMAC = cc_CAMAC;
    u_short reg;

    errno = 0;
/*    printf("ccioc_wait_lam : now wait...\n");*/
    if ( (ioctl(cc_path, CCIOC_WAIT_LAM, &timeout)) == -1 )
        return errno;
    return 0;
}

/* CESIG : not supported */
cesig_() { CESIG(); }
CESIG(funct)
void (*funct)();
{
    return 0;
}

/* CDSIG : not supported */
cdsig_() { CDSIG(); }
CDSIG()
{
    return 0;
}

/* CWSIG : not supported */
cwsig_() { CWSIG(); }
CWSIG()
{
    return 0;
}

/* CBWWAI, CDMAW, CamBloWorWAIt */
int cambloworwait_( mode, naf, buffer, length, ret_len, err_code )
u_short *buffer;
int *mode, *naf, *length, *ret_len, *err_code; {
    return CamBlokWait_common(
	 (*mode<<CC_K_MODES)|CC_K_MODE_BLOCK|CC_K_MODE_WS16,
	 *naf, buffer, (*length)*sizeof(short), ret_len, err_code );
}
int cbwwai_( mode, naf, buffer, length, ret_len, err_code )
u_short *buffer;
int *mode, *naf, *length, *ret_len, *err_code; {
    return CamBlokWait_common(
	 (*mode<<CC_K_MODES)|CC_K_MODE_BLOCK|CC_K_MODE_WS16,
	 *naf, buffer, (*length)*sizeof(short), ret_len, err_code );
}
int cdmaw_( mode, naf, buffer, length, ret_len, err_code )
u_short *buffer;
int *mode, *naf, *length, *ret_len, *err_code; {
    return CamBlokWait_common(
         (*mode<<CC_K_MODES)|CC_K_MODE_BLOCK|CC_K_MODE_WS16,
	 *naf, buffer, (*length)*sizeof(short), ret_len, err_code );
}
int CDMAW(mode, naf, buffer, length, ret_len, err_code )
u_short *buffer;
int mode, naf, length, *ret_len, *err_code; {
    return CamBlokWait_common(
         (mode<<CC_K_MODES)|CC_K_MODE_BLOCK|CC_K_MODE_WS16,
	 naf, buffer, length*sizeof(short), ret_len, err_code );
}

/* CBLWAI, CDMAL, CamBloLonWAIt */
int camblolonwait_( mode, naf, buffer, length, ret_len, err_code )
u_long *buffer;
int *mode, *naf, *length, *ret_len, *err_code; {
    int status, retlen;
    status =  CamBlokWait_common(
	 (*mode<<CC_K_MODES)|CC_K_MODE_BLOCK|CC_K_MODE_WS24,
	 *naf, buffer, (*length)*sizeof(long), &retlen, err_code );
    *ret_len = retlen/2;
    return status;
}
int cblwai_( mode, naf, buffer, length, ret_len, err_code )
u_long *buffer;
int *mode, *naf, *length, *ret_len, *err_code; {
    int status, retlen;
    status = CamBlokWait_common(
	 (*mode<<CC_K_MODES)|CC_K_MODE_BLOCK|CC_K_MODE_WS24,
	 *naf, buffer, (*length)*sizeof(long), &retlen, err_code );
    *ret_len = retlen/2;
    return status;
}
int cdmal_(mode, naf, buffer, length, ret_len, err_code )
u_long *buffer;
int *mode, *naf, *length, *ret_len, *err_code; {
    int status, retlen;
    status = CamBlokWait_common(
	 (*mode<<CC_K_MODES)|CC_K_MODE_BLOCK|CC_K_MODE_WS24,
	 *naf, buffer, (*length)*sizeof(long), &retlen, err_code );
    *ret_len = retlen/2;
    return status;
}
int CDMAL(mode, naf, buffer, length, ret_len, err_code )
u_long *buffer;
int mode, naf, length, *ret_len, *err_code; {
    int status, retlen;
    status = CamBlokWait_common(
	 (mode<<CC_K_MODES)|CC_K_MODE_BLOCK|CC_K_MODE_WS24,
	 naf, buffer, length*sizeof(long), &retlen , err_code );
    *ret_len = retlen/2;
    return status;
}

int CamBlokWait_common(mode, naf, buffer, length, ret_len, err_code )
u_long *buffer;
int mode, naf, length, *ret_len, *err_code; {
    int status;
    register struct CAMAC *CAMAC = cc_CAMAC;
    int i;

    i = length/2;
    status = camac_block(mode, naf, i, buffer);
    *err_code = CAMAC->CSR & CC_CSRK_EMASK;
    *ret_len = cc_retlen;
    return status;
}

int camac_single(mode,naf,data,qx)
int mode, naf, *qx;
u_short *data; {
    register struct CAMAC *CAMAC = cc_CAMAC;

    CAMAC->CSR = CC_CSRM_RST;
    CAMAC->CMAR = CC_CMAK_SINGLE;
    CAMAC->CMR = (u_short)(mode + cc_crate_pattern[cc_cur_crate]);
    CAMAC->CMR = (u_short)naf;
    CAMAC->CMAR = CC_CMAK_SINGLE;
    if( !(naf & CC_CMDK_F24) ) { /* CAMAC read function */
        CAMAC->CSR = CC_CSRM_GO;
	while(!(CAMAC->CSR & CC_CSRK_RD_ER));
	if( CAMAC->CSR & CC_CSRM_RDY ) {
	    if( mode & CC_CMDK_WS16 )
	        *data = CAMAC->DLR;
	    else {
	        *data++ = CAMAC->DHR & 0xFF;
	        *data = CAMAC->DLR;
	    }
	}
    } else if ( !(naf & CC_CMDK_F8) ) { /* CAMAC write function */
        CAMAC->CSR = CC_CSRK_WR_GO;
	if( mode & CC_CMDK_WS16 )
	    CAMAC->DLR = *data;
	else {
	    CAMAC->DHR = *data++;
	    CAMAC->DLR = *data;
	}
	while( !(CAMAC->CSR & CC_CSRK_DN_ER) );
    } else { /* non-data transfer */
        CAMAC->CSR = CC_CSRM_GO;
	while( !(CAMAC->CSR & CC_CSRK_DN_ER) );
    }
    *qx = CAMAC->CSR;
    return 0;
}

int camac_block( mode, naf, len, buffer )
int mode, naf, len;
char *buffer; {
    register struct CAMAC *CAMAC = cc_CAMAC;
    int status;

    CAMAC->CMAR = CC_CMAK_BLOCK;
    CAMAC->CMR = (u_short)(mode + cc_crate_pattern[cc_cur_crate]);
    CAMAC->CMR = (u_short)naf;
    CAMAC->CMR = 0;
    CAMAC->CMR = 0xFFFF;
    errno = 0;
    if( naf & CC_CMDK_F16 ) {
        CAMAC->DOCR = CC_DOCRK_INIT;
	CAMAC->CSR = CC_CSRM_DMA | CC_CSRM_WRITE;
	status = write(cc_path, buffer, len*2);
    } else {
        CAMAC->DOCR = CC_DOCRK_INIT + CC_DOCRK_READ;
	CAMAC->CSR = CC_CSRM_DMA;
	status = read(cc_path, buffer, len*2);
    } 
    cc_retlen = status/2;
    return(errno);
}

/* ===================
 * KEK List processing
 * ===================
*/

/* CamLisExeWAIt, CLEWAI */
camlisexewait_(list, lenlis, buf, lenbuf, iosb)
u_short *list, *buf;
int *lenlis, *lenbuf;
struct cc_iosb *iosb;
{
    return CLEWAI(list, *lenlis, buf, *lenbuf, iosb);
  }
clewai_(list, lenlis, buf, lenbuf, iosb)
u_short *list, *buf;
int *lenlis, *lenbuf;
struct cc_iosb *iosb;
{
    return CLEWAI(list, *lenlis, buf, *lenbuf, iosb);
  }
CLEWAI(list, lenlis, buf, lenbuf, iosb)
u_short *list, *buf;
int lenlis, lenbuf;
struct cc_iosb *iosb;
{
  errno = 0;
  cc_load_list(list, lenlis);
  cc_s_reg = iosb->s_reg;
  cc_exec_list(buf, lenbuf);
  cc_put_iosb(iosb);
  return errno;
}

/* CamExeWAIt, CEWAI */
camexewait_(buf, lenbuf, iosb)
u_short *buf;
int *lenbuf;
struct cc_iosb *iosb;
{
  return CEWAI(buf, *lenbuf, iosb);
}

cewai_(buf, lenbuf, iosb)
u_short *buf;
int *lenbuf;
struct cc_iosb *iosb;
{
  return CEWAI(buf, *lenbuf, iosb);
}

CEWAI(buf, lenbuf, iosb)
u_short *buf;
int lenbuf;
struct cc_iosb *iosb;
{
  errno = 0;
  cc_s_reg = iosb->s_reg;
  cc_exec_list(buf, lenbuf);
  cc_put_iosb(iosb);
  return errno;
}

/* CamLisLOAD, CLLOAD */
camlisload_(list, lenlis, err)
u_short *list;
int *lenlis;
int *err;
{
    return CLLOAD(list, *lenlis, err);
  }
clload_(list, lenlis, err)
u_short *list;
int *lenlis;
int *err;
{
    return CLLOAD(list, *lenlis, err);
  }
CLLOAD(list, lenlis, err)
u_short *list;
int lenlis;
int *err;
{
  *err = 0;
  errno = 0;
  cc_load_list(list, lenlis);
  return errno;
}

/* CamReaLIST, CRLIST */
camrealist_(fname, list, lenlis, retlen, err)
char *fname;
u_short *list;
int *lenlis, *retlen, *err;
{
  return CRLIST(fname, list, *lenlis, retlen, err);
}
crlist_(fname, list, lenlis, retlen, err)
char *fname;
u_short *list;
int *lenlis, *retlen, *err;
{
  return CRLIST(fname, list, *lenlis, retlen, err);
}
CRLIST(fname, list, lenlis, retlen, err)
char *fname;
u_short *list;
int lenlis, *retlen, *err;
{
  register FILE *fp;
  int size, a, b, c, d;
  
  if ((fp = fopen(fname, "r")) == NULL) {
    *err = errno;
    return errno;
  }
  (void)fscanf(fp, "%X", &size);
  if (size > lenlis) {
    *err = -1;
    return -1;
  }
  *retlen = size;
  while (size > 0) {
    (void)fscanf(fp, "%X%X%X%X", &a, &b, &c, &d);
    *list++ = a;
    *list++ = b;
    *list++ = c;
    *list++ = d;
    size -= 4;
  }
  *err = 0;
  return fclose(fp);
}

cc_load_list( list, lenlist )
u_short *list;
int lenlist;
{
  cc_len_list = lenlist;
  memcpy( cc_list_area, list, lenlist*sizeof(u_short) );
}

cc_exec_list( buf, lenbuf )
u_short *buf;
int lenbuf;
{
  struct CAMAC *CAMAC = cc_CAMAC;
  int step_count = 0, code;

  cc_ptr_udata_s = cc_ptr_udata = buf;
  cc_len_udata_t = lenbuf;
  cc_len_udata = 0;
  cc_ptr_list = cc_list_area;       /* initialize list pointer */

  if( !cc_len_list )
    cc_status = CC_STA_LIST_EMPTY;
  /* this code will find infinite loop */
  while (cc_status == CC_STA_OK  &&
	 cc_sys_status == CC_STA_OK &&
	 step_count++ < CC_LIST_MAX_STEP &&
	 cc_signal_flag == 0 &&
	 cc_ptr_list <= (cc_list_area + cc_len_list)) {
    code = 0x00FF & cc_ptr_list[0];
#ifdef DEBUG
      printf("camac_list: step_count = %d next code=0x%x\n",
	     step_count, code);
    printf("code = %x : done...\n", code);
#endif
    instruction_call = instruction[code];
    cc_status = (*instruction_call)();
    next_instruction();
  }

#ifdef DEBUG
    printf("camac_list: status = 0x%x  last code = 0x%x\n",
	   cc_status, code);
#endif
  cc_camac_err = CAMAC->CSR & CC_CSRK_EMASK;
  if( cc_status == CC_STA_LIST_STOP )
    cc_status = CC_STA_OK;
  else if ( step_count >= CC_LIST_MAX_STEP )
    cc_status = CC_STA_LIST_OVERSTEP;
}

cc_put_iosb(iosb)
struct cc_iosb *iosb;
{
  iosb->status = cc_status;
  iosb->ret_length = cc_len_udata;
  iosb->s_reg = cc_s_reg;
  iosb->devinfo = cc_camac_err;
}

/* ===================================
 * subroutines for KEK list processing
 * ===================================
*/

next_instruction()
{
    if (cc_branch_flag)
        cc_branch_flag = 0;
    else
        cc_ptr_list += (0xFF00 & *cc_ptr_list) >> 8;
    return 0;
}

check_buf(wc)
int wc;                                     /* wc: word counts */
{
#ifdef DEBUG
    printf("check_buf OUT: ptr_u=0x%x len_u=%d%d\n",
            cc_ptr_udata, cc_len_udata);
#endif
    if (cc_len_udata + wc >= cc_len_udata_t - cc_len_udata) {
#ifdef DEBUG
        printf("check_buf OUT: List stop\n");
#endif
        return (CC_STA_LIST_EMPTY);
    } else
        return 0;
}

cc_stop()
{
    return CC_STA_LIST_STOP;
}

cc_read16()
{
    register int status;
    int qx;

    if (status = check_buf(1))
        return status;
    if (status = camac_single(CC_CMDK_WS16, (int)cc_ptr_list[1],
			      cc_ptr_udata, &qx))
        return status;
    cc_ptr_udata += 1;
    cc_len_udata += 1;
    return 0;
}

cc_read24()
{
    register int status;
    int qx;

    if (status = check_buf(2))
        return status;
    if (status = camac_single(CC_CMDK_WS24, (int)cc_ptr_list[1],
			      cc_ptr_udata, &qx))
        return status;
    cc_ptr_udata[0] &= 0xFF;
    cc_ptr_udata += 2;
    cc_len_udata += 2;
    return 0;
}

cc_read16_a()
{
    int qx;

    camac_single(CC_CMDK_WS16, (int)cc_ptr_list[1],
			&swaps.shorts[0], &qx);
    cc_a_reg = (int)swaps.shorts[0];
}

cc_read24_a()
{
    register int status;
    int qx;

    status = camac_single(CC_CMDK_WS24, (int)cc_ptr_list[1],
			  &cc_a_reg, &qx);
    cc_a_reg &= 0xFFFFFF;
    return status;
}

cc_write16()
{
    int qx;

    return camac_single(CC_CMDK_WS16, (int)cc_ptr_list[1],
			&cc_ptr_list[2], &qx);
}

cc_write24()
{
    int qx;

    swaps.shorts[0] = cc_ptr_list[3];
    swaps.shorts[1] = cc_ptr_list[2];
    return camac_single(CC_CMDK_WS24, (int)cc_ptr_list[1],
			&swaps.longs, &qx);
}

cc_write16_a()
{
    int qx;

    swaps.longs = cc_a_reg;
    return camac_single(CC_CMDK_WS16, (int)cc_ptr_list[1],
			&swaps.shorts[1], &qx);
}


cc_write24_a()
{
    int qx;
    return camac_single(CC_CMDK_WS24, (int)cc_ptr_list[1],
			&cc_a_reg, &qx);
}

cc_ndt()
{
    int qx;
    return camac_single(CC_CMDK_WS16, (int)cc_ptr_list[1], &swaps.longs, &qx);
}

cc_setcrate_a()
{
    if (cc_a_reg >= 0 && cc_a_reg < CC_K_MAX_CRATE)
        cc_cur_crate = cc_a_reg;
    return 0;
}

cc_setcrate()
{
    if (cc_ptr_list[1] >= 0 && cc_ptr_list[1] < CC_K_MAX_CRATE)
        cc_cur_crate = (int)cc_ptr_list[1];
    return 0;
}

cc_genz()
{
    register int status;
    int dat;
    int qx;

    if (status = camac_single(CC_CMDK_WS24, NAFGEN(30, 0, 1), &dat, &qx))
        return status;
    dat |= 1;
    return camac_single(CC_CMDK_WS24, NAFGEN(30, 0, 17), &dat, &qx);
}

cc_genc()
{
    register int status;
    int dat;
    int qx;

    if (status = camac_single(CC_CMDK_WS24, NAFGEN(30, 0, 1), &dat, &qx))
        return status;
    dat |= 2;
    return camac_single(CC_CMDK_WS24, NAFGEN(30, 0, 17), &dat, &qx);
}

cc_remi()
{
    register int status;
    int dat;
    int qx;

    if (status = camac_single(CC_CMDK_WS24, NAFGEN(30, 0, 1), &dat, &qx))
        return status;
    dat &= ~4;
    return camac_single(CC_CMDK_WS24, NAFGEN(30, 0, 17), &dat, &qx);
}

cc_seti()
{
    register int status;
    int dat;
    int qx;

    if (status = camac_single(CC_CMDK_WS24, NAFGEN(30, 0, 1), &dat, &qx))
        return status;
    dat |= 4;
    return camac_single(CC_CMDK_WS24, NAFGEN(30, 0, 17), &dat, &qx);
}

cc_enbint()
{
    register int status;
    int dat;
    int qx;

#ifdef LIST_EXTENSION
    dat = cc_a_reg;
#else
    dat = 0xFFFFFF;                          /* mask */
#endif
    cc_signal_flag = 0;
    if (status = camac_single(CC_CMDK_WS24, NAFGEN(30,13,17), &dat, &qx))
        return status;
    if (status = camac_single(CC_CMDK_WS24, NAFGEN(30, 0, 1), &dat, &qx))
        return status;
    dat |= 0x0100;
    return camac_single(CC_CMDK_WS24, NAFGEN(30, 0, 17), &dat, &qx);
}

cc_dsbint()
{
    register int status;
    int dat;
    int qx;

    dat = 0x0;                              /* mask */
    if (status = camac_single(CC_CMDK_WS24, NAFGEN(30,13,17), &dat, &qx))
        return status;
    if (status = camac_single(CC_CMDK_WS24, NAFGEN(30, 0, 1), &dat, &qx))
        return status;
    dat &= ~0x0100;
    return camac_single(CC_CMDK_WS24, NAFGEN(30, 0, 17), &dat, &qx);
}

cc_qwait()
{
    register int status, i;
    int dat;
    int qx;

    cc_camac_qx = status = 0;
    for (i = 0; status == 0 && (cc_camac_qx & CC_CAMAC_Q) == 0 &&
	 i < cc_ptr_list[2]; i++)
        status = camac_single(CC_CMDK_WS24, cc_ptr_list[1], &dat, &qx);
    if (i >= cc_ptr_list[2])
        return CC_STA_LIST_TIMEOUT;
    else
        return status;
}

cc_qs16()
{
    int status;

    if (status = check_buf((int)cc_ptr_list[2]))
        return status;
    status = camac_block(CC_CMDK_BLK_QS_16, (int)cc_ptr_list[1],
		       (int)cc_ptr_list[2], cc_ptr_udata);
    cc_ptr_udata += cc_ptr_list[2];
    cc_len_udata += cc_ptr_list[2];
    return status;
}

cc_qs24()
{
    int status;

    if (status = check_buf((int)cc_ptr_list[2] * 2))
        return status;
    status = camac_block(CC_CMDK_BLK_QS_24, (int)cc_ptr_list[1],
		       (int)cc_ptr_list[2] * 2 , cc_ptr_udata);
    cc_ptr_udata += cc_ptr_list[2] * 2;
    cc_len_udata += cc_ptr_list[2] * 2;
    return status;
}

cc_qs16_a()
{
    int status;

    if (status = check_buf(cc_a_reg))
        return status;
    status = camac_block(CC_CMDK_BLK_QS_16, (int)cc_ptr_list[1],
		       cc_a_reg, cc_ptr_udata);
    cc_ptr_udata += cc_a_reg;
    cc_len_udata += cc_a_reg;
    return status;
}

cc_qs24_a()
{
    int status;

    if (status = check_buf(cc_a_reg * 2))
        return status;
    status = camac_block(CC_CMDK_BLK_QS_24, cc_ptr_list[1],
		       cc_a_reg * 2 , cc_ptr_udata);
    cc_ptr_udata += cc_a_reg * 2;
    cc_len_udata += cc_a_reg * 2;
    return status;
}

cc_as16()
{
    int status;

    if (status = check_buf((int)cc_ptr_list[2]))
        return status;
    status = camac_block(CC_CMDK_BLK_AS_16, (int)cc_ptr_list[1],
		       (int)cc_ptr_list[2], cc_ptr_udata);
    cc_ptr_udata += cc_ptr_list[2];
    cc_len_udata += cc_ptr_list[2];
    return status;
}

cc_as24()
{
    int status;

    if (status = check_buf((int)cc_ptr_list[2] * 2))
        return status;
    status = camac_block(CC_CMDK_BLK_AS_24, (int)cc_ptr_list[1],
		       (int)cc_ptr_list[2] * 2 , cc_ptr_udata);
    cc_ptr_udata += cc_ptr_list[2] * 2;
    cc_len_udata += cc_ptr_list[2] * 2;
    return status;
}

cc_as16_a()
{
    int status;

    if (status = check_buf(cc_a_reg))
        return status;
    status = camac_block(CC_CMDK_BLK_AS_16, (int)cc_ptr_list[1],
		       cc_a_reg, cc_ptr_udata);
    cc_ptr_udata += cc_a_reg;
    cc_len_udata += cc_a_reg;
    return status;
}

cc_as24_a()
{
    int status;

    if (status = check_buf(cc_a_reg * 2))
        return status;
    status = camac_block(CC_CMDK_BLK_AS_24, (int)cc_ptr_list[1],
		       cc_a_reg * 2 , cc_ptr_udata);
    cc_ptr_udata += cc_a_reg * 2;
    cc_len_udata += cc_a_reg * 2;
    return status;
}

cc_igq16()
{
    int status;

    if (status = check_buf((int)cc_ptr_list[2]))
        return status;
    status = camac_block(CC_CMDK_BLK_IG_16, (int)cc_ptr_list[1],
		       (int)cc_ptr_list[2], cc_ptr_udata);
    cc_ptr_udata += cc_ptr_list[2];
    cc_len_udata += cc_ptr_list[2];
    return status;
}

cc_igq24()
{
    int status;

    if (status = check_buf((int)cc_ptr_list[2] * 2))
        return status;
    status = camac_block(CC_CMDK_BLK_IG_24, (int)cc_ptr_list[1],
		       (int)cc_ptr_list[2] * 2 , cc_ptr_udata);
    cc_ptr_udata += cc_ptr_list[2] * 2;
    cc_len_udata += cc_ptr_list[2] * 2;
    return status;
}

cc_igq16_a()
{
    int status;

    if (status = check_buf(cc_a_reg))
        return status;
    status = camac_block(CC_CMDK_BLK_IG_16, (int)cc_ptr_list[1],
		       cc_a_reg, cc_ptr_udata);
    cc_ptr_udata += cc_a_reg;
    cc_len_udata += cc_a_reg;
    return status;
}

cc_igq24_a()
{
    int status;

    if (status = check_buf(cc_a_reg * 2))
        return status;
    status = camac_block(CC_CMDK_BLK_IG_24, (int)cc_ptr_list[1],
		       cc_a_reg * 2 , cc_ptr_udata);
    cc_ptr_udata += cc_a_reg * 2;
    cc_len_udata += cc_a_reg * 2;
    return status;
}

int cc_move_as()
{
    cc_s_reg = cc_a_reg;
    return 0;
}

int cc_move_at()
{
    cc_t_reg = cc_a_reg;
    return 0;
}

int cc_move_sa()
{
    cc_a_reg = cc_s_reg;
    return 0;
}

int cc_move_st()
{
    cc_t_reg = cc_s_reg;
    return 0;
}

int cc_move_ta()
{
    cc_a_reg = cc_t_reg;
    return 0;
}

int cc_move_ts()
{
    cc_s_reg = cc_t_reg;
    return 0;
}

int cc_move32_a()
{
    cc_a_reg = Tolong(1);
    return 0;
}

int cc_move32_s()
{
    cc_s_reg = Tolong(1);
    return 0;
}

int cc_move32_t()
{
    cc_t_reg = Tolong(1);
    return 0;
}

int cc_move16_a_ptr()
{
    register int status;

    if (status = check_buf(1))
        return status;
    swaps.longs = cc_a_reg;
    *cc_ptr_udata++ = swaps.shorts[1];
    cc_len_udata++;
    return 0;
}

int cc_move32_a_ptr()
{
   register int status;

    if (status = check_buf(2))
        return status;
   swaps.longs = cc_a_reg;
    *cc_ptr_udata++ = swaps.shorts[0];
    *cc_ptr_udata++ = swaps.shorts[1];
    cc_len_udata += 2;
    return 0;
}

int cc_move16_ptr()
{
    register int status;

    if (status = check_buf(1))
        return status;
    *cc_ptr_udata++ = cc_ptr_list[1];
    cc_len_udata++;
    return 0;
}

int cc_move32_ptr()
{
    register int status;

    if (status = check_buf(2))
        return status;
    *cc_ptr_udata++ = cc_ptr_list[2];
    *cc_ptr_udata++ = cc_ptr_list[1];
    cc_len_udata += 2;
    return 0;
}

int cc_move16_a_head()
{
    return memcpy(cc_ptr_udata_h + cc_ptr_list[1], (u_short *)&cc_a_reg, 2);
}

int cc_move16_head_a()
{
    return memcpy((u_short *)&cc_a_reg, cc_ptr_udata_h + cc_ptr_list[1], 2);
}

int cc_move16_head()
{
    return memcpy(cc_ptr_udata_h + cc_ptr_list[1], &cc_ptr_list[2], 2);
}

int cc_load_ptr()
{
    cc_a_reg = cc_len_udata;
    return 0;
}

int cc_load_remain()
{
    cc_a_reg = cc_len_udata_t - cc_len_udata;
    return 0;
}

int cc_load_timer()
{
    cc_a_reg = 0;
    return 0;
}

int cc_load_count()
{
    cc_a_reg = cc_event_count;
    return 0;
}

int cc_load_qx()
{
    cc_a_reg = cc_camac_qx;
    return 0;
}

int cc_head()
{
    cc_ptr_udata_h = cc_ptr_udata;
    cc_ptr_udata += cc_ptr_list[1];
    cc_len_udata += cc_ptr_list[1];
    return 0;
}

int cc_exchg_at()
{
    register u_int i;

    i = cc_a_reg;
    cc_a_reg = cc_t_reg;
    cc_t_reg = i;
    return 0;
}

int cc_swap_a()
{
    register u_short temp;

    swaps.longs = cc_a_reg;
    temp = swaps.shorts[0];
    swaps.shorts[0] = swaps.shorts[1];
    swaps.shorts[1] = temp;
    cc_a_reg = swaps.longs;
    return 0;
}


int cc_bis32_at()
{
    cc_t_reg |= cc_a_reg;
    return 0;
}

int cc_bis32_ta()
{
    cc_a_reg |= cc_t_reg;
    return 0;
}

int cc_bis32_a()
{
    cc_a_reg |= Tolong(1);
    return 0;
}

int cc_bis32_t()
{
    cc_t_reg |= Tolong(1);
    return 0;
}

int cc_bic32_at()
{
    cc_t_reg &= ~cc_a_reg;
    return 0;
}

int cc_bic32_ta()
{
    cc_a_reg &= ~cc_t_reg;
    return 0;
}

int cc_bic32_a()
{
    cc_a_reg &= ~Tolong(1);
    return 0;
}

int cc_bic32_t()
{
    cc_t_reg &= ~Tolong(1);
    return 0;
}

int cc_add32_at()
{
    cc_t_reg += cc_a_reg;
    return 0;
}

int cc_add32_ta()
{
    cc_a_reg += cc_t_reg;
    return 0;
}

int cc_add32_a()
{
    cc_a_reg += Tolong(1);
    return 0;
}

int cc_add32_t()
{
    cc_t_reg += Tolong(1);
    return 0;
}

int cc_sub32_at()
{
    cc_t_reg -= cc_a_reg;
    return 0;
}

int cc_sub32_ta()
{
    cc_a_reg -= cc_t_reg;
    return 0;
}

int cc_sub32_a()
{
    cc_a_reg -= Tolong(1);
    return 0;
}

int cc_sub32_t()
{
    cc_t_reg -= Tolong(1);
    return 0;
}

int cc_mul32_at()
{
    cc_t_reg *= cc_a_reg;
    return 0;
}

int cc_mul32_ta()
{
    cc_a_reg *= cc_t_reg;
    return 0;
}

int cc_mul32_a()
{
    cc_a_reg *= Tolong(1);
    return 0;
}

int cc_mul32_t()
{
    cc_t_reg *= Tolong(1);
    return 0;
}

int cc_div32_at()
{
    if (cc_a_reg == 0)
        return CC_STA_LIST_ZERODIV;
    cc_t_reg /= cc_a_reg;
    return 0;
}

int cc_div32_ta()
{
    if (cc_t_reg == 0)
        return CC_STA_LIST_ZERODIV;
    cc_a_reg /= cc_t_reg;
    return 0;
}

int cc_div32_a()
{
    register int i;

    if ((i = Tolong(1)) == 0)
        return CC_STA_LIST_ZERODIV;
    cc_a_reg /= i;
    return 0;
}

int cc_div32_t()
{
    register int i;

    if ((i = Tolong(1)) == 0)
        return CC_STA_LIST_ZERODIV;
    cc_t_reg /= i;
    return 0;
}

int cc_waitint()
{
    int timeout = CC_K_TIMEOUT_LAM;

    if ( (ioctl(cc_path, CCIOC_WAIT_LAM, &timeout)) == -1 )
      return CC_STA_LIST_TIMEOUT;
    return 0;
}

int cc_waitevent()
{
    int timeout = CC_K_TIMEOUT_LAM;

    if ( (ioctl(cc_path, CCIOC_WAIT_LAM, &timeout)) == -1 )
        return CC_STA_LIST_TIMEOUT;
    cc_event_count++;
    return 0;
}

int cc_br()
{
    cc_ptr_list = cc_list_area + cc_ptr_list[1] - 1;
    cc_branch_flag = 1;
    return 0;
}

int cc_beq()
{
    if (cc_t_reg == Tolong(2)) {
        cc_ptr_list = cc_list_area + cc_ptr_list[1] - 1;
        cc_branch_flag = 1;
    }
    return 0;
}

int cc_bne()
{
    if (cc_t_reg != Tolong(2)) {
        cc_ptr_list = cc_list_area + cc_ptr_list[1] - 1;
        cc_branch_flag = 1;
    }
    return 0;
}

int cc_bgt()
{
    if (cc_t_reg > Tolong(2)) {
        cc_ptr_list = cc_list_area + cc_ptr_list[1] - 1;
        cc_branch_flag = 1;
    }
    return 0;
}

int cc_bge()
{
    if (cc_t_reg >= Tolong(2)) {
        cc_ptr_list = cc_list_area + cc_ptr_list[1] - 1;
        cc_branch_flag = 1;
    }
    return 0;
}

int cc_blt()
{
    if (cc_t_reg < Tolong(2)) {
        cc_ptr_list = cc_list_area + cc_ptr_list[1] - 1;
        cc_branch_flag = 1;
    }
    return 0;
}

int cc_ble()
{
    if (cc_t_reg <= Tolong(2)) {
        cc_ptr_list = cc_list_area + cc_ptr_list[1] - 1;
        cc_branch_flag = 1;
    }
    return 0;
}

int cc_bbc()
{
    if (!(cc_t_reg & (1 << Tolong(2)))) {
        cc_ptr_list = cc_list_area + cc_ptr_list[1] - 1;
        cc_branch_flag = 1;
    }
    return 0;
}


int cc_bbs()
{
    if (cc_t_reg & (1 <<  Tolong(2))) {
        cc_ptr_list = cc_list_area + cc_ptr_list[1] - 1;
        cc_branch_flag = 1;
    }
    return 0;
}






