/********************************************************
 * rpclib.c
 * author C. Timmermans, University of Michigan
 ********************************************************/

/************************************************************************
*                                                                       *
* Copyright (c) 1993                                                    *
* The Regents of The University of Michigan                             *
* Universities Research Association, Inc.                               *
* All Rights Reserved                                                   *
*                                                                       *
*                Authors:                                               *
*                        R. Ball, C. Timmermans, UofM                   *
*                        Y. Takeuchi, T.I.T.                            *
*                        M. Nomachi, KEK                                *
*                        A. Fry, C. Erbas, D. Brenner, SSCL             *
*                                                                       *
************************************************************************/

#ifdef VXWORKS
#include <stdio.h> 
#include <string.h>
#include <rpc/rpc.h>
#define memcpy(s1,s2,i) bcopy(s2,s1,i)
#else
#include<stdio.h>
#include <string.h>
#include <strings.h>
#include<rpc/rpc.h>
#include<netdb.h>
#include<sys/socket.h>
#endif
#include "template.h"
#include "ipc_msg.h"

extern char this_node[];

rpc_init()
{
#ifdef VXWORKS
  rpcTaskInit();
#endif
}

CLIENT *rpc_Link_Server(node,inbufsize,outbufsize)
char *node;
int inbufsize,outbufsize;
{
  CLIENT *cl;
#if !defined(VXWORKS)
  int sock;
  struct hostent *hp;
  struct sockaddr_in server_addr;
#endif

#ifdef VXWORKS
  cl=clnt_create(node,MESSAGEPROG,MESSAGEVERS,"tcp"); 
#else
  sock = RPC_ANYSOCK;
  if((hp = gethostbyname(node)) == NULL){
    printf("Cannot get address for %s.\n",node);
    return(NULL);
  }
  bcopy(hp->h_addr,(caddr_t)&server_addr.sin_addr,hp->h_length);
  server_addr.sin_family = AF_INET;
  server_addr.sin_port = 0;
  cl = clnttcp_create(&server_addr,MESSAGEPROG,MESSAGEVERS,&sock,
                      inbufsize,outbufsize);
#endif
  return(cl);
}


int rpc_Send_Message(dst,msg)
char *dst,*msg;
{
  char *node;
  char mssend[1000],dest[100];
  char *dt_go;
  CLIENT *cl;
  int *result;

  strcpy(dest,dst);
  node = strrchr(dest,'@');
  if(node == NULL) return(-6);
  *node = '\0';
  node+=1;
  strcpy(mssend,dest);
  strcat(mssend," ");
  strcat(mssend,msg);
  dt_go = mssend;  
/*  printf("Trying to send %s\n",dt_go); */
  cl = rpc_Link_Server(node,sizeof(int),strlen(mssend));
  if(cl == NULL) return(-5);
  result = sendmessage_1(&dt_go, cl);
  clnt_destroy(cl);
  if(result == NULL){
    clnt_perror(cl,node);
    printf("No return at all\n");
    return(-4);
  }
  return(*result);
}

int rpc_Check_Process(node,proc)
char *node,*proc;
{
  CLIENT *cl;
  int *result;

  cl = rpc_Link_Server(node,sizeof(int),strlen(proc));
  if(cl == NULL) return(-5);
  result = checkprocess_1(&proc, cl);
  clnt_destroy(cl);
  if(result == NULL){
    clnt_perror(cl,node);
    printf("No return at all\n");
    return(-4);
  }
  return(*result);
}
int rpc_Start_Process(node,proc)
char *node,*proc;
{
  CLIENT *cl;
  int *result;

  cl = rpc_Link_Server(node,sizeof(int),strlen(proc));
  if(cl == NULL) return(-5);
  result = startprocess_1(&proc, cl);
  clnt_destroy(cl);
  if(result == NULL){
    clnt_perror(cl,node);
    printf("No return at all\n");
    return(-4);
  }
  return(*result);
}

void rpc_Update_Table(index,proc,node,status)
int index,status;
char *proc,*node;
{
  Table entry;
  int change;

  if(index < 0 ){
    if(status == DEAD ) return;
    strcpy(entry.process_name,proc);
    strcpy(entry.node,node);
    entry.status = status;
    entry.process_id = 0;
    entry.msgq_id = 0;
    Add_to_Table(&entry);
    return;
  }
  Read_from_Table(&entry,index);
  if((strncmp(entry.node,this_node,strlen(this_node)) == 0) ||
     (entry.node[0] == '\0')) {
    /***** Do NOT modify the table if the process is alive on this node ****/
    if(entry.status == ALIVE) return;
  }
  change = 0;
  if(((strncmp(entry.node,node,strlen(node)) != 0) &&
      (strncmp(entry.node,node,strlen(entry.node)) != 0)) ||
     (entry.node[0] == '\0') ){
    if(status == ALIVE) {
      change = 1;
      entry.process_id = 0;
      entry.msgq_id = 0;
      strcpy(entry.node,node);
    }
    /*** Do NOT modify nodes if the new status is dead ***/
    else return;
  }
  if(entry.status != status) {
    change = 1;
    entry.status = status;
  }
  if(change == 1) Write_to_Table(&entry,index); 
}

int rpc_get_nodes(nodelist,nodesize,n_nodes)
char *nodelist;
int nodesize;
int *n_nodes;
{
  FILE *fp;
  char fnam[200],junk[100];
  char *snd_node;

  *n_nodes = 0;
  if(decode_filename("$(USERUNIDAQ)/conf/nodefile",fnam) < 0){
    printf("Cannot decode USERUNIDAQ (/conf/nodefile)\n");
    return(-10);
  }
  if((fp = fopen(fnam,"r")) == NULL){
    printf("Cannot open %s\n",fnam);
    return(-11);
  }
  snd_node = nodelist;
  while(fscanf(fp,"%s",snd_node) > 0 ) {
    fscanf(fp,"%s",junk);
    if(strlen(snd_node) >= nodesize) snd_node[nodesize-1] = '\0'; 
    snd_node+=nodesize;
    *n_nodes = *n_nodes + 1;
  }
  fclose(fp);
  return(0);
}

int rpc_sp_read_value(dst,varname,type,value)
char *dst,*varname,*value;
short *type;
{
  char *node,dest[200];
  CLIENT *cl;
  struct rsp_ret *result;
  struct rsp_send sval;
  int size;

  strcpy(dest,dst);
  node = strrchr(dest,'@');
  if(node == NULL) return(-6);
  *node = '\0';
  node+=1;

  *type = -1;
  cl = rpc_Link_Server(node,sizeof(rsp_ret),sizeof(rsp_send));
  if(cl == NULL) return(-5);
  strcpy(sval.procname,dest);
  strcpy(sval.varname,varname);
  result = readsp_1(&sval, cl); 
  clnt_destroy(cl);
  if(result == NULL){
    clnt_perror(cl,node);
    printf("No return at all\n");
    return(-4);
  }
  *type = result->type;
  if(result->type == INTEGER)     memcpy(value,(char *)&(result->i_val),sizeof(int));
  else if(result->type == REAL)   memcpy(value,(char *)&(result->f_val),sizeof(float));
  else if(result->type == STRING) strcpy(value,result->s_val);
  size = result->rtcode;
  xdr_free(xdr_rsp_ret,(char *)result);
  return(size);
}

int rpc_sp_read_all_values(dst,varname,varsize,type,value,valsize,n_var)
char *dst,*varname,*value;
int valsize,varsize,*n_var;
short *type;
{
  char *node,dest[200];
  CLIENT *cl;
  struct rsp_ret_all *result;
  struct rsp_send_all sval;
  int i,size,i_n,i_v;

  strcpy(dest,dst);
  node = strrchr(dest,'@');
  if(node == NULL) return(-6);
  *node = '\0';
  node+=1;

  *n_var = 0;
  cl = rpc_Link_Server(node,sizeof(rsp_ret_all),sizeof(rsp_send_all));
  if(cl == NULL) return(-5);
  strcpy(sval.procname,dest);
  sval.valsize = valsize;
  sval.varsize = varsize;
  result = readspall_1(&sval, cl); 
  clnt_destroy(cl);
  if(result == NULL){
    clnt_perror(cl,node);
    printf("No return at all\n");
    return(-4);
  }
  *n_var = result->n_var;
  i_n = 0;
  i_v = 0;
  for(i=0;i<result->n_var;i++){
    type[i] = result->type[i];
    strcpy(&(varname[i_n]),&(result->varname[i_n]));
    i_n +=varsize;
    if(type[i] == INTEGER)     
      memcpy(&(value[i_v]),(char *)&(result->i_val[i]),sizeof(int));
    else if(type[i] == REAL)   
      memcpy(&(value[i_v]),(char *)&(result->f_val[i]),sizeof(float));
    else if(type[i] == STRING) 
      memcpy(&(value[i_v]),&(result->s_val[i_v]),valsize);
    i_v +=valsize;
  }
  i = result->rtcode;
  xdr_free(xdr_rsp_ret_all,(char *)result); 
  return(i);
}

