/*******************************************************************
 * splib.c      -- status path, version 0.2                        *
 * author:   C. Timmermans, University of Michigan                 *
 *******************************************************************/

/************************************************************************
*                                                                       *
* Copyright (c) 1993                                                    *
* The Regents of The University of Michigan                             *
* Universities Research Association, Inc.                               *
* All Rights Reserved                                                   *
*                                                                       *
*                Authors:                                               *
*                        R. Ball, C. Timmermans, UofM                   *
*                        Y. Takeuchi, T.I.T.                            *
*                        M. Nomachi, KEK                                *
*                        A. Fry, C. Erbas, D. Brenner, SSCL             *
*                                                                       *
************************************************************************/
#ifdef VXWORKS
#include "vxshm.h"
#include<stdio.h>
#define key_t int
#define memcpy(s1,s2,i) bcopy(s2,s1,i)
#else
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <stdlib.h>
#include<stdio.h>
#include <sys/stat.h>
#include <errno.h>
#endif
#include "status_path.h"
#include "template.h"
#include "generic.h"

int n_lock_requests;
struct status_path *this_path;
struct proc_entry *this_proc;
struct status_path *path[MAX_SP];
int n_path;

extern char ownpname[];
extern char this_node[];
extern struct var_dict_type var_dict_usr[];
extern int var_number_usr;
extern struct var_dict_type var_dict_sys[];
extern int var_number_sys;

int link_to_sp()
{
  this_path = NULL;
  this_proc = NULL;
  n_path = 0;
  if(sp_link_path() < 0) return(-2);
  return(0);
}

int add_proc_to_sp(size)
int size;
{
  int n_buf,i,ret_val;
  struct proc_entry *p;
  struct status_path *sp;
  char p_name[80];

  if(n_path == 0) return(-1);
  n_buf = size/B_SIZE;
  if((B_SIZE*n_buf) < size) n_buf++;
/*** If the requested size is just too big, don't do it ***/
  if(n_buf > NR_DB_BUFF) return(-5);
/*** check if process already exists ***/
  strcpy(p_name,ownpname);
  ownpname[0]='\0';
  if((i = sp_get_proc_structure(p_name,&p,&sp)) == 0) {
    strcpy(ownpname,p_name);
    return(-4);
  }
  strcpy(ownpname,p_name);
  i = 0;
  lock_sp();
  while(1) {
/*** Check if there is space in this path ***/
    if((path[i]->empty_segments >= n_buf) &&
       (path[i]->empty_proc_entry > 0)) break;
/*** Try the next path ***/
    i++;
    if(i>= n_path) {
      ret_val = sp_link_path();
      if(ret_val < 0) {
	free_sp();
	return(-3);
      }
    }
  }
  this_path = path[i];
  ret_val = sp_add_proc(ownpname,path[i],n_buf);
  free_sp();
  if(ret_val != 0) return(-2);
  return(0);
}

int add_variable_to_sp(varname,type,length,value)
short type,length;
char *varname,*value;
{
  struct data_buffer *d;
  struct data_pointer d_seg;
  short *s;
  char *c;
  int size,space,ret_val;

  if((this_path == NULL) || (this_proc == NULL)) return(-1);
/*** check if variable is already there ***/
  if(sp_get_datasegment(this_path,this_proc,varname,&d_seg) == 0) return(-2);
/*** Calculate the amount of space needed, adding 2 shorts and a character ***/
  size = 5 + strlen(varname) + 1 + length;
/*** make sure the next word will be aligned ***/
  if((2*(size/2)) != size) size++;
/*** check if datasegment fits within a buffersegment ***/
  d = &(this_path->buf_info[this_proc->first]);
  space = 1 + d->b_last - d->b_first;
  if(size >= (space - 2)) return(-3);
/*** find the first available buffersegment, big enough to hold the data ***/
  while(d != NULL) {
    c = &(this_path->buffer[d->b_first]);
    s = (short *)c;
    while(*s > 0) {
      c+=(int)(*s);
      s = (short *)c;
    }
    space = 1 + ((int)&(this_path->buffer[d->b_last]))- (int)c;
    if((space-2) > size) break;
    if(d->next == -1) d = NULL;
    else d = &(this_path->buf_info[d->next]);
    if(d == NULL) {
      ret_val = sp_add_buffersegment();
      if(ret_val < 0) return(-4);
      d = &(this_path->buf_info[this_proc->first]);
      while(d->next != -1) d = &(this_path->buf_info[d->next]);
    }
  }
/*** fill the datasegment ***/
  sp_fill_datasegment(c,(short)size,type,'N',varname,value);
/*** and set the next 0 ***/
  c+=size;
  s = (short *)c;
  *s = 0;
  return(0);
}

int set_value_in_sp(varname,value)
char *varname,*value;
{
  struct data_pointer d_seg;
  int space,size,ret_val;

  if(n_path == 0) return(-1);
  if((ret_val = sp_get_datasegment(this_path,this_proc,varname,&d_seg)) < 0) 
    return(-2);
/*** d_seg now contains data segment pointers ***/
/*** Calculate the amount of space available ***/
  space = 1 + ((int)(d_seg.p_last)) - ((int)(d_seg.p_val));
/*** Calculate how much space is needed ***/
  if(*(d_seg.p_type) == INTEGER) size = sizeof(int);
  else if (*(d_seg.p_type) == REAL) size = sizeof(float);
  else size = strlen(value)+1;
  if(space < size) return(-3);
/*** copy the data ***/
  memcpy(d_seg.p_val,value,size);
  return(0);
}

int read_value_from_sp(procname,varname,type,value)
char *procname,*varname,*value;
short *type;
{
   char dest[100],node[100];
   int index;
   Table entry;

   char nlist[50][100];
   int i,n_nodes;
   int result;

   Get_Proc_and_Host(procname,dest,node);
   index = Search_Table(dest);

   if(node[0] != '\0') {
     if((strncmp(node,this_node,strlen(this_node)) == 0) ||
        (strncmp(node,this_node,strlen(node)) == 0))
       return(sp_read_value(dest,varname,type,value));
     else {
       result = rpc_sp_read_value(procname,varname,type,value);
       if(result >= 0) rpc_Update_Table(index,dest,node,ALIVE);
       if(result == -3)  rpc_Update_Table(index,dest,node,DEAD);
       return(result);
     }
   } 

   result = -1;
   if(index >= 0) {
     Read_from_Table(&entry,index);
     if((strncmp(entry.node,this_node,strlen(entry.node)) == 0) ||
	(entry.node[0] == '\0')) {
       if(entry.status != DEAD) 
	 return(sp_read_value(procname,varname,type,value));
     }
     else if(entry.status != DEAD) {
       sprintf(dest,"%s%c%s",procname,'@',entry.node);
       result = rpc_sp_read_value(dest,varname,type,value);
       if(result == -3)  rpc_Update_Table(index,procname,entry.node,DEAD);
     }
   }
   if(result >=0) return(result);

    if((result = rpc_get_nodes(nlist,100,&n_nodes)) < 0) return(result);
    result = -1;
    i = 0;
    while((i<n_nodes) && (result != 1)){
      if(strncmp(this_node,nlist[i],strlen(nlist[i])) != 0) { 
        result = rpc_Check_Process(nlist[i],dest);
      }
      i++;
    }
    if(result == 1) {
      sprintf(dest,"%s%c%s",procname,'@',nlist[i-1]);
      rpc_Update_Table(index,procname,nlist[i-1],ALIVE);
      return(rpc_sp_read_value(dest,varname,type,value));
    }
    return(-12);
}

int sp_read_value(procname,varname,type,value)
char *procname,*varname,*value;
short *type;
{
  int size,ret_val;
  struct proc_entry *p;
  struct data_pointer d_seg;
  struct status_path *sp;

  if(n_path == 0) return(-1);
  if((ret_val = sp_get_proc_structure(procname,&p,&sp)) < 0) return(-3);
/*** locate the datasegment ***/
  if((ret_val = sp_get_datasegment(sp,p,varname,&d_seg)) < 0) return(-2);
/*** Calculate how much space is to be copied ***/
  if(*(d_seg.p_type) == INTEGER) size = sizeof(int);
  else if (*(d_seg.p_type) == REAL) size = sizeof(float);
  else size = strlen(d_seg.p_val)+1;
/*** copy the type and value ***/
  *type = *(d_seg.p_type);
  memcpy(value,d_seg.p_val,size);
  return(0);
}

int set_all_values_in_sp()
{
  int i;

  if(n_path == 0) return(-1);
/*** sets the value of all user and system level variables in the SP ***/
  for(i=0;i<var_number_sys;i++){
    set_value_in_sp(var_dict_sys[i].name,(char *)(var_dict_sys[i].addr));
  }
  for(i=0;i<var_number_usr;i++){
    set_value_in_sp(var_dict_usr[i].name,(char *)(var_dict_usr[i].addr));
  }
  return(0);
}

int read_all_values_from_sp(procname,varname,varsize,type,value,valsize,n_var)
char *procname,*varname,*value;
int valsize,varsize,*n_var;
short *type;
{
   char dest[100],node[100];
   int index;
   Table entry;

   char nlist[50][100];
   int i,n_nodes;
   int result;

   Get_Proc_and_Host(procname,dest,node);
   index = Search_Table(dest);

   if(node[0] != '\0') {
     if((strncmp(node,this_node,strlen(this_node)) == 0) ||
        (strncmp(node,this_node,strlen(node)) == 0))
       return(sp_read_all_values(dest,varname,varsize,type,value,valsize,
				 n_var));
     else {
       result = rpc_sp_read_all_values(procname,varname,varsize,type,value,
				       valsize,n_var);
       if(result >= 0) rpc_Update_Table(index,dest,node,ALIVE);
       if(result == -2) rpc_Update_Table(index,dest,node,DEAD);
       return(result);
     }
   } 

   result = -1;
   if(index >= 0) {
     Read_from_Table(&entry,index);
     if((strncmp(entry.node,this_node,strlen(entry.node)) == 0) ||
	(entry.node[0] == '\0')) {
       if(entry.status != DEAD) 
	 return(sp_read_all_values(procname,varname,varsize,type,value,valsize,
			      n_var));
     }
     else if(entry.status != DEAD) {
       sprintf(dest,"%s%c%s",procname,'@',entry.node);
       result = rpc_sp_read_all_values(dest,varname,varsize,type,value,
				       valsize,n_var);
       if(result == -2)  rpc_Update_Table(index,procname,entry.node,DEAD);
     }
   }
   if(result >=0) return(result);

    if((result = rpc_get_nodes(nlist,100,&n_nodes)) < 0) return(result);
    result = -1;
    i = 0;
    while((i<n_nodes) && (result != 1)){
      if(strncmp(this_node,nlist[i],strlen(nlist[i])) != 0) { 
        result = rpc_Check_Process(nlist[i],dest);
      }
      i++;
    }
    if(result == 1) {
      sprintf(dest,"%s%c%s",procname,'@',nlist[i-1]);
      rpc_Update_Table(index,procname,nlist[i-1],ALIVE);
      return(rpc_sp_read_all_values(dest,varname,varsize,type,value,valsize,
				    n_var));
    }
    return(-12);
}

int sp_read_all_values(procname,varname,varsize,type,value,valsize,n_var)
char *procname,*varname,*value;
int valsize,varsize,*n_var;
short *type;
{
  struct proc_entry *p;
  struct status_path *sp;
  struct data_buffer *d;
  struct data_pointer dp;
  char *c,*v,*vl;
  short *s;
  int i,size;

  *n_var = 0;
  if(n_path == 0) return(-1);
  if(sp_get_proc_structure(procname,&p,&sp) < 0) return(-2);
  v = varname;
  vl = value;
  i = 0;
  d = &(sp->buf_info[p->first]);
  while(d != NULL){
    c = &(sp->buffer[d->b_first]);
    s = (short *)c;
    while(*s != 0) {
      sp_fill_data_pointer(c,&dp);
      strcpy(v,dp.p_var);
      v+=varsize;
      type[i] = *(dp.p_type);
      if((*(dp.p_type)) == INTEGER) size = sizeof(int);
      else if ((*(dp.p_type)) == REAL) size = sizeof(float);
      else size = strlen(dp.p_val)+1;
      memcpy(vl,dp.p_val,size);
      vl+=valsize;
      c = dp.p_last+1;;
      s = (short *) c;
      i++;
    }
    if(d->next == -1) d = NULL;
    else d = &(sp->buf_info[d->next]);
  }
  *n_var = i - 1;
  return(0);
}

int remove_proc_from_sp(procname)
char *procname;
{
  int ret_val;
  struct proc_entry *p;
  struct status_path *sp;

  if(n_path == 0) return(-1);
  if(sp_get_proc_structure(procname,&p,&sp) < 0) return(-2);
  sp_stor_critvar(p,sp);
  if(sp_remove_proc(p,sp) < 0) return(-3);
  if(strcmp(ownpname,procname) == 0){
    this_proc = NULL;
    this_path = NULL;
  }
  return(0);
}

int unlink_sp()
{
  int i;

  for(i = 0;i<n_path;i++) {
    shmdt(path[i]);
  }
  n_path = 0;
  return(0);
}

int Crit_Var(name)
char *name;
{
  struct data_pointer d_seg;

  if(sp_get_datasegment(this_path,this_proc,name,&d_seg) < 0)return(-1);
/*** list the variable to be critical ***/
  *(d_seg.p_crit) = 'Y';
  sp_get_critval(name,d_seg.p_val);
  return(0);
}

int sp_link_path()
{
  int ret_val;

  ret_val = sp_link_existing_path();
  if(ret_val < 0) {
    ret_val = sp_link_new_path();
  }
  return(ret_val);
}

int sp_link_existing_path()
{
  key_t key;
  int size = sizeof(struct status_path);
  int shmflg = 00666;
  int shmid;

  if(n_path >= MAX_SP) return(-2);
  key = FIRST_SP_KEY+ n_path;
  if((shmid = shmget(key,size,shmflg)) < 0) return(-1);
  path[n_path] = (struct status_path *)shmat(shmid,0,SHM_RND);
  if(((int)path[n_path]) == -1) return(-1);
  n_path++;
  return(0);
}

int sp_link_new_path()
{
  key_t key;
  int size = sizeof(struct status_path);
  int shmflg = 00666 | IPC_CREAT | IPC_EXCL;
  int shmid;

  if(n_path >= MAX_SP) return(-2);
  key = FIRST_SP_KEY+ n_path;
  if((shmid = shmget(key,size,shmflg)) < 0) return(-1);
  path[n_path] = (struct status_path *)shmat(shmid,0,SHM_RND);
  if(((int)path[n_path]) == -1) return(-1);
/*** Path is created and linked, initialize the path ***/
  if(n_path == 0) {
    n_lock_requests = 0;
    if(Create_Sem(&(path[0]->sp_lock)) == ERROR) return(-3);
#ifndef VXWORKS
    Update_Sem_Wait(path[0]->sp_lock,SP_FREE);
#endif    
  }
  sp_init_path(path[n_path]);
  n_path++;
  return(0);
}

sp_init_path(sp)
struct status_path *sp;
{
  int i,t;
  short *ts;

  lock_sp();
  sp->empty_segments = NR_DB_BUFF;
  sp->empty_proc_entry = MAX_DB_PROC;
  for(i=0;i<MAX_DB_PROC;i++) {
    sp->proc_info[i].name[0] = '\0';
    sp->proc_info[i].first = -1;
  }
  for(i=0;i<NR_DB_BUFF;i++) {
    sp->buf_info[i].proc = -1;
    sp->buf_info[i].next = -1;
    t = i*B_SIZE;
/*** make sure it is aligned ***/
    if((2*(t/2)) != t) t++;
/*** make sure the last address of a segment is known ***/
    if(i > 0) sp->buf_info[i-1].b_last = t - 1;
/*** say the segment is empty ***/
    ts = (short *)(&(sp->buffer[t]));
    *ts = 0;
    sp->buf_info[i].b_first = t; 
  }
/*** and also the last address ***/
  sp->buf_info[NR_DB_BUFF - 1].b_last = DB_BUFF_SIZE -1; 
  free_sp();
}

int sp_add_proc(p_name,sp,n_buf)
char *p_name;
struct status_path *sp;
int n_buf;
{
  struct proc_entry *p;
  struct data_buffer *d,*dt;
  int i,j;
  short *s;

  lock_sp();
  if((sp->empty_segments < n_buf) ||
     (sp->empty_proc_entry <= 0)) {
    free_sp();
    return(-5);
  }
/*** locate an empty process structure ***/
  for(i=0;i<MAX_DB_PROC;i++) {
    p = &(sp->proc_info[i]);
    if(p->first == -1) break;
  }
  if(p->first != -1) {
    free_sp();
    return(-6);
  }
/*** found the empty process structure, use it ***/
  sp->empty_proc_entry-=1;
  this_proc = p;
  strcpy(p->name,p_name);
/*** locate all free buffer segments needed ***/
  i = 0;
  for(j=0;j<n_buf;j++) {
    while (i<NR_DB_BUFF) {
      if(sp->buf_info[i].proc == -1) break;
      i++;
    }
    if(sp->buf_info[i].proc != -1) {
/*** there are not enough free buffers, restore original situation ***/
      sp_remove_proc(p,sp);
      free_sp();
      return(-7);
    }
/*** found a free buffer, add it ***/
    sp_add_known_buffer(sp,p,&(sp->buf_info[i]));
  }
  free_sp();
  return(0);
}

int sp_add_buffersegment()
{
  int i,n_buf, ret_val;
  struct data_buffer *d,*dt,*d_old;
  short *s;
  struct status_path *old_path;
  struct proc_entry *old_proc;

  lock_sp();
  if(this_path->empty_segments > 0) {
/*** add one empty segment to the process ***/
    i = 0;
    while(i<NR_DB_BUFF){
      if(this_path->buf_info[i].proc == -1) break;
      i++;
    }
    if(this_path->buf_info[i].proc != -1) {
      free_sp();
      return(-1);
    }
    sp_add_known_buffer(this_path,this_proc,&(this_path->buf_info[i]));
    free_sp();
    return(0);
  }
  free_sp();
/*** have to move all data to a different memory ***/
  n_buf = 0;
  d= &(this_path->buf_info[this_proc->first]);
  while( d != NULL) {
    n_buf++;
    if(d->next == -1) d = NULL;
    else d = &(this_path->buf_info[d->next]);
  }
  n_buf +=1;
/*** check if the status path limit on nr of buffers is crossed ***/
  if(n_buf > NR_DB_BUFF) return(-1);
/*** find a path with enough free buffers ***/
  lock_sp();
  i = 0;
  while (1){
    if((path[i]->empty_segments >=n_buf) &&
       (path[i]->empty_proc_entry > 0)) break;
    i++;
    if(i>=n_path) {
      ret_val = sp_link_path();
      if(ret_val < 0) {
	free_sp();
	return(-1);
      }
    }
  }
/*** the path is found, now add a process structure ***/
  old_path = this_path;
  old_proc = this_proc;
  this_path = path[i];
  ret_val = sp_add_proc(ownpname,this_path,n_buf);
  if(ret_val < 0){
    this_path = old_path;
    this_proc = old_proc;
    free_sp();
    return(-1);
  }
/*** copy the original data to the new buffer segments ***/
  d = &(this_path->buf_info[this_proc->first]);
  d_old = &(old_path->buf_info[old_proc->first]);
  while(d_old != NULL){
    memcpy(&(this_path->buffer[d->b_first]),&(old_path->buffer[d_old->b_first])
	   ,B_SIZE);
    if(d->next == -1) d = NULL;
    else d = &(this_path->buf_info[d->next]);
    if(d_old->next == -1) d_old = NULL;
    else d_old = &(old_path->buf_info[d_old->next]);
  }
/*** clean up the original path ***/
  sp_remove_proc(old_proc,old_path);
  free_sp();
  return(0);
}

int sp_add_known_buffer(sp,p,b)
struct status_path *sp;
struct proc_entry *p;
struct data_buffer *b;
{
  struct data_buffer *d;
  short *s;

  if(b->proc != -1) return(-1);
  lock_sp();
  sp->empty_segments-=1;
  b->proc = (((int)p) - ((int)&(sp->proc_info[0])))/sizeof(struct proc_entry);
  if(p->first == -1) 
    p->first = (((int)b)-((int)&(sp->buf_info[0])))/sizeof(struct data_buffer);
  else {
    d = &(sp->buf_info[p->first]);
    while(d->next != -1) d = &(sp->buf_info[d->next]);
    d->next = (((int)b)-((int)&(sp->buf_info[0])))/sizeof(struct data_buffer);
  }
/*** empty the buffer ***/
  b->next = -1;
  s = (short *)&(sp->buffer[b->b_first]);
  *s = 0;
  free_sp();
  return(0);
}


int sp_get_proc_structure(procname,p,sp)
char *procname;
struct proc_entry **p;
struct status_path **sp;
{
  int i,i_p;

/*** no hassle for this process ***/
  if(strcmp(procname,ownpname) == 0) {
    if((this_proc == NULL) || (this_path == NULL)) return(-1);
    *p = this_proc;
    *sp = this_path;
    return(0);
  }
/**** Locate the correct process structure for other processes ***/
  *p = NULL;
  for(i=0;i<n_path;i++) {
    *sp = path[i];
    for(i_p = 0;i_p<MAX_DB_PROC;i_p++){
      if(strcmp(path[i]->proc_info[i_p].name,procname) == 0) {
	*p = &(path[i]->proc_info[i_p]);
	return(0);
      }
    }
/*** See if there is another path to add ***/
    if (i == (n_path - 1)) sp_link_existing_path();
  }
  *sp = NULL;
  return(-1);
}

sp_remove_proc(p,sp)
struct proc_entry *p;
struct status_path *sp;
{
  struct data_buffer *d,*dt;
  short *s;

  lock_sp();
  sp->empty_proc_entry +=1;
  p->name[0] = '\0';
  d = &(sp->buf_info[p->first]);
  p->first = -1;
  while (d != NULL) {
    sp->empty_segments +=1;
    d->proc = -1;
    if(d->next == -1) dt = NULL;
    else dt = &(sp->buf_info[d->next]);
    d->next = -1;
    s = (short *)&(sp->buffer[d->b_first]);
    *s = 0;
    d = dt;
  }
  free_sp();
}

sp_fill_datasegment(begin,size,type,crit,varname,value)
char *begin, crit, *varname,*value;
short size,type;
{
  int length;
  short *s;
  char *c;

  s = (short *)begin;
  *s++ = size;
  *s = type;
  c = (char *)(((int)begin) + 4);
  *c++ = crit;
  strcpy(c,varname);
  c+=(strlen(varname)+1);
  length = size + ((int)c) - ((int)begin);
  memcpy(c,value,length);
  c+=length;
}

int sp_get_datasegment(sp,p,varname,dp)
struct status_path *sp;
struct proc_entry *p;
char *varname;
struct data_pointer *dp;
{
  struct data_buffer *d;
  char *c;
  short *s;

/*** locate the datasegment ***/
  if(p->first == -1) d = NULL;
  else d = &(sp->buf_info[p->first]);
  while (d != NULL) {
    c = &(sp->buffer[d->b_first]);
    s = (short *)c;
    while (*s != 0) {
      c+=5;
      if(strcmp(c,varname) == 0) break;
      c += (((int) *s) - 5);
      s = (short *)c;
    }
    if(*s != 0) break;
    if(d->next == -1) d = NULL;
    else d = &(sp->buf_info[d->next]);
  }
  if(d == NULL) return(-1);
  sp_fill_data_pointer((char *)s,dp);
  return(0);
}

sp_fill_data_pointer(begin,dp)
char *begin;
struct data_pointer *dp;
{
  int strt;

  strt = (int)begin;
  dp->p_size = (short *)strt;
  dp->p_type = (short *)(strt+2);
  dp->p_crit = (char *)(strt+4);
  dp->p_var = (char *)(strt+5);
  dp->p_val = (char *)(strt + 5 + strlen(dp->p_var) + 1);
  dp->p_last = (char *)(strt + *(dp->p_size) - 1);
}

sp_stor_critvar(proc,sp)
struct status_path *sp;
struct proc_entry *proc;
{
  FILE *fp;
  char tmp[200],pathname[200],tpe[30];
  struct data_buffer *d;
  char *c,*val;
  short *s;
  int i_val;
  float f_val;

/*** first get the filename ***/
  sprintf(tmp,"$(USERUNIDAQ)/log/var/%s",proc->name);
  if(decode_filename(tmp,pathname) != 0) return(-1);
  if((fp = fopen(pathname,"w")) == NULL) return(-2);
#ifndef VXWORKS
  fchmod(fp,0666);
#endif
  if(proc->first == -1) d = NULL;
  else d = &(sp->buf_info[proc->first]);
  while(d != NULL){
    c = &(sp->buffer[d->b_first]);
    s = (short *)c;
    while (*s != 0) {
      if(*(c+4) == 'Y') {
	val = (c + 5 + strlen((c+5)) + 1);
	if(*(s+1) == INTEGER)      strcpy(tpe,"INT");
	else if( *(s+1) == REAL)   strcpy(tpe,"REAL");
	else if( *(s+1) == STRING) sprintf(tpe,"STRING*%d",strlen(val));
	else                       tpe[0] = '\0';
	if(tpe[0] != '\0') {
	  fprintf(fp,"%s %s ",(c+5),tpe);
	  if(*(s+1) == INTEGER) {
	    memcpy(&i_val,val,sizeof(int));
	    fprintf(fp," %d\n",i_val);
	  }
	  else if(*(s+1) == REAL) {
	    memcpy(&f_val,val,sizeof(float));
	    fprintf(fp," %g\n",f_val);
	  }
	  else
	    fprintf(fp," %s\n",val);
	}
      }
      c += ((int) *s);
      s = (short *)c;
    }
    if( d->next == -1) d = NULL;
    else d = &(sp->buf_info[d->next]);
  }
  fclose(fp);
  return(0);
}

sp_get_critval(name,val)
char *name, *val;
{
  char tmp[200],pathname[200],wrd[40],type[40],wrd2[40],*c;
  int i_val,slen;
  float f_val;
  FILE *fp;

  sprintf(tmp,"$(USERUNIDAQ)/log/var/%s",ownpname);
  if(decode_filename(tmp,pathname) != 0) return(-1);
/*** got the filename, now open it ****/
  if((fp = fopen(pathname,"r")) == NULL) return(-2);
/*** read until the correct parameter is found ***/
  while(fgets(tmp,200,fp) != NULL) {
    sscanf(tmp,"%s %s",wrd,type);
    if(strcmp(wrd,name) == 0) {
/*** found the correct parameter ****/
      if(strcmp(type,"INT") == 0) {
	sscanf(tmp,"%s %s %d",wrd,type,&i_val);
	memcpy(val,&i_val,sizeof(int));
      }
      else if (strcmp(type,"REAL") == 0) {
	sscanf(tmp,"%s %s %f",wrd,type,&f_val);
	memcpy(val,&f_val,sizeof(float));
      }
      else if (strncmp(type,"STRING",6) == 0) {
	slen = 0;
	sscanf(tmp,"%s %s %s",wrd,type,wrd2);
	sscanf(&(type[7]),"%d",&slen);
/*** the first word can be the same as the variable name ***/
	if(strcmp(wrd,wrd2) == 0) {
	  c = (char *)strstr(tmp,wrd2);
	  c++;
	}
/*** the first word can be the same as the variable type ***/
	if(strcmp(type,wrd2) == 0) {
	  c = (char *)strstr(tmp,wrd2);
	  c++;
	}
	c = (char *)strstr(tmp,wrd2);
/*** set the value in the SP, and check if we have it all ***/
	if (slen > 0) {
	  strcpy(val,c);
	}
	else {
	  val[0] = '\0';
	}
	while(strlen(val) < slen) {
	  if(fgets(tmp,200,fp) == NULL) break;
	  strcat(val,tmp);
	}
	val[slen] = '\0';
      }
      else {
	fclose(fp);
	return(-3);
      }
      fclose(fp);
      return(0);
    }
  }
  fclose(fp);
  return(-4);
}


int lock_sp()
{
/*** make sure the process is not waiting for itself ***/
  n_lock_requests +=1;
  if(n_lock_requests > 1) return(0);
  Update_Sem_Wait(path[0]->sp_lock,SP_LOCK);
  return(0);
}

int free_sp()
{
/*** make sure the process has not a pending request ***/
  n_lock_requests -=1;
  if(n_lock_requests > 0) return(0);
  else n_lock_requests = 0;
/*** really unlock the status path ***/
  Update_Sem_Wait(path[0]->sp_lock,SP_FREE);
  return(0);
}

delete_sp_lock()
{
  Delete_Sem(path[0]->sp_lock);
}
