/*************************************************
 * CVS Log
 * mur_def.h,v
 * Revision 1.1.1.1  1993/07/14  19:43:19  dbrenner
 * UNIDAQ 2.1 distribution - July 14,1993
 *
*************************************************/
#ifndef MUR_DEF_HEADER
#define MUR_DEF_HEADER

/*==========================================================================
 * murmur client definitions mur_def.h
 * 
 * modifications:
 *
 *	OCT 18, 92	GAO	Reorder internal errors. The SRVRADD and 
 *				COM errors are lowest numbered errors
 *
 *	NOV  5, 92 	CLM	Removed definition of MUR_SUCCESS for
 *			 	MUR_VMS flavor. 
 *
 *==========================================================================
*/

#include "mur_user.h"  /* Include argument type definitions */

/*===========================================================================
**===========================================================================
**
** 		BASIC TYPE DEFINITIONS AND CONSTANTS
*/
#define MUR_CURRENT_VERSION "MUR V1.0"
#define MUR_SERVER_PORT	    50008		/* Port number for server */
#define MUR_SADD_NAME	    "MURMUR_SERVER_ADD" /* Internet address name 
                                                   (logical name for VMS or
                                                   environmental variable for
                                                   UNIX) */
#ifdef MUR_VXWORKS
#define MUR_ISP_MAXMSGS	    30	    /* Max # messages in Vxworks msg queue */
#endif

#ifdef MUR_VMS
#endif

#define MUR_QUEUE_DEPTH	    5

/* NOTE THAT THIS VALUE (MUR_INIT_SEND_SIZE) MUST NEVER BE CHANGED IN ORDER TO 
   ENSURE THE MURMUR SERVER IS BACKWARDS COMPATIBLE */
#define MUR_INIT_SEND_SIZE  450

/* NOTE THAT THIS VALUE (MUR_SEND_SIZE) SHOULD NOT BE CHANGED.  IF SO,
   CLIENT AND SERVER CODE MUST BE MODIFIED ACCORDINGLY */
#define MUR_SEND_SIZE	    MUR_INIT_SEND_SIZE

#ifdef MUR_VMS
#define MUR_TZ_LOGICAL	    "MULTINET_TIMEZONE"
#endif
 
typedef unsigned long   MUR_MSG_STATUS; /* MURMUR message status */
typedef MUR_MSG_STATUS  MUR_MSG_CODE;   /* MURMUR message code */
typedef MUR_MSG_STATUS  MUR_CLIENT_ERR; /* MURMUR client error status */
typedef unsigned char	MUR_BOOL;	/* MURMUR's own boolean true/false */

#define MUR_SADD_LEN	15		/* Length of server Internet address */
#define MUR_SADD_SLEN	MUR_SADD_LEN+1	/* Actual string length of ...*/
typedef char		MUR_SADD[MUR_SADD_SLEN];  /* Server Internet address */
typedef char		*MUR_SADD_PTR;

/*===========================================================================
**===========================================================================
**
** 		MURMUR MACROS
*/
#if defined(MUR_UNIX) || defined(MUR_VMS)
#define MUR_GETPID		(int) getpid()
#define MUR_CLEAR_CTX(a)	memset((a), 0, sizeof(MUR_CLIENT_CTX))
#define MUR_STDERR		fprintf(stderr,
#define MUR_GETHOSTNAME(a) \
	{\
	int i;\
	gethostname((char *)(a), MUR_NODE_LEN);\
	(a)[MUR_NODE_LEN]=0;\
	i=(int)strcspn((char *)(a), ".");\
        if ((i>0) && (i <= MUR_NODE_LEN)) {(a)[i]=0;}\
        }
#endif

#ifdef MUR_UNIX
#define MUR_ASCIIPID(a, b)	sprintf((a), "%d", (b))
#endif

#ifdef MUR_VMS
#define MUR_ASCIIPID(a, b)	sprintf((a), "%x", (b))
#endif

#ifdef MUR_VXWORKS
#define MUR_GETPID		(int) taskIdSelf()
#define MUR_STDERR		printErr(
#define MUR_ASCIIPID(a, b)	sprintf((a), "%d", (b))
#define MUR_CLEAR_CTX(a)   \
        {\
        int i;\
        for (i=0; i < sizeof(MUR_CLIENT_CTX); i++) {((char *)(a))[i] = 0;}\
        }
#define MUR_GETHOSTNAME(a)	strcpy((char *)(a), "VxWorks")
#endif


/*===========================================================================
**===========================================================================
**
** 		USER RECORD HEADER TYPE DEFINITIONS AND CONSTANTS
**
** The User Record Header contains the information sent to the MURMUR server
** upon initial contact from a client.  This header is sent only once (to the
** server) during the lifetime of a client process.  The header resides within
** the client context structure and consists of the following data items:
**
**	Message type (MSG_INIT) : enumeration (MUR_MSG_TYPE_ENUM)
**	Version number		: ASCII string
**	Copy of process ID	: ASCII string
**	Host node name		: ASCII string
**	Time zone		: ASCII string
**	User record length	: unsigned long
**
*/
#define MUR_VERS_LEN	11		/* Length of version number */
#define MUR_PID_LEN	11		/* Length of process ID */
#define MUR_NODE_LEN    11		/* Length of node name */
#define MUR_TZ_LEN	7		/* Length of time zone */

#define MUR_VERS_SLEN	MUR_VERS_LEN+1	/* Actual string length of ...*/
#define MUR_PID_SLEN	MUR_PID_LEN+1	/* Actual string length of ...*/
#define MUR_NODE_SLEN	MUR_NODE_LEN+1	/* Actual string length of ...*/
#define MUR_TZ_SLEN	MUR_TZ_LEN+1	/* Actual string length of ...*/

/*
** USER RECORD HEADER LENGTH AND TYPEDEF 
*/
/* The length is set as follows:

      (message type longword) + (version string)+(4 XDR bytes) + 
      (pid string)+(4 XDR bytes) + (node string)+(4 XDR bytes) + 
      (time zone string)+(4 XDR bytes) + (record length longword) + 
      (32 bytes padding) 

   NOTE THAT THIS VALUE MUST NEVER EXCEED MUR_INIT_SEND_SIZE IN ORDER TO 
   ENSURE THE MURMUR SERVER IS BACKWARDS COMPATIBLE
*/
#define	MUR_USER_HDR_LEN (4 + MUR_VERS_SLEN+4 + MUR_PID_SLEN+4 + \
                          MUR_NODE_SLEN+4 + MUR_TZ_SLEN+4 + 4 + 32)

typedef char	MUR_USER_REC_HDR[MUR_USER_HDR_LEN];
typedef char	*MUR_USERRECHDR_PTR;

typedef char 	MUR_VERSION[MUR_VERS_SLEN];  /* MURMUR version # */
typedef char    *MUR_VERSION_PTR;
typedef char	MUR_PID[MUR_PID_SLEN];	     /* ASCII Process ID */
typedef char    *MUR_PID_PTR;
typedef char	MUR_NODE[MUR_NODE_SLEN];     /* Host node name */
typedef char    *MUR_NODE_PTR;
typedef char	MUR_TZ[MUR_TZ_SLEN];	     /* Time zone */
typedef char    *MUR_TZ_PTR;


/*===========================================================================
**===========================================================================
**
** 		MURMUR USER RECORD TYPE DEFINITIONS AND CONSTANTS
** 
** The User Record contains the information sent to the MURMUR server for each
** message reported.  Included in the record are a variable number of message
** arguments (up to 8 maximum) and possibly an appended text string.  Of the 8
** allowed arguments, up to 4 of them can be character strings (limited to 30
** characters apiece).  The record resides within the client context structure
** and consists of the following data items:
**
**	Message type		: enumeration (MUR_MSG_TYPE_ENUM)
**	Message code		: unsigned long
**	Client time		: ASCII string
**	Client date		: ASCII string
**	User argument flags	: 2 unsigned longs (1 byte for each argument)
**	Extra argument flags	: 1 unsigned long  (1 byte for appended text,
**                                                  1 for prefix, 2 unused)
**	User arguments		: (long, float, string, etc.)
**	Extra arguments		: (process name prefix and/or appended text)
**
*/
#define MUR_NUM_STR_ARGS    4	/* Number of string arguments allowed */
#define MUR_NUM_ARGS	    8	/* Number of message arguments allowed */
#define MUR_NUM_APPEND	    1   /* Number of appended text strings allowed */

#define MUR_CTIME_LEN	    15	/* Length of client time */
#define MUR_CDATE_LEN	    15	/* Length of client date */
#define MUR_PREFIX_LEN	    15	/* Length of process name prefix */
#define MUR_APP_TEXT_LEN    132	/* Length of appended text string */
#define MUR_ARG_STRING_LEN  30	/* Length of argument strings */
#define MUR_TEXT_ONLY_LEN   MUR_APP_TEXT_LEN    /* Length of text only string */
/* NOTE: if MUR_TEXT_ONLY_LEN is changed, be sure MUR_USER_REC_LEN is 
   big enough */

#define MUR_CTIME_SLEN	    MUR_CTIME_LEN+1	/* Actual string length of ...*/
#define MUR_CDATE_SLEN	    MUR_CDATE_LEN+1	/* Actual string length of ...*/
#define MUR_PREFIX_SLEN	    MUR_PREFIX_LEN+1	/* Actual string length of ...*/
#define MUR_APP_TEXT_SLEN   MUR_APP_TEXT_LEN+1  /* Actual string length of ...*/
#define MUR_TEXT_ONLY_SLEN  MUR_TEXT_ONLY_LEN+1 /* Actual string length of ...*/
#define MUR_ARG_STRING_SLEN MUR_ARG_STRING_LEN+1 

typedef char 	MUR_CTIME[MUR_CTIME_SLEN];   /* Client time */
typedef char    *MUR_CTIME_PTR;
typedef char 	MUR_CDATE[MUR_CDATE_SLEN];   /* Client date */
typedef char    *MUR_CDATE_PTR;
typedef char	MUR_PREFIX[MUR_PREFIX_SLEN]; /* Message prefix */
typedef char    *MUR_PREFIX_PTR;
typedef char	MUR_APP_TEXT[MUR_APP_TEXT_SLEN];     /* Appended text string */
typedef char    *MUR_APPTEXT_PTR;
typedef char	MUR_ARG_STRING[MUR_ARG_STRING_SLEN]; /* Argument string */
typedef char    *MUR_ARGSTRING_PTR;
typedef char	MUR_TEXT_ONLY[MUR_TEXT_ONLY_SLEN];   /* "Text only" string */
typedef char    *MUR_TEXTONLY_PTR;

/* THE FOLLOWING UNIONS ARE USED TO CONTAIN THE LIST OF USER ARGUMENTS */

typedef union
   {
   unsigned char    c[MUR_NUM_ARGS];
   unsigned long    l[MUR_NUM_ARGS/4];
   } MUR_ARG_FLAGS;		    /* Message argument flags */
typedef union
   {
   MUR_BOOL	    c[sizeof(unsigned long)];
   unsigned long    l;
   } MUR_EXTRA_FLAGS;		    /* Extra argument flags */

#define	    MUR_APP_INDEX	0   /* Appended text index in MUR_EXTRA_FLAGS */
#define	    MUR_PREFIX_INDEX	1   /* Prefix index in MUR_EXTRA_FLAGS */

/*
** USER RECORD LENGTH AND TYPEDEF 
*/
/* The length is set as follows (based on the "worst case" number of arguments):

      (message type longword) + (message code longword) +
      (client time string)+(4 XDR bytes) + (client date string)+(4 XDR bytes) + 
      (argument flag string)+(4 XDR bytes) + (extra flag string)+(4 XDR bytes) +
      ((argument string)+(4 XDR bytes))*4 + (floating point argument)*4 +
      (prefix string)+(4 XDR bytes) + (appended text string)+(4 XDR bytes) + 
      (53 bytes padding) = 450

   NOTE THAT THIS VALUE SHOULD NOT BE CHANGED.  IF SO, CLIENT AND SERVER CODE
   MUST BE MODIFIED ACCORDINGLY 
*/
#define	MUR_USER_REC_LEN (4 + 4 + MUR_CTIME_SLEN+4 + MUR_CDATE_SLEN+4 + \
                          MUR_NUM_ARGS+4 + 4+4 + \
                          (MUR_ARG_STRING_SLEN+4)*MUR_NUM_STR_ARGS + 32 + \
                          MUR_PREFIX_SLEN+4 + MUR_APP_TEXT_SLEN+4 + 53)

typedef char	MUR_USER_REC[MUR_USER_REC_LEN];
typedef char    *MUR_USERREC_PTR;

/*===========================================================================
**===========================================================================
**
**	    MUR_CTX_STATE_ENUM - MURMUR CLIENT CONTEXT STATES (ENUMERATED)
** 
** This enumeration identifies the states of the client context.
*/
typedef enum
   {
   CTX_NOT=0,	    /* CTX not used - not initialized (must equal 0) */
   CTX_NEW_LINK,    /* CTX has been initialized, but needs to have a new
                       communication link established */
   CTX_LINKED,	    /* Communication link with server has been established */
   CTX_LOCAL	    /* Messages to be reported locally - no link with server */
   } MUR_CTX_STATE_ENUM;


/*===========================================================================
**===========================================================================
**
**	    MUR_MSG_TYPE_ENUM - MURMUR MESSAGE TYPE (ENUMERATED)
** 
** This enumeration identifies the message types recognized by the MURMUR
** server.
*/
typedef enum
   {
   MSG_INIT,	/* Initialization message */
   MSG_NORMAL,	/* Normal message */
   MSG_STACK,	/* Message stack */
   MSG_RETURN,	/* Return message text */
   MSG_SDIAG,    /* Server diagnostic message */   
   MSG_SKILL	/* Kill off the server via its signal 16 handler */
   } MUR_MSG_TYPE_ENUM;


/*===========================================================================
**===========================================================================
**
**	    MUR_PREFIX_ENUM - MURMUR PROCESS NAME STATE (ENUMERATED)
** 
** This enumeration identifies the state of the process name field in the 
** client context.
*/
typedef enum
   {
   PFX_NOT=0,	/* Prefix is not defined */
   PFX_SEND,	/* Prefix is defined, but needs to be sent to server */
   PFX_DEFINED	/* Prefix is defined */
   } MUR_PREFIX_ENUM;


/*===========================================================================
**===========================================================================
**
** 		MUR_CLIENT_CTX - MURMUR CLIENT CONTEXT
** 
** This structure contains the context information for each client process 
** (i.e., user application) connected to the MURMUR server.  There is only
** one such structure per client process.  When a MURMUR routine is invoked
** for the first time, the structure is initialized.  Subsequent calls will
** retrieve and update the context.  Information maintained includes both 
** client state and communication data (this allows communication links to
** be established only once - instead of reconnecting every time a MURMUR
** routine is called).
*/
typedef struct mur_client_ctx
   {
   MUR_CTX_STATE_ENUM	    cstate;	/* Current state of context */
   unsigned long	    cstatus;	/* Status of operations */
   long int		    sock;	/* Communication socket */
   MUR_MSG_STATUS	    msg_code;	/* Message being reported */
   int			    pid;	/* Current process ID */
   MUR_PREFIX		    prefix;	/* Process name prefix */
   MUR_PREFIX_ENUM	    new_prefix; /* prefix enumeration */
   XDR			    xdrs;	/* XDR structure (for byte swapping) */
   MUR_USER_REC_HDR	    user_hdr;	/* User record header */
   MUR_USER_REC		    user_rec;	/* User record */
   } MUR_CLIENT_CTX;

#define MUR_CTX_INIT {CTX_NOT}


/*===========================================================================
**===========================================================================
**
** 		MUR_ARG_STRUCT - MURMUR MESSAGE ARGUMENT UNION STRUCTURE
** 
** This is a structur/union of the allowed argument types
*/
typedef struct mur_arg_struct
{
union
   {
   void           *generic;
   long           longy;
   unsigned long  ulongy;
   float          floater;
   char           *stringyptr;
   } un;
MUR_ARG_STRING stringy;
} MUR_ARG_STRUCT;


/*===========================================================================
**===========================================================================
**
** 		MURMUR CLIENT INTERNAL ERROR DEFINITIONS
**
**	NOTE: communications errors (SRVRADD and COM errors) are the lowest
**		numbered errors (1-5). This fact is used in the code to 
**	        determine if a switchto local mode has occured, so be 
**		carefule when changing these codes!!!
*/
#ifdef MUR_INTERNAL_ERROR_ROUTINE
char *mur_cerr[] =
   {
   "",						/* 0  - No message */
   "Server address not specified (MURMUR_SERVER_ADD)",  /* 1-MUR_CERR_SRVRADD */
   "Can't ALLOCATE socket",			/* 2  - MUR_CERR_COM_ALLOC */
   "Can't CONNECT to MURMUR server",		/* 3  - MUR_CERR_COM_CNCT */
   "Can't SEND to MURMUR server - IO error",	/* 4  - MUR_CERR_COM_SEND */
   "Can't SEND header to MURMUR server - IO error",     /* 5-MUR_CERR_COM_HDR */
   "Can't create client context",		/* 6  - MUR_CERR_CREATE_CTX */
   "Can't allocate client context",		/* 7  - MUR_CERR_ALLOC_CTX */
   "Too many string arguments passed",		/* 8  - MUR_CERR_STR_ARGS */
   "Too many arguments passed",			/* 9  - MUR_CERR_ARGS */
   "Only one appended text string allowed",	/* 10  - MUR_CERR_APP_ARGS */
   "No other arguments allowed with text-only message", /* 11-MUR_CERR_TEXT_ONLY */
   "Header packing failure using XDR",		/* 12 - MUR_CERR_HDR_PACK */
   "Record packing failure using XDR",		/* 13 - MUR_CERR_REC_PACK */
   "Argument packing failure using XDR",	/* 14 - MUR_CERR_ARG_PACK */
   "Invalid argument type was passed",		/* 15 - MUR_CERR_INV_ARG */
   "Invalid message type",			/* 16 - MUR_CERR_INV_MSGTYPE */

   "Cannot create VxWorks message queue",	/* 17 - MUR_CERR_NO_MSGQ */
   "VxWorks message queue I/O error",		/* 18 - MUR_CERR_MSGQ_IO */
   "Can't translate VMS error code"		/* 19-MUR_CERR_CANT_TRANSLATE */
   };
#endif
#define MUR_CERR_SRVRADD	1
#define MUR_CERR_COM_ALLOC	2
#define MUR_CERR_COM_CNCT	3 
#define MUR_CERR_COM_SEND	4 
#define MUR_CERR_COM_HDR	5 
#define MUR_MAX_COM_ERROR	5	/* all communications errors <= this */
#define	MUR_CERR_CREATE_CTX	6
#define	MUR_CERR_ALLOC_CTX	7
#define MUR_CERR_STR_ARGS	8
#define MUR_CERR_ARGS		9
#define MUR_CERR_APP_ARGS	10
#define MUR_CERR_TEXT_ONLY	11
#define	MUR_CERR_HDR_PACK	12
#define	MUR_CERR_REC_PACK	13
#define	MUR_CERR_ARG_PACK	14
#define MUR_CERR_INV_ARG	15
#define MUR_CERR_INV_MSGTYPE	16
#define MUR_CERR_NO_MSGQ	17
#define MUR_CERR_MSGQ_IO	18
#define MUR_CERR_CANT_TRANSLATE 19
#endif
