/*************************************************
 * CVS Log
 * mur_c_util.c,v
 * Revision 1.1.1.1  1993/07/14  19:43:22  dbrenner
 * UNIDAQ 2.1 distribution - July 14,1993
 *
*************************************************/
/* ==========================================================================
**
** MUR_C_UTIL.C
**
** DESCRIPTION
**	This file contains various utility "C" routines used within the 
**	MURMUR client library.  Included are the routines used for
**	communications between the client (user application) and the MURMUR
**	server.
**
** DEVELOPERS
**	Gary P. Sergey
**
**	Fermilab Data Acquisition Group
**	Batavia, Il 60510, U.S.A.
**
** MODIFICATIONS
**         Date       Initials	Description
**	18-Feb-1992	GPS	Original Entry
**	12-Sep-1992	GAO	Add support for server address specification in
**				doamin name format in additonm to dot format
**				(Unix & VMS - no name resolver under VxWorks?)
** 
** ENTRY POINTS			 SCOPE
**	mur_internal_err	external
**	mur_timezone		external
**	mur_get_ctx		external
**	mur_com_init		internal
**	mur_com_send		external
**	mur_com_cleanup		external
**	mur_get_srvr_add	external
**	mur_client_init		external
*/
#include "mur_cond.h"



#ifndef MUR_VXWORKS
#include <signal.h>
#include <time.h>
#include <stdio.h>
#endif

#ifdef MUR_UNIX
#include <stdlib.h>	    /* Needed for getenv */
#endif

#ifdef MUR_VMS
#include <DESCRIP.H> /* Define descriptors                 */
#include <LNMDEF.H>  /* Define Logical Name values         */
#include <PSLDEF.H>  /* Define Processor Status Longword   */
#include <SSDEF.H>   /* Define System Statuses             */
#include "MULTINET_COMMON_ROOT:[multinet.include.vms]inetiodef.h"
#include "MULTINET_COMMON_ROOT:[multinet.include.sys]types.h"
/* #include "MULTINET_COMMON_ROOT:[multinet.include.sys]time.h" */
#include "MULTINET_COMMON_ROOT:[multinet.include.netinet]in.h"
#include "MULTINET_COMMON_ROOT:[multinet.include.sys]socket.h"
#include "MULTINET_COMMON_ROOT:[multinet.include]errno.h"
#include "MULTINET_COMMON_ROOT:[multinet.include.rpc]rpc.h"
#include "MULTINET_COMMON_ROOT:[multinet.include.rpc]xdr.h"
#include "MULTINET_COMMON_ROOT:[multinet.include]netdb.h"
#endif

#ifdef MUR_SGI
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <stdio.h>	/* Need for SGI (__file_s declaration) */
#include <rpc/types.h>
#include <rpc/xdr.h>
#include <netdb.h>
#define socket_close close
#endif


#ifdef MUR_ULTRIX
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <rpc/rpc.h>
#include <netdb.h>
#define socket_close close
#endif

#ifdef _AIX
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <rpc/rpc.h>
#include <netdb.h>
#define socket_close close
#endif

#ifdef MUR_SUN
#include <rpc/types.h>
#include <rpc/xdr.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#define socket_close close
#endif

#ifdef MUR_VXWORKS
#include <vxWorks.h>
#include <rpc/rpctypes.h>
#include <rpc/xdr.h>
#include <in.h>
#include <socket.h>
#define socket_close close
#endif

#ifdef MUR_VMS
#define  VMS_CAST short     /* Special casting required for VMS systems */
#else
#define  VMS_CAST           /* No casting for non-VMS systems */
#endif

#include "mur_macros.h"
#include "murmur_msg_c.h"

#define MUR_INTERNAL_ERROR_ROUTINE
#include "mur_def.h"
#include "mur_func.h"
#undef  MUR_INTERNAL_ERROR_ROUTINE

/* ==========================================================================
**
** GLOBAL CLIENT CONTEXT STRUCTURE REFERENCE
*/
#ifdef MUR_VXWORKS
extern MUR_CLIENT_CTX *global_cctx_ptr;
#else
extern MUR_CLIENT_CTX global_cctx;
#endif

#ifdef MUR_VXWORKS
/* ==========================================================================
**
** VXWORKS GLOBAL SERVER ADDRESS STRING
*/
extern MUR_SADD murmur_server_add;
#endif


/* ==========================================================================
**
** ROUTINE: mur_internal_err
*/
#ifdef MUR_STDC
void mur_internal_err
(
MUR_CLIENT_ERR	a_cerr,		    /* IN :  Internal client error number */
MUR_MSG_STATUS	a_msgcode,	    /* IN :  Message being reported */
MUR_BOOL	a_printerrno	    /* IN :  Print errno if true */
)
#else
void mur_internal_err(a_cerr, a_msgcode, a_printerrno)

MUR_CLIENT_ERR	a_cerr;		    /* IN :  Internal client error number */
MUR_MSG_STATUS	a_msgcode;	    /* IN :  Message being reported */
MUR_BOOL	a_printerrno;	    /* IN :  Print errno if true */
#endif
/*
** DESCRIPTION
** 	Used to report errors internal to MURMUR client library routines.
** 
** VALUES RETURNED
**	N/A
** 
** CALLED BY
**      Various MURMUR routines
**
** CALLS TO
**	fprintf
**
** ========================================================================== 
*/
{ /* begin: mur_internal_err */

#ifdef  MUR_VMS
#define MUR_PERR fprintf(stderr, "   Errno message : <%s>\n", vms_errno_string())
#endif

#ifdef  MUR_UNIX
extern  int  errno;		
extern  char *sys_errlist[];
#define MUR_PERR fprintf(stderr, "   Errno message : <%s>\n", (char *)sys_errlist[errno])
#endif

#ifdef  MUR_VXWORKS
#define MUR_PERR printErrno((int)errnoGet())
#endif

MUR_STDERR "\n-- MURMUR CLIENT STATUS : \"%s\" --\n", mur_cerr[a_cerr]);
if (a_msgcode)
   {
   MUR_STDERR "   Occurred when trying to report message : <%x>\n",
          a_msgcode);
   }
if (a_printerrno)
   {
   MUR_PERR;
   }

} /* end: mur_internal_err */




/* ==========================================================================
**
** ROUTINE: mur_getiaddr
*/
#ifdef MUR_STDC
unsigned long mur_getiaddr
(
char 	*addrstr	    /* address string */
)
#else
unsigned long mur_getiaddr(addrstr)
char 	*addrstr;
#endif
/*
** DESCRIPTION
** 	Given an internet address string (dot or domain-name form), returns
** 	the 32 bit integer Internet address, or -1 (0xFFFFFFFF) on failure
**	Can only get dot notation under VxWorks, both for Unix and VMS.
** 
** VALUES RETURNED
**	32 bit Internet address on success
**	0xFFFFFFFF on failure
**
** 
** CALLED BY
**	
**
** CALLS TO
**
**
** ========================================================================== 
*/

{
unsigned long iaddr;

#ifndef MUR_VXWORKS
struct hostent *hoste;
#endif


/* first try dot notation */

if( (iaddr=inet_addr(addrstr))==0xFFFFFFFF)
    {
#ifndef MUR_VXWORKS
    /* then try name lookup if this failed */
    hoste = gethostbyname(addrstr);
    if(hoste)iaddr =  *( (unsigned long *) hoste->h_addr_list[0]);
#else
    iaddr = hostGetByName(addrstr);
#endif
    }

return(iaddr);
}





/* ==========================================================================
**
** ROUTINE: mur_timezone
*/
#ifdef MUR_STDC
void mur_timezone
(
MUR_TZ_PTR	a_zone		   /* OUT:  Time zone in ASCII */
)
#else
void mur_timezone(a_zone)

MUR_TZ_PTR	a_zone; 	   /* OUT:  Time zone in ASCII */
#endif
/*
** DESCRIPTION
** 	Used to retrieve the current time zone in ASCII.
** 
** VALUES RETURNED
**	N/A
** 
** CALLED BY
**
** CALLS TO
**
** ========================================================================== 
*/
{ /* begin: mur_timezone */

/*---------------------------------------------------------------------------
**
** GET TIMEZONE UNDER UNIX
*/
#ifdef MUR_UNIX
int	    dummy;
time_t	    ltime;
struct tm   *ltmptr;

/* 
** GET TIME ZONE
*/
ltime  = time(0);
if (ltime == (time_t) -1)
   {
   strcpy((char *)a_zone, "---");  /* Since error, set default */
   goto mur_err_exit;
   }

/*
** CONVERT TO "TM" STRUCTURE FORMAT 
*/
ltmptr = localtime(&ltime);

/*
** CONVERT TO DESIRED ASCII STRING
*/
dummy = strftime((char *)a_zone, MUR_TZ_LEN, "%Z", ltmptr);
#endif

/*---------------------------------------------------------------------------
**
** GET TIMEZONE UNDER VMS
*/
#ifdef MUR_VMS
char			tzone[] = MUR_TZ_LOGICAL;
static unsigned long	attr   = LNM$M_CASE_BLIND;	/* TRNLNM attributes  */
static unsigned char	acmode = PSL$C_USER;		/* TRNLNM access mode */
static $DESCRIPTOR	(tabnam, "LNM$FILE_DEV");	/* TRNLNM table name  */

unsigned long int	status;
unsigned long int	i = 0;

struct dsc$descriptor lognam = {0, DSC$K_DTYPE_T, DSC$K_CLASS_S, 0};
struct
   {
   unsigned short	buflen;	    /* Buffer length (bytes) */
   unsigned short	item;	    /* Nature of our request */
            char	*bufadr;    /* Buffer address        */
            short	*retlen;    /* Return length address */
   }

geteqv []= {{MUR_TZ_LEN,  LNM$_STRING, a_zone,  &i}, {0, 0, 0, 0}};

lognam.dsc$w_length  = strlen(tzone);
lognam.dsc$a_pointer =        tzone;

if ((status = sys$trnlnm (&attr, &tabnam, &lognam, &acmode, geteqv)) == SS$_NORMAL)
   {
   ((char *)a_zone)[i] = 0; /* Null-terminate the string */
   }
else
   {
   strcpy((char *)a_zone, "---");
   }
#endif

/*---------------------------------------------------------------------------
**
** THERE IS NO TIMEZONE UNDER VXWORKS
*/
#ifdef MUR_VXWORKS
strcpy((char *)a_zone, "---");
#endif

mur_err_exit:
return;

} /* end: mur_timezone */


/* ==========================================================================
**
** ROUTINE: mur_get_ctx
*/
#ifdef MUR_STDC
MUR_CLIENT_ERR mur_get_ctx
(
MUR_CLIENT_CTX **a_ptr_ctxptr,      /* OUT:  Client context pointer */
MUR_MSG_STATUS a_msgcode	    /* IN :  Message being reported */
)
#else
MUR_CLIENT_ERR mur_get_ctx(a_ptr_ctxptr, a_msgcode)

MUR_CLIENT_CTX **a_ptr_ctxptr;      /* OUT:  Client context pointer */
MUR_MSG_STATUS a_msgcode;	    /* IN :  Message being reported */
#endif
/*
** DESCRIPTION
** 	Retrieves the client context for the current user process (or task).
** 	If this is a new process (no prior calls to MURMUR routines) or
**	a recently forked process, the context will be initialized.  If this
**	is an existing process (previous MURMUR calls), the context pointer
**	is just returned.  Under UNIX or VMS, the context is simply a global
**	(static) structure.  Under VxWorks, task variables (via taskVarLib) 
**	are utilized.
** 
** VALUES RETURNED
**	MUR_SUCCESS
**	MUR_CERR_CREATE_CTX
**	MUR_CERR_ALLOC_CTX
** 
** GLOBALS REFERENCED 
**	global_cctx   - Global client context structure. 
**
** CALLED BY
**      mur_send
**
** CALLS TO
**	malloc		    (VxWorks only)
**	taskVarAdd	    (VxWorks only)
**	taskVarGet	    (VxWorks only)
**
** ========================================================================== 
*/
{ /* begin: mur_get_ctx */

MUR_CLIENT_ERR	err_status;	/* MURMUR client internal error status */
MUR_CLIENT_CTX  *lcl_ctxptr;
int		cpid;

/*---------------------------------------------------------------------------
**
** GET POINTER TO CONTEXT STRUCTURE
*/

/*
** FOR UNIX OR VMS, USE GLOBAL CLIENT CONTEXT STRUCTURE 
*/
#ifndef MUR_VXWORKS

lcl_ctxptr = (MUR_CLIENT_CTX *)&global_cctx;

/*
** FOR VXWORKS, SEE IF A TASK VARIABLE EXISTS - IF NOT, CREATE ONE 
*/
#else

/* Call taskVarInit to initialize task variable facility.  After the first 
   call to this routine, subsequent calls have no effect. */
if (taskVarInit() != OK)
   {
   err_status = MUR_CERR_CREATE_CTX;	/* Somethings scrogged up */
   goto mur_err_exit;			/* Blow this clip joint */
   }

/* If a task variable doesn't exist... */
if (taskVarGet(0, (int *)&global_cctx_ptr) == ERROR)
   {
   if (taskVarAdd(0, (int *)&global_cctx_ptr) != OK) /* No, try to create one */
      {
      err_status = MUR_CERR_CREATE_CTX;	    /* Uh oh, error occurred */
      goto mur_err_exit;		    /* Get your ass out of here */
      }

   /* Malloc space for context */
   global_cctx_ptr = (MUR_CLIENT_CTX *)malloc(sizeof(MUR_CLIENT_CTX));
   lcl_ctxptr = (MUR_CLIENT_CTX *)global_cctx_ptr;

   if (lcl_ctxptr == NULL)
      {
      err_status = MUR_CERR_ALLOC_CTX;	    /* Oh no, malloc failure */
      goto mur_err_exit;		    /* Time to give up */
      }
   lcl_ctxptr->cstate = CTX_NOT;            /* Indicate context has not
                                               been initialized */
   }

/* ...Otherwise there already is a task variable, so get context pointer back */
else
   {
   lcl_ctxptr = (MUR_CLIENT_CTX *)global_cctx_ptr;
   }
#endif

/*
** SAVE CONTEXT POINTER IN PASSED ARGUMENT
*/

*a_ptr_ctxptr = (MUR_CLIENT_CTX *)lcl_ctxptr;


/*---------------------------------------------------------------------------
**
** INITIALIZE CONTEXT STRUCTURE IF THIS IS A NEW PROCESS
*/

/*
** FOR VMS AND VXWORKS, INITIALIZE IF CONTEXT STATE IS UN-INITIALIZED (ZERO) 
*/
#if defined(VAXC) || defined(MUR_VXWORKS)

if (lcl_ctxptr->cstate == CTX_NOT)
   {
   cpid = MUR_GETPID;  /* Get current process ID */

/*
** FOR UNIX, ALSO INITIALIZE IF THIS IS A FORKED (CHILD) PROCESS 
*/
#else

cpid = MUR_GETPID;  /* Get current process ID */

/* If the current process ID did not match what was in the existing client
   context, then we must have forked.  So re-initialize the context - the
   child must not inherit the parents context. */

if ((lcl_ctxptr->cstate == CTX_NOT) ||
    (lcl_ctxptr->pid != cpid))
   {
#endif

   MUR_CLEAR_CTX(lcl_ctxptr); 		 /* First, clear out context */
   lcl_ctxptr->msg_code = a_msgcode;	 /* Save message being reported - NOTE:
                                            only saved for debug purposes */
   lcl_ctxptr->pid	= cpid;          /* Save current Process ID */
   lcl_ctxptr->cstate	= CTX_NEW_LINK;  /* Context has been initialized */
   }  

/*---------------------------------------------------------------------------
**
** RETURN FROM MUR_GET_CTX
*/

/*
** NORMAL SUCCESSFUL COMPLETION 
*/
return(MUR_SUCCESS);

/*
** ERROR EXIT 
*/
mur_err_exit:
return(err_status);

} /* end: mur_get_ctx */


/* ==========================================================================
**
** ROUTINE: mur_com_init
*/
#ifdef MUR_STDC
MUR_CLIENT_ERR mur_com_init
(
MUR_CLIENT_CTX  *a_ctxptr,	/* MOD:  Client context */
char		*a_destaddr,    /* IN :  MURMUR server Internet address */
int		a_port 		/* IN :  MURMUR server port number */
)
#else
MUR_CLIENT_ERR mur_com_init(a_ctxptr, a_destaddr, a_port)

MUR_CLIENT_CTX  *a_ctxptr;	/* MOD:  Client context */
char		*a_destaddr;    /* IN :  MURMUR server Internet address */
int		a_port;		/* IN :  MURMUR server port number */
#endif
/*
** DESCRIPTION
** 	Initializes socket communications with the MURMUR server.
** 
** VALUES RETURNED
**	MUR_SUCCESS
**	MUR_CERR_COM_ALLOC
**	MUR_CERR_COM_CNCT
** 
** CALLED BY
**      mur_send
**
** CALLS TO
**	connect
**	socket
**
** ========================================================================== 
*/
{ /* begin: mur_com_init */

MUR_CLIENT_ERR	    err_status;	 /* MURMUR client internal error status */
struct sockaddr_in  sin;
long int	    sock;


#ifndef MURDER_LOCAL
/*
** SETUP SOCKET STRUCTURE 
*/

sin.sin_family	    = AF_INET;            /* address family = INTERNET */
sin.sin_port	    = htons(a_port);      /* service port ID */
sin.sin_addr.s_addr = mur_getiaddr(a_destaddr);

/*
** ALLOCATE A SOCKET 
*/

sock = (long int) socket(sin.sin_family, SOCK_STREAM, 0);
if (sock < 0) {err_status = MUR_CERR_COM_ALLOC; goto mur_err_exit;}
a_ctxptr->sock = sock;	    /* Save socket in client context */

/*
** ESTABLISH A CONNECTION WITH THE SERVER 
*/

if (connect((VMS_CAST int)sock, &sin, sizeof(sin)))
   {err_status = MUR_CERR_COM_CNCT; goto mur_err_exit;}

#endif /* MURDER_LOCAL */

/*
** ALL DONE, GOODBYE 
*/

/* For successful return, indicate communication link has been initialized */
a_ctxptr->cstate = CTX_LINKED;  
return(MUR_SUCCESS);

mur_err_exit:
return(err_status);

} /* end: mur_com_init */


/* ==========================================================================
**
** ROUTINE: mur_com_send
*/
#ifdef MUR_STDC
MUR_CLIENT_ERR mur_com_send
(
MUR_CLIENT_CTX  *a_ctxptr,	/* MOD:  Client context */
char		*a_buffer,	/* IN :  Buffer to be sent to server */
long int	a_sendsize	/* IN :  Number of bytes to send */
)
#else
MUR_CLIENT_ERR mur_com_send(a_ctxptr, a_buffer, a_sendsize)

MUR_CLIENT_CTX  *a_ctxptr;	/* MOD:  Client context */
char		*a_buffer;	/* IN :  Buffer to be sent to server */
long int	a_sendsize;	/* IN :  Number of bytes to send */
#endif
/*
** DESCRIPTION
** 	Sends message buffer to the MURMUR server.
** 
** VALUES RETURNED
**	MUR_SUCCESS
** 
** CALLED BY
**      mur_send
**
** CALLS TO
**	send
**
** ========================================================================== 
*/
{ /* begin: mur_com_send */

MUR_CLIENT_ERR	    err_status;	    /* MURMUR client internal error status */
struct sockaddr_in  sin;
long int	    bytes_to_send, bytes_xferred;

#ifndef MURDER_LOCAL
/*
** LOOP AND SEND ALL THE DATA (IN FRAGMENTS IF REQUIRED)
*/

/* Since we haven't sent anything yet, all bytes must be sent */
bytes_to_send = a_sendsize; 

while(bytes_to_send > 0)
   {
   bytes_xferred = (long int)send((VMS_CAST int)a_ctxptr->sock, 
                   (char *)&a_buffer[(a_sendsize - bytes_to_send)],
                   bytes_to_send, (int)0);

   if (bytes_xferred > (long int)0)
      {
      bytes_to_send -= bytes_xferred;
      }
   else
      {break;}
   }

/* Check for I/O errors which may have occurred when sending */
if (bytes_xferred <= 0)
   {err_status = MUR_CERR_COM_SEND; goto mur_err_exit;}

#else
/*
** FOR MURMUR DIAGNOSTICS SIMPLY PRINT THE MESSAGE BEING SENT
*/
mur_print_msg(a_buffer);
#endif

/*
** ALL DONE, GOODBYE 
*/
return(MUR_SUCCESS);

mur_err_exit:
return(err_status);

} /* end: mur_com_send */


/* ==========================================================================
**
** ROUTINE: mur_com_cleanup
*/
#ifdef MUR_STDC
void mur_com_cleanup
(
MUR_CLIENT_CTX  *a_ctxptr	/* MOD:  Client context */
)
#else
void mur_com_cleanup(a_ctxptr)

MUR_CLIENT_CTX  *a_ctxptr;	/* MOD:  Client context */
#endif
/*
** DESCRIPTION
** 	Closes the clients communication link with the MURMUR server, and
**      the appropriate elements in the client context structure are updated
**	accordingly.  This routine is called only for error cleanup.
** 
** VALUES RETURNED
**	N/A
** 
** CALLED BY
**      mur_send
**
** CALLS TO
**	socket_close
**
** ========================================================================== 
*/
{ /* begin: mur_com_cleanup */

/*
** CLOSE THE SOCKET 
*/
#ifndef MURDER_LOCAL
socket_close((VMS_CAST int)a_ctxptr->sock);                               
#endif

/*
** CLEANUP THE CLIENT CONTEXT 
*/

/* Reset the state so the next time the user calls a MURMUR routine, the 
   messages are logged to stderr */
a_ctxptr->cstate = CTX_LOCAL;

} /* end: mur_com_cleanup */


/* ==========================================================================
**
** ROUTINE: mur_get_srvr_add
*/
#ifdef MUR_STDC
void mur_get_srvr_add
(
MUR_SADD_PTR	a_sadd		   /* OUT:  Server Internet address */
)
#else
void mur_get_srvr_add(a_sadd)

MUR_SADD_PTR	a_sadd; 	   /* OUT:  Server Internet address */
#endif
/*
** DESCRIPTION
** 	Used to get the Internet address of the MURMUR server.
** 
** VALUES RETURNED
**	N/A
** 
** CALLED BY
**
** CALLS TO
**	getenv     (Unix only)
**	sys$trnlnm (VMS only)
**
** GLOBALS REFERENCED
**	murmur_server_add (VxWorks only)
**
** ========================================================================== 
*/
{ /* begin: mur_get_srvr_add */

/*---------------------------------------------------------------------------
**
** GET SERVER ADDRESS UNDER UNIX (TRANSLATE ENVIRONMENTAL VARIABLE)
*/
#ifdef MUR_UNIX
char *l_sadd;

l_sadd = getenv(MUR_SADD_NAME);
if (l_sadd != NULL)
   {
   strncpy(a_sadd, l_sadd, MUR_SADD_LEN);
   a_sadd[MUR_SADD_LEN] = 0;  /* Null terminate */
   } 
else
   {a_sadd[0] = 0;}
#endif

/*---------------------------------------------------------------------------
**
** GET SERVER ADDRESS UNDER VXWORKS
*/
#ifdef MUR_VXWORKS
murmur_server_add[MUR_SADD_LEN] = (char)0;  /* Null terminate just in case */
strcpy(a_sadd, murmur_server_add);
#endif

/*---------------------------------------------------------------------------
**
** GET SERVER ADDRESS UNDER VMS (TRANSLATE LOGICAL)
*/
#ifdef MUR_VMS
char			sadd[] = MUR_SADD_NAME;
static unsigned long	attr   = LNM$M_CASE_BLIND;	/* TRNLNM attributes  */
static unsigned char	acmode = PSL$C_USER;		/* TRNLNM access mode */
static $DESCRIPTOR	(tabnam, "LNM$FILE_DEV");	/* TRNLNM table name  */

unsigned long int	status;
unsigned long int	i = 0;

struct dsc$descriptor lognam = {0, DSC$K_DTYPE_T, DSC$K_CLASS_S, 0};
struct
   {
   unsigned short	buflen;	    /* Buffer length (bytes) */
   unsigned short	item;	    /* Nature of our request */
            char	*bufadr;    /* Buffer address        */
            short	*retlen;    /* Return length address */
   }

geteqv []= {{MUR_SADD_LEN,  LNM$_STRING, a_sadd,  &i}, {0, 0, 0, 0}};

lognam.dsc$w_length  = strlen(sadd);
lognam.dsc$a_pointer =        sadd;

if ((status = sys$trnlnm (&attr, &tabnam, &lognam, &acmode, geteqv)) == SS$_NORMAL)
   {
   ((char *)a_sadd)[i] = 0; /* Null-terminate the string */
   }
else
   {
   ((char *)a_sadd)[0] = 0; /* Undefined */
   }
#endif
} /* end: mur_get_srvr_add */


/* ==========================================================================
**
** ROUTINE: mur_client_init
*/
#ifdef MUR_STDC
MUR_CLIENT_ERR mur_client_init
(
MUR_CLIENT_CTX	*a_ctxptr,	   /* MOD:  Client context */
unsigned long	a_msgcode	   /* IN:   Message code being reported */
)
#else
MUR_CLIENT_ERR mur_client_init(a_ctxptr, a_msgcode)

MUR_CLIENT_CTX	*a_ctxptr;	   /* MOD:  Client context */
unsigned long	a_msgcode;	   /* IN:   Message code being reported */
#endif
/*
** DESCRIPTION
** 	Used to initilize the client and connect to the MURMUR server.
** 
** VALUES RETURNED
**	MUR_SUCCESS
**	MUR_CERR_COM_HDR
**	MUR_CERR_HDR_PACK
** 
** CALLED BY
**	mur_send
**
** CALLS TO
**	mur_com_init
**	mur_get_srvr_add
**      mur_internal_err
**	mur_rec_hdr_pack
**	mur_timezone
**	various XDR routines
**
** ========================================================================== 
*/
{ /* begin: mur_client_init */

MUR_CLIENT_ERR	err_status;
MUR_SADD	srvr_add;
MUR_PID		lpid;
MUR_TZ		lzone;
MUR_NODE	lnode;
long int	rec_length;
XDR		lxdr;
MUR_MSG_TYPE_ENUM msgtype;
int sighand;

#ifndef MUR_VXWORKS
 sighand =  (int) signal(SIGPIPE,SIG_IGN);     /* set to ignore the signal */
 if(sighand > 0)signal(SIGPIPE, (void (*)()) sighand); /* if one already exists, reestablish it */
#endif

/*
** TRANSLATE THE EQUIVALENCE STRING (UNIX) OR THE LOGICAL (VMS) OR THE 
** GLOBAL VARIABLE (VXWORKS) TO GET THE SERVER INTERNET ADDRESS
*/
mur_get_srvr_add(srvr_add);  /* Get address of server */

/*
** PACK THE DATA INTO THE HEADER IF THE SERVER ADDRESS IS DEFINED
*/
if (srvr_add[0] == 0)
   {
   err_status = MUR_CERR_SRVRADD;  /* Server address not defined - get out */
   goto mur_err_exit;
   }
else
   {
   msgtype = MSG_INIT;		/* This is an initialization message */
   rec_length = MUR_USER_REC_LEN;	/* Get length of the user record */
   MUR_ASCIIPID(lpid, a_ctxptr->pid);	/* Get ASCII copy of PID */
   MUR_GETHOSTNAME(lnode);		/* Get (ASCII) host node name */
   mur_timezone(lzone);		/* Get (ASCII) time zone */

   /* Set XDR buffer to point to User Header area */
   xdrmem_create((XDR *)&lxdr, (char *)a_ctxptr->user_hdr,
                    MUR_USER_HDR_LEN, XDR_ENCODE);

   /* Pack message type */
   if (!(xdr_enum((XDR *)&lxdr, (enum_t *)&msgtype)))
      {err_status = MUR_CERR_HDR_PACK; goto mur_err_exit;}

   /* Pack the rest */
   err_status = mur_rec_hdr_pack(&lxdr, (MUR_PID_PTR)lpid, (MUR_NODE_PTR)lnode, 
                                    (MUR_VERSION_PTR)MUR_CURRENT_VERSION, 
                                    (MUR_TZ_PTR)lzone, (long *)&rec_length);
   if (err_status != MUR_SUCCESS)
      {
      err_status = MUR_CERR_HDR_PACK;
      goto mur_err_exit;
      }

   /*
   ** INITIALIZE THE LINK 
   */
   err_status = mur_com_init(a_ctxptr, (char *)srvr_add, MUR_SERVER_PORT);
   if (err_status != MUR_SUCCESS) goto mur_com_err_exit;

   /*
   ** SEND THE USER RECORD HEADER TO THE SERVER 
   */
   err_status = mur_com_send(a_ctxptr, (char *)a_ctxptr->user_hdr, MUR_SEND_SIZE);
   if (err_status != MUR_SUCCESS)
      {
      err_status = MUR_CERR_COM_HDR;  /* Change error status to what we want */
      goto mur_com_err_exit;
      }

   } /* else (if (srvr_add[0] == 0)...) */

/*
** NORMAL EXIT
*/
return(MUR_SUCCESS);

/*
** COMMUNICATION ERROR 
*/
mur_com_err_exit:
mur_com_cleanup(a_ctxptr);      /* Close socket and clean up client context */

/*
** ERROR EXIT 
*/
mur_err_exit:
/* Since some type of error occurred, switch to local message reporting.
   That is, messages will be sent to stderr only.  No connection to the
   server was made. */
a_ctxptr->cstate = CTX_LOCAL;	/* Switch to local mode */
return(err_status);

} /* end: mur_client_init */





