/*************************************************
 * CVS Log
 * murder.c,v
 * Revision 1.1.1.1  1993/07/14  19:43:23  dbrenner
 * UNIDAQ 2.1 distribution - July 14,1993
 *
*************************************************/
/* ==========================================================================
**
** MURDER.C
**
** DESCRIPTION
**	This file contains MURMUR Diagnostic and Error Reporting routines.
**
**	THESE ROUTINES ARE FOR DEBUGGING PURPOSES ONLY.
**
** DEVELOPERS
**	Gary P. Sergey
**
**	Fermilab Data Acquisition Group
**	Batavia, Il 60510, U.S.A.
**
** MODIFICATIONS
**         Date       Initials	Description
**	31-Mar-1992	GPS	Original Entry
** 
** ENTRY POINTS			 SCOPE
**	murder_test		external
**      murder_stdset		internal
**      murder_stdset2		internal
**      murder_server_diag	internal
*/
#include "mur_cond.h"

#ifdef MUR_VMS
#include "MULTINET_COMMON_ROOT:[multinet.include.rpc]rpc.h"
#include "MULTINET_COMMON_ROOT:[multinet.include.rpc]xdr.h"
#endif

#ifdef MUR_SGI
#include <stdio.h>	/* Need for SGI (__file_s declaration) */
#endif

#ifdef MUR_UNIX
#include <rpc/types.h>
#include <rpc/xdr.h>
#endif

#ifdef MUR_VXWORKS
#include <vxWorks.h>
#include <rpc/rpctypes.h>
#include <rpc/xdr.h>
#endif

#include "mur_def.h"
#include "mur_func.h"
#include "murmur_msg_c.h"
#include "demo_msg_c.h"


/* ==========================================================================
**
** ROUTINE: murder test main program
*/
#ifdef MUR_VXWORKS
void murder_main()
#else
main()
#endif
/*
** DESCRIPTION
**	MURMUR Diagnostic and Error Reporting test, used to simulate a MURMUR
**	client.  Interactive input of message codes (and message parameters) 
**	is supported, as well as a fixed set of test calls.
**
** ========================================================================== 
*/
{
#define TESTLIMIT   12  /* Maximum number of arguments to try to send */
long int status;
unsigned long	msgcode;
int		option, numargs, i, ii;
unsigned long	arg[TESTLIMIT];
unsigned long   argtype[TESTLIMIT];
char            lstring[TESTLIMIT][150];

printf("\n\n");
printf("              ***************************************************\n");
printf("              ***************************************************\n");
printf("              ***                                             ***\n");
printf("              ***                  MURDER                     ***\n");
printf("              ***                                             ***\n");
printf("              ***    MURMUR Diagnostic Error Reporting Test   ***\n");
printf("              ***                                             ***\n");
printf("              ***************************************************\n");
printf("              ***************************************************\n");
while(1)
   {
   printf("\n\n");
   printf("TESTS:\n  0 - Exit\n  1 - Interactive\n  2 - Standard set\n");
   printf("  3 - Standard set loop\n  4 - Server client list dump\n");
   printf("  5 - Infinite standard set loop\n");
   printf("\nEnter option : ");
   scanf("%1d", &option);
   printf("\n");

   if (option == 0) {break;}
   switch(option)
      {
      case 1:
      printf("\n==================== INTERACTIVE TEST ====================\n");
      while(1)
         {
         printf("\nEnter message code in hexidecimal (0 to exit) : ");
         scanf("%8x", &msgcode);
         printf("\n");
         if (msgcode == 0) {break;}
         numargs = 0;
         do 
            {
            printf("Enter number of arguments : ");
            scanf("%1d", &numargs);
            printf("\n");
            if (numargs <= TESTLIMIT) 
               {break;}
            else
               {printf("Too many arguments for test program\n");}
            } while(numargs > TESTLIMIT);

         if (numargs > 0)
            {
            printf("\nValid argument types :\n");
            printf("  1 - MUR_LONG\n");
            printf("  2 - MUR_U_LONG\n");
            printf("  3 - MUR_FLOAT\n");
            printf("  4 - MUR_STRING\n");
            printf("  5 - MUR_APPEND\n");
            printf("  6 - MUR_TEXT_ONLY_MSG\n\n");
            for (i=0; i<numargs; i++)
               {
               printf("Enter argument %d type : ",i);
               scanf("%1d", &argtype[i]);
               printf("\n");
               switch(argtype[i])
                  {
                  case 1: 
                  case 2:
                  printf("Enter longword argument in decimal : ");
                  scanf("%8d", &arg[i]);
                  printf("\n");
                  break;

                  case 3:
                  printf("Enter floating point value : ");
                  scanf ("%9f", &arg[i]);
                  printf("\n");
                  break;

                  case 4: 
                  printf("Enter text string : ");
                  scanf("%40s", &lstring[i][0]);
                  printf("\n");
                  arg[i] = (unsigned long)&lstring[i][0];
                  break;
   
                  case 5:
                  case 6:
                  printf("Enter text string : ");
                  scanf("%150s", &lstring[i][0]);
                  printf("\n");
                  arg[i] = (unsigned long)&lstring[i][0];
                  break;
   
                  default:
                  printf("-- Not implemented yet --");
                  break;
                  }
               } /* for (i... */
            } /* if (numargs > 0) ... */

         printf("\nCalling mur_send - \n");
         status = (long int)mur_send((MUR_MSG_STATUS)msgcode, (int)numargs,
                      (MUR_ARG_TYPE)argtype[0], (unsigned long)arg[0],
                      (MUR_ARG_TYPE)argtype[1], (unsigned long)arg[1],
                      (MUR_ARG_TYPE)argtype[2], (unsigned long)arg[2],
                      (MUR_ARG_TYPE)argtype[3], (unsigned long)arg[3],
                      (MUR_ARG_TYPE)argtype[4], (unsigned long)arg[4],
                      (MUR_ARG_TYPE)argtype[5], (unsigned long)arg[5],
                      (MUR_ARG_TYPE)argtype[6], (unsigned long)arg[6],
                      (MUR_ARG_TYPE)argtype[7], (unsigned long)arg[7],
                      (MUR_ARG_TYPE)argtype[8], (unsigned long)arg[8],
                      (MUR_ARG_TYPE)argtype[9], (unsigned long)arg[9],
                      (MUR_ARG_TYPE)argtype[10], (unsigned long)arg[10],
                      (MUR_ARG_TYPE)argtype[11], (unsigned long)arg[11]);

         if (status != MUR_SUCCESS) {printf("Unsuccessful return, status= %X\n",status);}
         printf("\n----------------------------------------------------------\n");
         } /* while(1) */
      break;

      case 2:
      printf("\n==================== STANDARD SET TEST ===================\n\n");
      printf("\n----------------------------------------------------------\n");
      printf("CALLING SEQUENCE : mur_set_proc_name(\"TEST_DUDE\");\n");
      status = (long int)mur_set_proc_name("TEST_DUDE");
      if (MUR_MSG_FAIL(status)) {printf("Unsuccessful return, status= %X\n",status);}
      murder_stdset();
      break;

      case 3:
      printf("\n================== STANDARD SET LOOP TEST ================\n\n");
      printf("Enter number of loops (5 messages per loop): ",ii);
      scanf("%1d", &ii);
      printf("\n");
      for(i=0; i < ii; i++)
         {
         status = murder_stdset();
         printf("\nLOOP %d COMPLETE\n", i+1);
         if (MUR_MSG_FAIL(status) && (status != MUR_LCL_SWITCH) ) {break;}
         }
      break;

      case 4:
      printf("\n==================== SERVER DIAGNOSTICS ====================\n\n");
      status = murder_server_diag();
      if (MUR_MSG_SUCCESS(status))
         {
         printf("Server diagnostics successfull");
         printf(" - See MURMUR internal error file\n");
         }
      else
         {printf("Server diagnostics failed\n");}
      break;

      case 5:
      printf("\n============== INFINITE STANDARD SET LOOP TEST =============\n\n");
      printf("\n");
      while(1)
         {
         for(i=0; i < 100; i++)
            {
            status = murder_stdset2();
            sleep(5);
            if (MUR_MSG_FAIL(status) && (status != MUR_LCL_SWITCH) ) {break;}
            }
         if (MUR_MSG_FAIL(status) ) 
            {
            printf("Unsuccessful return, status= %X\n",status);
            if(status != MUR_LCL_SWITCH)break;
            }
         printf("MURDER HEARTBEAT\n");
         }
      break;

      default:
      printf("Invalid option bonehead\n");
      break;
      };  /* switch */
   } /* while(1) */
}


/* ==========================================================================
**
** ROUTINE: murder standard set routine
*/
long int murder_stdset()

{
long int status;

printf("\n----------------------------------------------------------\n");
printf("CALLING SEQUENCE : mur_send(DEMO_SUCCESS,0);\n");
status = (long int)mur_send(DEMO_SUCCESS, 0);
if (MUR_MSG_FAIL(status) && (status != MUR_LCL_SWITCH) ) {goto exitit;}

printf("CALLING SEQUENCE : mur_send(DEMO_WARNING, 0);\n");
status = (long int)mur_send(DEMO_WARNING, 0);
if (MUR_MSG_FAIL(status) && (status != MUR_LCL_SWITCH) ) {goto exitit;}

printf("CALLING SEQUENCE : mur_send(DEMO_INFORM, 0);\n");
status = (long int)mur_send(DEMO_INFORM, 0);
if(MUR_MSG_FAIL(status) && (status != MUR_LCL_SWITCH) ) {goto exitit;}

printf("CALLING SEQUENCE : mur_send(DEMO_ERROR, 0);\n");
status = (long int)mur_send(DEMO_ERROR, 0);
if (MUR_MSG_FAIL(status) && (status != MUR_LCL_SWITCH) ) {goto exitit;}

printf("CALLING SEQUENCE : mur_send(DEMO_TEST_FORMATS, 3, MUR_STRING,\n");
printf("                            \"Mbula was here\", MUR_LONG, -123,\n");
printf("                            MUR_U_LONG, 456);\n");
status = (long int) mur_send(DEMO_TEST_FORMATS, 3, MUR_STRING, "Mbula was here",
                            MUR_LONG, -123, MUR_U_LONG, 456);
if( MUR_MSG_FAIL(status) && (status != MUR_LCL_SWITCH) ) {goto exitit;}

exitit:
if (MUR_MSG_FAIL(status)) {printf("Unsuccessful return, status= %X\n",status);}
return(status);
}


/* ==========================================================================
**
** ROUTINE: murder standard set routine II
*/
long int murder_stdset2()

{
long int status;

status = (long int)mur_send(DEMO_SUCCESS, 0);
if (status != MUR_SUCCESS) {goto exitit;}
status = (long int)mur_send(DEMO_WARNING, 0);
if (status != MUR_SUCCESS) {goto exitit;}
status = (long int)mur_send(DEMO_INFORM, 0);
if (status != MUR_SUCCESS) {goto exitit;}
status = (long int)mur_send(DEMO_ERROR, 0);
if (status != MUR_SUCCESS) {goto exitit;}
status = (long int)mur_send(DEMO_TEST_FORMATS, 3, MUR_STRING, "Mbula was here",
                            MUR_LONG, -123, MUR_U_LONG, 456);
if (status != MUR_SUCCESS) {goto exitit;}

exitit:
if (status != MUR_SUCCESS) {printf("Unsuccessful return, status= %X\n",status);}
return(status);
}


/* ==========================================================================
**
** ROUTINE: murder_server_diag
*/
MUR_MSG_STATUS murder_server_diag()
/*
** DESCRIPTION
**	MURMUR Diagnostic and Error Reporting routine used to call the
**      server and tell it to dump the list of connected clients to the
**      MURMUR internal error file.
** 
** ========================================================================== 
*/
{ /* begin: mur_server_diag */

MUR_CLIENT_ERR	err_status;	    /* MURMUR client internal error status */
MUR_BOOL	prnt_errno  = FALSE;
MUR_ARG_FLAGS	argtypeflags;
MUR_EXTRA_FLAGS extraflags;
char		*argtypeflagsptr, *extraflagsptr;
unsigned int    argtypelen, extralen;
XDR		*xdrs;
MUR_CLIENT_CTX  *ctxptr;
MUR_PID		lpid;
MUR_NODE	lnode;
long int	rec_length;
MUR_CTIME	ltime;
MUR_CDATE	ldate;
MUR_TZ		lzone;
MUR_SADD	srvr_add;
MUR_MSG_TYPE_ENUM msgtype;
MUR_MSG_STATUS  l_msgcode=0;
/*
** GET POINTERS TO FLAG VARIABLES 
*/
argtypeflagsptr = (char *)&argtypeflags.c[0];
extraflagsptr   = (char *)&extraflags.c[0];

/*---------------------------------------------------------------------------
**
** RETRIEVE CONTEXT FOR THIS CLIENT - OR CREATE CONTEXT IF NEW CLIENT
*/
err_status = mur_get_ctx(&ctxptr, l_msgcode);
if (err_status != MUR_SUCCESS) goto mur_ctx_err_exit;

/*---------------------------------------------------------------------------
**
** PACK MESSAGE INFORMATION FIELDS INTO USER RECORD
*/

/* Get the client time and date in ASCII */
mur_time_date((MUR_CTIME_PTR)ltime, (MUR_CDATE_PTR)ldate);

/* Set XDR structure to point to user record area */
xdrmem_create((XDR *) &ctxptr->xdrs, (char *)ctxptr->user_rec,
              MUR_USER_REC_LEN, XDR_ENCODE);

xdrs = (XDR *) &ctxptr->xdrs;  /* get XDR handle pointer */

/* Pack message type */
msgtype = MSG_SDIAG;
if (!(xdr_enum(xdrs, (enum_t *)&msgtype)))
   {err_status = MUR_CERR_REC_PACK; goto mur_err_exit;}

err_status = mur_rec_pack(xdrs, &l_msgcode, (MUR_CTIME_PTR)ltime, 
                          (MUR_CDATE_PTR)ldate);
if (err_status != MUR_SUCCESS)
   {
   err_status = MUR_CERR_REC_PACK;
   goto mur_err_exit;
   }


/*---------------------------------------------------------------------------
**
** SINCE THERE ARE NO ARGUMENTS, PACK NULL ARGUMENT FLAGS
*/
argtypeflags.l[0] = 0;
argtypeflags.l[1] = 0;
extraflags.l      = 0;
argtypelen = MUR_NUM_ARGS;
extralen   = sizeof(long);

if (!(xdr_bytes(xdrs, (char **)&argtypeflagsptr, &argtypelen, argtypelen)))
   {err_status = MUR_CERR_ARG_PACK; goto mur_err_exit;}
if (!(xdr_bytes(xdrs, (char **)&extraflagsptr, &extralen, extralen)))
    {err_status = MUR_CERR_ARG_PACK; goto mur_err_exit;}

/*---------------------------------------------------------------------------
**
** INITIALIZE COMMUNICATION LINK WITH SERVER IF NECESSARY 
*/
if (ctxptr->cstate == CTX_NEW_LINK)   /* Initialize if this is a new client */
   {
   /*
   ** TRANSLATE THE EQUIVALENCE STRING (UNIX) OR THE LOGICAL(VMS)
   ** TO GET THE SERVER INTERNET ADDRESS
   */
   mur_get_srvr_add(srvr_add);
   if (srvr_add[0] == 0)
      {err_status = MUR_CERR_SRVRADD; goto mur_err_exit;}

   /*
   ** PACK THE DATA INTO THE HEADER
   */
   msgtype = MSG_INIT;			/* This is an initialization message */
   rec_length = MUR_USER_REC_LEN;	/* Get length of the user record */
   MUR_ASCIIPID(lpid, ctxptr->pid);	/* Get ASCII copy of PID */
   MUR_GETHOSTNAME(lnode);		/* Get (ASCII) host node name */
   mur_timezone(lzone);			/* Get (ASCII) time zone */

   /* Set XDR buffer to point to User Header area */
   xdrmem_create((XDR *)xdrs, (char *)ctxptr->user_hdr,
                 MUR_USER_HDR_LEN, XDR_ENCODE);

   /* Pack message type */
   if (!(xdr_enum((XDR *)xdrs, (enum_t *)&msgtype)))
      {err_status = MUR_CERR_HDR_PACK; goto mur_err_exit;}

   /* Pack the rest */
   err_status = mur_rec_hdr_pack(xdrs, (MUR_PID_PTR)lpid, (MUR_NODE_PTR)lnode, 
                                 (MUR_VERSION_PTR)MUR_CURRENT_VERSION, 
                                 (MUR_TZ_PTR)lzone, (long *)&rec_length);
   if (err_status != MUR_SUCCESS)
      {
      err_status = MUR_CERR_HDR_PACK;
      goto mur_err_exit;
      }

   /*
   ** INITIALIZE THE LINK 
   */
   err_status = mur_com_init(ctxptr, (char *)srvr_add, MUR_SERVER_PORT);
   if (err_status != MUR_SUCCESS) goto mur_com_err_exit;

   /*
   ** SEND THE USER RECORD HEADER TO THE SERVER 
   */
   err_status = mur_com_send(ctxptr, (char *)ctxptr->user_hdr, MUR_SEND_SIZE);
   if (err_status != MUR_SUCCESS)
      {
      err_status = MUR_CERR_COM_HDR;  /* Change error status to what we want */
      goto mur_com_err_exit;
      }
   }


/*---------------------------------------------------------------------------
**
** SEND USER RECORD TO MURMUR SERVER 
*/

err_status = mur_com_send(ctxptr, (char *)ctxptr->user_rec, MUR_SEND_SIZE);
if (err_status != MUR_SUCCESS) goto mur_com_err_exit;


/*---------------------------------------------------------------------------
**
** RETURN FROM MUR_SEND
*/

/*
** NORMAL SUCCESSFUL COMPLETION 
*/
return(MUR_SUCCESS);

/*
** COMMUNICATION ERROR 
*/
mur_com_err_exit:
mur_com_cleanup(ctxptr);    /* Close socket and clean up client context */
prnt_errno  = TRUE;	    /* Print errno when mur_internal_err is called */

/*
** ERROR EXIT 
*/
mur_err_exit:
mur_ctx_err_exit:
/* Report error as best we know how */
mur_internal_err(err_status, l_msgcode, prnt_errno); 
return(0);

} /* end: murder_server_diag */
