/**********************************************************************/
/*  This is file pack_log_message.C                                   */
/*  It contains the routines to pack parameters of murmur calls, then */
/*  call the logbook program to record these.  Alternately, the       */
/*  packed parameters can be interpreted as simple messages to be     */
/*  logged in the case where a message code of zero is specified.     */
/*  Entry points:  add_err_string ( char *s)                          */
/*                 send_err_string ( u_long err_no )                  */
/*                 send_err_mess ( u_long err_no, char *s)            */
/**********************************************************************/

/************************************************************************
*                                                                       *
* Copyright (c) 1993                                                    *
* The Regents of The University of Michigan                             *
* Universities Research Association, Inc.                               *
* All Rights Reserved                                                   *
*                                                                       *
*                Authors:                                               *
*                        R. Ball, C. Timmermans, UofM                   *
*                        Y. Takeuchi, T.I.T.                            *
*                        M. Nomachi, KEK                                *
*                        A. Fry, C. Erbas, D. Brenner, SSCL             *
*                                                                       *
************************************************************************/

#include <sys/types.h>
#include <stdio.h>

#ifdef _DEBUG
#include <stdlib.h>
#endif

#include <string.h>

#ifdef _CPP
extern "C" int Send_Message(char *p,char *s);
#endif

#define COMMAND "RoundUp"

/***********************************************************************/
/*  #include "mur_pack_defs.h"  */
/***********************************************************************/
/*  This is file mur_pack_defs.h                                       */
/*  It contains the definitions used within logbook clients in order   */
/*  for logbook to successfully and usefully be a murmur client.       */
/***********************************************************************/

/*  First, the structure used to transmit arguments to the logbook     */
/*  which is where the murmur server is called.  This info is then     */
/*  sent as a message to logbook.                                      */

#define  MUR_LOG_LEN  256
struct mur_arg {
                long   string_count;
                long   char_count;
                char   log_arg[MUR_LOG_LEN];
	       };

#define  INIT_STR_CNT   0
#define  INIT_CHR_CNT  13
struct mur_arg mess_storage = {
                               INIT_STR_CNT,
                               INIT_CHR_CNT,
                               " "
			      };
/*  End of included file  */
/***********************************************************************/

#ifdef _CPP
long add_err_string(char *s);
long send_err_string(u_long p1);
long send_err_mess ( u_long err_no, char *s);
#endif

extern char ownpname[15];          /*  Contains process name   */

/**************************************************************************/
#ifdef _CPP
long add_err_string ( char *err_mess )
#else
long add_err_string ( err_mess )
char *err_mess;
#endif
/*  Copy the message string "err_mess" into the storage structure.    */
/*  The string is packed with a leading NULL as a separator between   */
/*  multiple messages and other information packed into the leading   */
/*  edge of the message.  A count of characters is maintained along   */
/*  with the count of messages.                                       */

/*  Return zero for success, 1 for failure.                           */

{
  if ( (strlen(err_mess)+mess_storage.char_count) >= MUR_LOG_LEN)  {
    return ((long) 1);
  } else   {
    (mess_storage.string_count)++;
    strcpy (&mess_storage.log_arg[mess_storage.char_count], err_mess);
    mess_storage.log_arg[mess_storage.char_count-1] = '\0';
    mess_storage.char_count += (strlen(err_mess)+1);
  }
  return ((long) 0);
}

/**************************************************************************/
#ifdef _CPP
long send_err_string ( u_long err_code )
#else
long send_err_string ( err_code )
u_long err_code;
#endif
/*  Pack up the complete set of strings with the err_code, string count,  */
/*  and program name and send it to logbook.                              */

/*  Returns 0 for OK, 1 for error.                                        */

{
#ifdef _DEBUG
  char c;
  register int k;
  long init_k;
#endif

  char   send_buffer[512];
  register int j, k_stor;

  if ( (err_code == 0) && (mess_storage.string_count == 0) ) {

/*  Later, send a warning message to logbook from here                     */

    return ((long) 1);
  } else {

/*  Place the process name and the command to invoke murmur in the buffer  */
/*    strcpy (send_buffer, ownpname);
    k_stor = strlen(ownpname); */
    k_stor = 0;
    send_buffer[k_stor++] = ' ';
    strcpy (&send_buffer[k_stor],COMMAND);
    k_stor = strlen(send_buffer);

/*  Change trailing NULL to a blank.  Also, stuff in the error code and    */
/*  the number of associated strings.                                      */
    send_buffer[k_stor] = ' ';     
    sprintf (&send_buffer[k_stor+1], "%10x %d", err_code,
             mess_storage.string_count);
    send_buffer[INIT_CHR_CNT+k_stor] = '\0';
    if (mess_storage.string_count > 0)   {
      for (j=0; j<mess_storage.char_count; j++)
	send_buffer[k_stor+14+j] = mess_storage.log_arg[INIT_CHR_CNT+j];
    }
#ifdef _DEBUG
    init_k = 0;
    for (j=0; j<mess_storage.string_count+1; j++)   {
      for (k=init_k; (c=send_buffer[k])!='\0'; k++) {
        printf ("%c",c);
      }
      init_k = k+1;
      printf ("!");
    }
    printf ("\n");
#endif

/*  Before sending the buffer off, the NULLs must be converted to \n.      */
/*  Otherwise the sending routines will terminate at the first NULL.       */
    for (j=0; j<k_stor+mess_storage.char_count; j++)
              if (send_buffer[j] == '\0') send_buffer[j] = '\n';
    j = Send_Message("logbook", send_buffer);
/*  Reset the internal counters.                                           */

    mess_storage.string_count = INIT_STR_CNT;
    mess_storage.char_count = INIT_CHR_CNT;
  }
  return ((long) 0);
}
/**************************************************************************/
#ifdef _CPP
long send_err_mess ( u_long err_no, char *s)
#else
long send_err_mess ( err_no, s)
u_long err_no;
char *s;
#endif
/*  Combine the functions of add_err_string and send_err_string           */
{
  long j;
  if (s != NULL)	{
    j = add_err_string ( s );
    if (j != 0)    return(j);
  }
  j = send_err_string ( err_no );
  return (j);
}

