/*//////////////////////////////////////////////////////////////////////////
// program: logbook.C
// date:    7/22/92
// author:  R. Ball
/////////////////////////////////////////////////////////////////////////*/

/************************************************************************
*                                                                       *
* Copyright (c) 1993                                                    *
* The Regents of The University of Michigan                             *
* Universities Research Association, Inc.                               *
* All Rights Reserved                                                   *
*                                                                       *
*                Authors:                                               *
*                        R. Ball, C. Timmermans, UofM                   *
*                        Y. Takeuchi, T.I.T.                            *
*                        M. Nomachi, KEK                                *
*                        A. Fry, C. Erbas, D. Brenner, SSCL             *
*                                                                       *
************************************************************************/

#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <time.h>
#include <sys/time.h>
#include "logbook.h"
#include "mur_mess_defs.h"          /*  The "interpreted" murmur errors  */
#include "sdc_pack_msg_c.h"         /*  In pdaq/error/inc   */

#ifdef NO_MURMUR
#define MUR_PROTO(x) x
#define MUR_STRING (u_long)1
u_long mur_send();
#define  mur_send_text(a_text) mur_send(0,1,MUR_STRING, (a_text))
#else
#include "mur_cond.h"               /*  $MURMUR_CLIENT_DIR/include    */
#include "mur_user.h"            /*   ditto    */
#include "murmur_msg_c.h"           /*   ditto    */
#endif

#define ENV_NAME  "LGBK_LOC"
extern  void rd_mess_list();
void         mess_cat ();
int          Process_Event ();

/*****************************************************/
/* some variables for general use                    */
/*****************************************************/
#define MURMUR_FAIL "FAILed to log via murmur"
char log_file[256];
extern char ownpname[];

/***********************************************************/
void Init_Process(argc, argv) /*perform startup operations*/
int argc;
char **argv;
{
	int j;
	char tmfile[255], *env_val;

#ifdef _DEBUG
	printf("LOGBOOK:: entering init_process\n");
#endif
        if(strcmp(ownpname,"logbook") != 0) {
          Send_Message(ownpname,"EXIT");
          return;
        }
	rd_mess_list();

/*
 * See if the environmental variable for the logbook file exists.  If it
 * does, use that as the logbook file.  If it does not, use that one hardwired
 * at installation.
 */
	if((env_val = getenv(ENV_NAME)) == NULL)  strcpy(tmfile,LOGBOOK);
        else                                      strcpy(tmfile,env_val);

	j = decode_filename ( tmfile, log_file );
#ifdef _DEBUG
	printf("entering process event\n");
#endif
	j = Process_Event("logbook RoundUp 0 1\nLogbook Process Starting");
#ifdef _DEBUG
	printf("Leaving process event\n");
#endif
}
/***********************************************************/
int Process_Event(string)
char *string;
{
	FILE *fp, *fp2;
	struct timeval tp;
	struct timezone tzp;
	struct tm *now;
	time_t nowtime;
	char wrd[512], lne[512], proc[512];
	char dated_file[512];
	u_long mval, list_val, status;
	int   npar, j;
	char nstrng[512];
	int kloc[5], init_k;

        if(strcmp(ownpname,"logbook") != 0) {
          return(0);
        }
	gettimeofday (&tp, &tzp);
	nowtime = time(NULL);
	now = localtime ( &nowtime );
	strcpy (dated_file, log_file);
	sprintf (&dated_file[strlen(log_file)], "%d%02d%02d\0", now->tm_year,
		now->tm_mon+1, now->tm_mday);
	sscanf (string, "%s %s", proc, wrd);

/* Command is READ, ie, read a file and copy it to the log book file.  */
	if (strcasecmp(wrd, "read") == 0)     {
	  sscanf (string, "%s %s %s", proc, wrd, lne);
	  fp = fopen (dated_file, "a+");
	  fprintf(fp,"==================== %s ========================\n",
                  proc);
	  fprintf(fp,"%s",ctime((time_t *)&(tp.tv_sec)));
	  fp2 = fopen(lne,"r");
	  if(fp2 != 0) {
	    fprintf(fp,"Copying file %s to logbook.\n", lne);
	    while(fgets(lne,99,fp2) != 0){
	      fprintf(fp,"%s",lne);
	    }
	    fclose(fp2);
	  }
	  else {
	    fprintf (fp, "Failed to open/copy the requested file %s\n", lne);
	  }
	  fclose(fp);
        }
	else {

/*  This is a message of one kind or another.  The first kind is one in */
/*  which the entire content following 'proc' is a TEXTONLY message     */
/*  to be logged.  The second kind, handled below, will have multiple   */
/*  parameters, \n-separated.  The parameters are the error number      */
/*  (zero for TEXTONLY), the number of strings, then the \n-separated   */
/*  strings.  Of course, the sending-process name always leads.         */

#ifdef _DEBUG
	  printf ("Sending process and command are '%s' and '%s'\n", proc,wrd);
#endif
	  if (strcasecmp(wrd, "log") == 0)  {
	    mval = 0;
	    npar = 1;
	    string[strlen(proc)+strlen(wrd)+1] = '\0';
	  } 
	  else if (strcmp(wrd, "RoundUp") == 0)  {

/*  This is a more 'standard' message.  This means that any embedded \n */
/*  must be converted to a NULL.   So do it.                            */
	    for (j=0; (string[j]!='\0'); j++)
		if (string[j] == '\n') string[j]='\0';
	    if(sscanf (string, "%s %s %x %d", proc, wrd, &mval, &npar) < 3) {
	      return(0);
	    }
	  }
	  else  {
#ifdef _DEBUG
	    printf ("Command not recognized\n");
#endif
	    return(1);
	  }

	  fp = fopen (dated_file, "a+");
	  fprintf(fp,"==================== %s ========================\n",
                  proc);
	  fprintf(fp,"%s",ctime((time_t *)&(tp.tv_sec)));
	  if (mval == 0) {

/*  The error code is zero, so the parameters will simply be logged as  */
/*  messages in the logbook file and sent as murmur text.               */

	    init_k = strlen(string)+1;
	    for (j=0; j<npar; j++) {
              fprintf(fp,"%s\n",&string[init_k]);
	      status = (u_long)mur_send_text ( &string[init_k] );
	      init_k += (strlen(&string[init_k])+1);
	    }
	  } 
	  else {
/*  LOG the message via murmur, if possible.  The NULLs separating      */
/*  string parameters are located via the 'kloc' array.                 */
	    kloc[0] = strlen(string)+1;
	    switch ( npar )
		{
		case 0:
		   status = (u_long)mur_send ( mval, npar );
		   break;
		case 1:
		   status = (u_long)mur_send ( mval, npar, MUR_STRING,
			 &string[kloc[0]] );
		   break;
		case 2:
		   kloc[1] = kloc[0] + strlen(&string[kloc[0]]) + 1;
		   status = (u_long)mur_send ( mval, npar, MUR_STRING,
		    &string[kloc[0]], MUR_STRING, &string[kloc[1]] );
		   break;
		case 3:
		   kloc[1] = kloc[0] + strlen(&string[kloc[0]]) + 1;
		   kloc[2] = kloc[1] + strlen(&string[kloc[1]]) + 1;
		   status = (u_long)mur_send ( mval, npar, MUR_STRING,
		    &string[kloc[0]], MUR_STRING, &string[kloc[1]], MUR_STRING,
		    &string[kloc[2]] );
		   break;
		case 4:
		   kloc[1] = kloc[0] + strlen(&string[kloc[0]]) + 1;
		   kloc[2] = kloc[1] + strlen(&string[kloc[1]]) + 1;
		   kloc[3] = kloc[2] + strlen(&string[kloc[2]]) + 1;
		   status = (u_long)mur_send ( mval, npar, MUR_STRING,
		    &string[kloc[0]], MUR_STRING, &string[kloc[1]], MUR_STRING,
		    &string[kloc[2]], MUR_STRING, &string[kloc[3]] );
		   break;
		default:
		   status = (u_long)999;
		   break;
		}              /* end of switch on number of args */

/*  Also, LOG the message locally, with appropriate formatting.   */
	    for (init_k=0; (list_val = opr_mess_list[init_k].mesnum) != 0;
						 init_k++)  {
	        if (list_val == mval) break;
	    }
	    mess_cat ( nstrng, list_val, npar, string, kloc, init_k );
	    fprintf (fp, "%s\n", nstrng);
#ifdef NO_MURMUR
#else
	    if (status != MUR_SUCCESS) {
		fprintf (fp, "%s\n", MURMUR_FAIL);
	    }
#endif

	  }         /*  End, message no. 'mval' non-zero   */
	  fclose(fp);
	}            /*  End, not reading/copying file      */
	return(0);
}

/***********************************************************/
void mess_cat ( nstrng, list_val, npar, string, kloc, init_k )
char *nstrng,*string; 
u_long list_val;
long npar; 
int *kloc, init_k;

/*  Copy the NULL separated strings of 'string' contiguously into the */
/*  %s indicated positions of the "interpreted" murmur message, with  */
/*  the whole mess stored in 'nstrng'.                                */

{
	int j, j2, j3;
	char c;
	j3=0;
	j2=0;
	if (list_val != 0)  {
	   for (j=0; j<npar; j++)   {
	      for ( ; (c=opr_mess_list[init_k].mesmess[j2]) != '\0'; j2++) {
		if (c == '%')  {
		   strcpy (&nstrng[j3], &string[kloc[j]]);
		   j3 = strlen(nstrng);
		   j2+=2;
		   break;
		}
		else {
		   nstrng[j3++] = c;
		}
	      }
	   }
	   strcpy (&nstrng[j3], &(opr_mess_list[init_k].mesmess[j2]));
	}
	else {
	   strcpy (nstrng, "Requested murmur msg code not found\n");
	   strcat (nstrng, string);
	}
}
/***********************************************************/
void No_Event()
{
	idle(1);
}
/***********************************************************/
void Exit_Process()
{
	int j;

        if(strcmp(ownpname,"logbook") != 0) {
          return;
        }
	j = Process_Event("logbook RoundUp 0 1\nLogbook Process Stopping");
}
#ifdef NO_MURMUR
u_long mur_send()
{
  return ((u_long)0);
}
#endif

