/*****************************************************************
 UNIwish.c The interactions between UNIDAQ, NOVA and TCL/TK
 author: C. Timmermans
 ******************************************************************/
/************************************************************************
*                                                                       *
* Copyright (c) 1993                                                    *
* The Regents of The University of Michigan                             *
* Universities Research Association, Inc.                               *
* All Rights Reserved                                                   *
*                                                                       *
*                Authors:                                               *
*                        R. Ball, C. Timmermans, UofM                   *
*                        Y. Takeuchi, T.I.T.                            *
*                        M. Nomachi, KEK                                *
*                        A. Fry, C. Erbas, D. Brenner, SSCL             *
*                                                                       *
************************************************************************/
#include <time.h>
#ifdef HP
#include <X11/Xlibint.h>
#endif
#include "tkHead.h" 
#include "UNIwish.h"

#include "template.h"

#include "nova.h"
#define NOVA_HOSTNAME_MAX  32    /*  extracted from novanet.h   */
#define DEFAULT_NAME     "/tmp/nova"
#define DEFAULT_PRI      21

void Process_Idle();
extern char *get_name_from_args();
extern long send_err_mess();

extern char ownpname[];
struct   NOVAD *nd = NULL;
int *data, length,buf_pos,buf_pos_next;

main(argc, argv)
int argc;
char * argv[];
{
  if(Template_Init(get_name_from_args(argc,argv)) < 0){
    printf("Process of name %s already exists\n",
	   get_name_from_args(argc,argv));
    printf("Please use -n to set a different name\n");
    exit(0);
  }

  init_UNIwish(argc,argv);
  Template_Main(NULL, NULL, (void (*)())Process_Idle);
  if(nd != NULL) UNI_nova_close(nd);
  Tcl_DeleteInterp(interp);
  Tcl_DStringFree(&command);
  exit(0);
}

int UNIexit(cd,ip,argc,argv)
ClientData cd;
Tcl_Interp *ip;
int argc;
char *argv[];
{
  Send_Message(ownpname,"exit");
  ip->result = "";
  return TCL_OK;
}

int UNIask(cd,ip,argc,argv)
ClientData cd;
Tcl_Interp *ip;
int argc;
char *argv[];
{
  char tmbf[200];
  int i;

  ip->result = "wrong # args";
  if(argc < 3) return TCL_ERROR;
  ip->result = "";
  tmbf[0] = '\0';
  for(i=2;i<argc;i++) {
    strcat(tmbf,argv[i]);
    strcat(tmbf," ");
  }
  if(Send_Message(argv[1],tmbf) != 0) {
    ip->result = "Error in Send Message";
    return TCL_ERROR;
  }
  return TCL_OK;
}

int UNIvalue(cd,ip,argc,argv)
ClientData cd;
Tcl_Interp *ip;
int argc;
char *argv[];
{
  char val[200];
  int i;
  float f;
  short tp;

  ip->result = "wrong # args";
  if(argc != 3) return TCL_ERROR;
  if(read_value_from_sp(argv[1],argv[2],&tp,val) != 0) {
    ip->result = "Error in read_value";
    return TCL_OK;
  }
  if(tp == INTEGER) {
    memcpy(&i,val,sizeof(int));
    sprintf(val,"%d",i);
  }
  else if(tp == REAL) {
    memcpy(&f,val,sizeof(float));
    sprintf(val,"%g",f);
  }
  ip->result = val;
  return TCL_OK;
}

int UNIvalnum(cd,ip,argc,argv)
ClientData cd;
Tcl_Interp *ip;
int argc;
char *argv[];
{
  char val[30][200],vars[30][60],retval[100];
  int i,ntot,nval;
  float f;
  short tp[30];

  ip->result = "wrong # args";
  if(argc != 3) return TCL_ERROR;
  sscanf(argv[2],"%d",&nval);
  if(read_all_values_from_sp(argv[1],vars,60,tp,val,200,&ntot) != 0) {
    ip->result = "Error in read_value";
    return TCL_OK;
  }
  if(ntot<nval){
    ip->result = "Error in read_value";
    return TCL_OK;
  }
  nval-=1;
  if(tp[nval] == INTEGER) {
    memcpy(&i,val[nval],sizeof(int));
    sprintf(val[nval],"%d",i);
  }
  else if(tp[nval] == REAL) {
    memcpy(&f,val[nval],sizeof(float));
    sprintf(val[nval],"%g",f);
  }
  sprintf(retval,"%s %s",vars[nval],val[nval]);
  ip->result = retval;
  return TCL_OK;
}

int UNInumvar(cd,ip,argc,argv)
ClientData cd;
Tcl_Interp *ip;
int argc;
char *argv[];
{
  char val[30][200],vars[30][60],retval[100];
  int ntot;
  short tp[30];

  ip->result = "wrong # args";
  if(argc != 2) return TCL_ERROR;
  if(read_all_values_from_sp(argv[1],vars,60,tp,val,200,&ntot) != 0) {
    ip->result = "Error in read_value";
    return TCL_OK;
  }
  sprintf(retval,"%d",ntot);
  ip->result = retval;
  return TCL_OK;
}

int UNIdate(cd,ip,argc,argv)
ClientData cd;
Tcl_Interp *ip;
int argc;
char *argv[];
{
  int dtsec;
  char val[200];

  ip->result = "wrong # args";
  if(argc != 2) return TCL_ERROR;
  sscanf(argv[1],"%d",&dtsec);
  strcpy(val,ctime((time_t *)&dtsec)); 
  val[strlen(val)-1] = '\0';
  ip->result = val;
  return TCL_OK;
}

int NOVAget(cd,ip,argc,argv)
ClientData cd;
Tcl_Interp *ip;
int argc;
char *argv[];
{
  int status;

/*** Has NOVA started ??? ***/
  if (nd == NULL) open_NOVA();
  if(nd == NULL) {
    ip->result = "-1";
    return TCL_OK;
  }
  ip->result = "0";
  alarm(1);
  status = UNI_nova_get(nd, &data, &length);
  if (status != 0) ip->result = "-1"; 
  buf_pos = 0;
  buf_pos_next = 0;
  return TCL_OK;
}

int NOVAput(cd,ip,argc,argv)
ClientData cd;
Tcl_Interp *ip;
int argc;
char *argv[];
{

  ip->result = "0";
/*** Has NOVA started ??? ***/
  if (nd == NULL) return TCL_OK;
  UNI_nova_put(nd, data, length, 0); 
  return TCL_OK;
}
int UNInext(cd,ip,argc,argv)
ClientData cd;
Tcl_Interp *ip;
int argc;
char *argv[];
{

  ip->result = "-1";
  buf_pos = buf_pos_next;
/*** Go through individual events ***/
  if ((buf_pos*4) >= length) return TCL_OK;
  if (data[buf_pos] == 0) return TCL_OK;
  buf_pos_next = buf_pos+data[buf_pos];
  ip->result = "0";
  return TCL_OK;
}
int UNIdata(cd,ip,argc,argv)
ClientData cd;
Tcl_Interp *ip;
int argc;
char *argv[];
{
  int i,tpe;
  char tmp[200];

  ip->result = "wrong # args";
  if(argc != 3) return TCL_ERROR;
  if(sscanf(argv[1],"%d",&i) != 1){
    ip->result = "error in arg 1";
    return TCL_ERROR;
  }
  if(strncasecmp(argv[2],"INTEGER",strlen(argv[2])) == 0) tpe = 0;
  else if(strncasecmp(argv[2],"REAL",strlen(argv[2])) == 0) tpe = 1;
  else if(strncasecmp(argv[2],"STRING",strlen(argv[2])) == 0) tpe = 2;
  else {
    ip->result = "error in arg 2";
    return TCL_ERROR;
  }
  if (tpe == 0) sprintf(tmp,"%d",data[buf_pos+6+i]);
  else if (tpe == 1) sprintf(tmp,"%g",*((float *)(&(data[buf_pos+6+i]))));
  else strcpy(tmp,(char *)(&(data[buf_pos+6+i])));
  ip->result = tmp;
  return TCL_OK;
}

int UNIheader(cd,ip,argc,argv)
ClientData cd;
Tcl_Interp *ip;
int argc;
char *argv[];
{
  int i;
  char tmp[200];

  ip->result = "wrong # args";
  if(argc != 2) return TCL_ERROR;
  if(sscanf(argv[1],"%d",&i) != 1){
    ip->result = "error in arg";
    return TCL_ERROR;
  }
  sprintf(tmp,"%d",data[buf_pos+i]);
  ip->result = tmp;
  return TCL_OK;
}

init_UNIwish(argc,argv)
int argc;
char * argv[];
{
/*** Directly copied from wish, as distributed in tk3.3b3 ***/
  char *args, *p, *msg;
  char buf[20];
  int code;

  interp = Tcl_CreateInterp();
#ifdef TCL_MEM_DEBUG
  Tcl_InitMemory(interp);
#endif

  /*
   * Parse command-line arguments.
   */
  
  if (Tk_ParseArgv(interp, (Tk_Window) NULL, &argc, argv, argTable, 0)
      != TCL_OK) {
    fprintf(stderr, "%s\n", interp->result);
    Send_Message(ownpname,"exit");
    return(0);
  }
  if (name == NULL) {
    if (fileName != NULL) {
      p = fileName;
    } else {
      p = argv[0];
    }
    name = strrchr(p, '/');
    if (name != NULL) {
      name++;
    } else {
      name = p;
    }
  }

  /*
   * If a display was specified, put it into the DISPLAY
   * environment variable so that it will be available for
   * any sub-processes created by us.
   */
  
  if (display != NULL) {
    Tcl_SetVar2(interp, "env", "DISPLAY", display, TCL_GLOBAL_ONLY);
  }

  /*
   * Initialize the Tk application.
   */

  mainWindow = Tk_CreateMainWindow(interp, display, name);
  if (mainWindow == NULL) {
    fprintf(stderr, "%s\n", interp->result);
    Send_Message(ownpname,"exit");
    return(0);
  }
  Tk_SetClass(mainWindow, "Tk");
  if (synchronize) {
    XSynchronize(Tk_Display(mainWindow), True);
  }
  Tk_GeometryRequest(mainWindow, 200, 200);
  
  /*
   * Make command-line arguments available in the Tcl variables "argc"
   * and "argv".  Also set the "geometry" variable from the geometry
   * specified on the command line.
   */
  
  args = Tcl_Merge(argc-1, argv+1);
  Tcl_SetVar(interp, "argv", args, TCL_GLOBAL_ONLY);
  ckfree(args);
  sprintf(buf, "%d", argc-1);
  Tcl_SetVar(interp, "argc", buf, TCL_GLOBAL_ONLY);
  Tcl_SetVar(interp, "argv0", (fileName != NULL) ? fileName : argv[0],
	     TCL_GLOBAL_ONLY);
  if (geometry != NULL) {
    Tcl_SetVar(interp, "geometry", geometry, TCL_GLOBAL_ONLY);
  }
  
  /*
   * Set the "tcl_interactive" variable.
   */
  
  tty = isatty(0);
  Tcl_SetVar(interp, "tcl_interactive",
	     ((fileName == NULL) && tty) ? "1" : "0", TCL_GLOBAL_ONLY);
  
  /*
   * Add a few application-specific commands to the application's
   * interpreter.
   */
  
#ifdef SQUARE_DEMO
  Tcl_CreateCommand(interp, "square", SquareCmd, (ClientData) mainWindow,
		    (void (*)()) NULL);
#endif

  /*
   * Invoke application-specific initialization.
   */

  if (Tcl_AppInit(interp) != TCL_OK) {
    fprintf(stderr, "Tcl_AppInit failed: %s\n", interp->result);
  }
  
  /*
   * Set the geometry of the main window, if requested.
   */
  
  if (geometry != NULL) {
    code = Tcl_VarEval(interp, "wm geometry . ", geometry, (char *) NULL);
    if (code != TCL_OK) {
      fprintf(stderr, "%s\n", interp->result);
    }
  }
  
  /*
   * Invoke the script specified on the command line, if any.
   */
  
  if (fileName != NULL) {
    code = Tcl_VarEval(interp, "source ", fileName, (char *) NULL);
    if (code != TCL_OK) {
      goto error;
    }
    tty = 0;
  } else {
    /*
     * Commands will come from standard input, so set up an event
     * handler for standard input.  If the input device is aEvaluate the
     * .rc file, if one has been specified, set up an event handler
     * for standard input, and print a prompt if the input
     * device is a terminal.
     */
    
    if (tcl_RcFileName != NULL) {
      Tcl_DString buffer;
      char *fullName;
      
      fullName = Tcl_TildeSubst(interp, tcl_RcFileName, &buffer);
      if (fullName == NULL) {
	fprintf(stderr, "%s\n", interp->result);
      } else {
	if (access(fullName, R_OK) == 0) {
	  code = Tcl_EvalFile(interp, fullName);
	  if (code != TCL_OK) {
	    fprintf(stderr, "%s\n", interp->result);
	  }
	}
      }
      Tcl_DStringFree(&buffer);
    }
    Tk_CreateFileHandler(0, TK_READABLE, StdinProc, (ClientData) 0);
    if (tty) {
      Prompt(interp, 0);
    }
  }
  fflush(stdout);
  Tcl_DStringInit(&command);
  (void) Tcl_Eval(interp, "update");
  return(0);
 error:
  msg = Tcl_GetVar(interp, "errorInfo", TCL_GLOBAL_ONLY);
  if (msg == NULL) {
    msg = interp->result;
  }
  fprintf(stderr, "%s\n", msg);
  Tcl_Eval(interp, "destroy .");
  Send_Message(ownpname,"exit");
  return 0;			/* Needed only to prevent compiler warnings. */
}

/*
   Function: int UNIstatusnumvar(cd,ip,argc,argv)
   Returns number of status table entries
*/

int UNIstatusnumvar(cd,ip,argc,argv)
ClientData cd;
Tcl_Interp *ip;
int argc;
char *argv[];
{
  static char retval[100];
  Table prt_table[TABLE_SIZE];
  int num;

  num=Get_Table(prt_table);
  sprintf(retval, "%d", num);
  ip->result=retval;
  return TCL_OK;
}


/*
   Function: int UNIstatusvarinfo(cd,ip,argc,argv)
   Given a status entry,
   Returns process name, node, and status string.
   Truncates node entry to first '.'
*/

int UNIstatusvarinfo(cd,ip,argc,argv)
ClientData cd;
Tcl_Interp *ip;
int argc;
char *argv[];
{
  static char retval[100];
  Table prt_table[TABLE_SIZE];
  int num;
  int entry;
  char this_machine[64];
  char mach_name[128];
  char *PTc;

  if (argc!=2) {
    ip->result="wrong # args";
    return TCL_ERROR;
  }
  sscanf(argv[1], "%d", &entry);
  num=Get_Table(prt_table);
  
  /*sprintf(this_machine, "this_machine");*/
  gethostname(this_machine, 50);
  strcpy(mach_name, (!prt_table[entry].node[0] ? this_machine : prt_table[entry].node));
  /* truncate mach_name to first '.' 
   */
  PTc=mach_name;
  while (*PTc) {
    if (*PTc=='.') *PTc=0;
    PTc++;
  }
  

  if (entry<num)
    {
      sprintf(retval, "%s %s %s", 
	      prt_table[entry].process_name,
	      mach_name,
	      (prt_table[entry].status==ALIVE ? "ALIVE" :
	       (prt_table[entry].status==DEAD ? "DEAD" : "BUSY")));
    }
  else{
    sprintf(retval, "status number out of range\n");
    ip->result=retval;
    return TCL_ERROR;
  }
  ip->result=retval;
  return TCL_OK;
}



/*
 *----------------------------------------------------------------------
 *
 * StdinProc --
 *
 *	This procedure is invoked by the event dispatcher whenever
 *	standard input becomes readable.  It grabs the next line of
 *	input characters, adds them to a command being assembled, and
 *	executes the command if it's complete.
 *
 * Results:
 *	None.
 *
 * Side effects:
 *	Could be almost arbitrary, depending on the command that's
 *	typed.
 *
 *----------------------------------------------------------------------
 */

/* ARGSUSED */
static void
  StdinProc(clientData, mask)
ClientData clientData;		/* Not used. */
int mask;				/* Not used. */
{
#define BUFFER_SIZE 4000
  char input[BUFFER_SIZE+1];
  static int gotPartial = 0;
  char *cmd;
  int code, count;
  
  count = read(fileno(stdin), input, BUFFER_SIZE);
  if (count <= 0) {
    if (!gotPartial) {
      if (tty) {
	Tcl_Eval(interp, "exit");
	Send_Message(ownpname,"exit");
      } else {
	Tk_DeleteFileHandler(0);
      }
      return;
    } else {
      count = 0;
    }
  }
  cmd = Tcl_DStringAppend(&command, input, count);
  if (count != 0) {
    if ((input[count-1] != '\n') && (input[count-1] != ';')) {
      gotPartial = 1;
      goto prompt;
    }
    if (!Tcl_CommandComplete(cmd)) {
      gotPartial = 1;
      goto prompt;
    }
  }
  gotPartial = 0;
  
  /*
   * Disable the stdin file handler while evaluating the command;
   * otherwise if the command re-enters the event loop we might
   * process commands from stdin before the current command is
   * finished.  Among other things, this will trash the text of the
   * command being evaluated.
   */
  
  Tk_CreateFileHandler(0, 0, StdinProc, (ClientData) 0);
  code = Tcl_RecordAndEval(interp, cmd, 0);
  Tk_CreateFileHandler(0, TK_READABLE, StdinProc, (ClientData) 0);
  Tcl_DStringFree(&command);
  if (*interp->result != 0) {
    if ((code != TCL_OK) || (tty)) {
      printf("%s\n", interp->result);
    }
  }

  /*
   * Output a prompt.
   */
  
 prompt:
  if (tty) {
    Prompt(interp, gotPartial);
  }
}

/*
 *----------------------------------------------------------------------
 *
 * Prompt --
 *
 *	Issue a prompt on standard output, or invoke a script
 *	to issue the prompt.
 *
 * Results:
 *	None.
 *
 * Side effects:
 *	A prompt gets output, and a Tcl script may be evaluated
 *	in interp.
 *
 *----------------------------------------------------------------------
 */

static void
  Prompt(interp, partial)
Tcl_Interp *interp;			/* Interpreter to use for prompting. */
int partial;   			        /* Non-zero means there already
					 * exists a partial command, so use
					 * the secondary prompt. */
{
  char *promptCmd;
  int code;
  
  promptCmd = Tcl_GetVar(interp,
		  partial ? "tcl_prompt2" : "tcl_prompt1", TCL_GLOBAL_ONLY);
  if (promptCmd == NULL) {
  defaultPrompt:
    if (!partial) {
      fputs("% ", stdout);
    }
  } else {
    code = Tcl_Eval(interp, promptCmd);
    if (code != TCL_OK) {
      Tcl_AddErrorInfo(interp,
		       "\n    (script that generates prompt)");
      fprintf(stderr, "%s\n", interp->result);
      goto defaultPrompt;
    }
  }
  fflush(stdout);
}

/*
 *----------------------------------------------------------------------
 *
 * Tcl_AppInit --
 *
 *	This procedure performs application-specific initialization.
 *	Most applications, especially those that incorporate additional
 *	packages, will have their own version of this procedure.
 *
 * Results:
 *	Returns a standard Tcl completion code, and leaves an error
 *	message in interp->result if an error occurs.
 *
 * Side effects:
 *	Depends on the startup script.
 *
 *----------------------------------------------------------------------
 */

int
  Tcl_AppInit(interp)
Tcl_Interp *interp;		/* Interpreter for application. */
{
  Tk_Window main;
  
  main = Tk_MainWindow(interp);
  
  /*
   * Call the init procedures for included packages.  Each call should
   * look like this:
   *
   * if (Mod_Init(interp) == TCL_ERROR) {
   *     return TCL_ERROR;
   * }
   *
   * where "Mod" is the name of the module.
   */
  
  if (Tcl_Init(interp) == TCL_ERROR) {
    return TCL_ERROR;
  }
  if (Tk_Init(interp) == TCL_ERROR) {
    return TCL_ERROR;
  }
  /*
   * Create my own global vaiable 
   */
  Tcl_SetVar(interp,"ownpname",ownpname,TCL_GLOBAL_ONLY);
  
  /*
   * Call Tcl_CreateCommand for application-specific commands, if
   * they weren't already created by the init procedures called above.
   */
  Tcl_DeleteCommand(interp,"exit");
  Tcl_CreateCommand(interp,"exit",UNIexit,NULL,NULL);

  Tcl_CreateCommand(interp,"ask",UNIask,NULL,NULL);
  Tcl_CreateCommand(interp,"proc_var",UNIvalue,NULL,NULL);
  Tcl_CreateCommand(interp,"proc_var_num",UNIvalnum,NULL,NULL);
  Tcl_CreateCommand(interp,"proc_num_var",UNInumvar,NULL,NULL);
  Tcl_CreateCommand(interp,"date_val",UNIdate,NULL,NULL);

  Tcl_CreateCommand(interp,"nova_get",NOVAget,NULL,NULL);
  Tcl_CreateCommand(interp,"nova_put",NOVAput,NULL,NULL);
  Tcl_CreateCommand(interp,"next_event",UNInext,NULL,NULL);
  Tcl_CreateCommand(interp,"event_data",UNIdata,NULL,NULL);
  Tcl_CreateCommand(interp,"event_header",UNIheader,NULL,NULL);
  Tcl_CreateCommand(interp,"status_num_var", UNIstatusnumvar,NULL,NULL);
  Tcl_CreateCommand(interp,"status_var_info", UNIstatusvarinfo,NULL,NULL);
  /*
   * Specify a user-specific startup file to invoke if the application
   * is run interactively.  Typically the startup file is "~/.apprc"
   * where "app" is the name of the application.  If this line is deleted
   * then no user-specific startup file will be run under any conditions.
   */
  
  tcl_RcFileName = "~/.wishrc";
  return TCL_OK;}
  


open_NOVA()
{
  char name[NV_MAXNAME + 1];
  char sec_name[NV_MAXNAME + NOVA_HOSTNAME_MAX + 1];


  UNI_nova_parms(DEFAULT_NAME, DEFAULT_PRI, name, sec_name);
  if ((int)(nd = (struct NOVAD *) UNI_nova_open(name, nova_prio, 1)) 
      == -1) {
    nd = NULL;
  }
}


void Process_Idle()
{

/*** The tcl/tk loop ***/
  if(tk_NumMainWindows > 0) {
    if(Tk_DoOneEvent(TK_ALL_EVENTS | TK_DONT_WAIT) == 0) my_wait(1);
  }
  else
    Send_Message(ownpname,"exit");
}

my_wait(sec)
int sec;
/* ///////////////////////////////////////////////////////////////
// waits for the next event without removing it from the queue. The routine 
// waits for a maximum of 'sec' seconds.
// This routine is more or less copied from -XWaitForReadable from the Xlib.
// On the DEC-station, not all include-files are there, so  some things
// are hardcoded.
/////////////////////////////////////////////////////////////// */
{
  struct timeval tv;
/*** MSKCNT is not there, when calculated it is 2 on HP. It cannot hurt 
 *** to assign somewhat more space and be on the safe side ***/ 
/*  unsigned long r_mask[MSKCNT]; */
  unsigned long r_mask[10];
  int result,fd;
  
  tv.tv_sec = sec;
  tv.tv_usec = 0;
/*** Originally   CLEARBITS(r_mask); set the value of all r_mask bits to zero
 *** (from Xlibos.h)                                                ***/
  memset((char *)r_mask,0,10*sizeof(long));
/*** Originally  BITSET(r_mask,display->fd); set the value of the appropriate 
 *** word in the buffer (see the select man-pages for more detail)   ***/
  fd = ConnectionNumber(tkDisplayList->display);
  r_mask[(fd >> 5)] |= (1<<(fd & 31));
/*** Wait for data until a timeout       ***/
  select(fd+1,r_mask,NULL,NULL,&tv);
}

