/*
 *  $USERUNIDAQ/collector/sem_test.c
 */
/************************************************************************
 *	cam3.c       	 22-SEP-1991		Y.Takeuchi		*
 *									*
 *	CAMAC LAM Interrupt test (New)					*
 *									*
 *	cam3 [N_of_interrupt_source [loop_count [timeout]]]		*
 ************************************************************************/

/************************************************************************
*                                                                       *
* Copyright (c) 1993                                                    *
* The Regents of The University of Michigan                             *
* Universities Research Association, Inc.                               *
* All Rights Reserved                                                   *
*                                                                       *
*                Authors:                                               *
*                        R. Ball, C. Timmermans, UofM                   *
*                        Y. Takeuchi, T.I.T.                            *
*                        M. Nomachi, KEK                                *
*                        A. Fry, C. Erbas, D. Brenner, SSCL             *
*                                                                       *
************************************************************************/

#include <stdio.h>
#include <sys/fcntl.h>
#include <sys/file.h>
#include "camlib.h"
#include <signal.h> 
#include <sys/time.h> 

#define DEFN        7 
#define DEFLOP      10
#define DEFTIMEOUT  1000

int n_events, initial_sec;
void sig_handler(), dummy(), exit_handler();
int flag, semid;
struct timeval tp;
struct timezone tzp;

/****************************
 * Main routine             *
 ****************************/
main(argc, argv)
int argc;
char *argv[];
{
    int i, n, mask, status;
    int nafenalam, nafdislam, nafclrlam;
    int dat, q, x;
    int loop, timeo;

/* Read options */
    n     = (argc > 1) ? atoi(argv[1]) : DEFN;
    loop  = (argc > 2) ? atoi(argv[2]) : DEFLOP;
    timeo = (argc > 3) ? atoi(argv[3]) : DEFTIMEOUT;

    printf("n=%d\n", n);

/* Initialize variable */
    mask = 1 << (n - 1);
    nafenalam = NAF(n, 0, 26);
    nafdislam = NAF(n, 0, 24);
    nafclrlam = NAF(n, 0, 10);

/* Open and Initialize CAMAC */
    status = 0;
    status += CAM_Open("/k3922",1);
    status += CSETBR(1);
    status += CSETCR(0);
    status += CGENC();
    status += CGENZ();
    status += CREMI();

    if (status != 0) {
	printf("INIT. ERROR!\n");
	exit(1);
    }

/* Enable LAM */
    status = 0;
    status = CAM_EnableLAM(mask);
    status += CAMAC(nafenalam, &dat, &q, &x);
    if (status != 0) {
	printf("2_Enable LAM ERROR! N=%d  status=0x%x\n", n, status);
/*
	exit(2);
*/
    }

/********
 * loop *
 ********/
    printf("\n\n*** Now waiting LAM ... N=%d Loop=%d Timeout=%d sec\n\n",
	   n, loop, timeo / 100);

    signal(SIGALRM, dummy);
    signal(SIGINT, exit_handler);
    Create_Sem(&semid);
    CESIG((void (*))sig_handler);
    gettimeofday(&tp, &tzp);
    initial_sec = tp.tv_sec;
    n_events = 0;

    while (1) {
	dat=3;
	CAMAC(NAF(4,1,17),&dat,&q,&x);

        CWSIG();
        while (Update_Sem_NoWait(semid, -1) == -1)
           idle(1);
     
	dat=0;
	CAMAC(NAF(4,1,17),&dat,&q,&x);
        n_events++;

        CAMAC(NAF(7, 11, 2), &dat, &q, &x);
	CAMAC(nafclrlam, &dat, &q, &x);    /* clear LAM */

    }
    CAM_DisableLAM();
    CAM_Close();

    printf("\n\n*** cam3 nomal end.\n\n");
}

void sig_handler()
{
    Update_Sem_NoWait(semid,1);
}
  
idle(timeout)
int timeout;
 
{
   alarm(timeout);
   pause();
}
 
void dummy()
{

}

void exit_handler()
{
    gettimeofday(&tp, &tzp);
    printf("# seconds = %d\n", tp.tv_sec - initial_sec);
    printf("# events = %d\n", n_events);
    printf("rate = %f\n",  (float)(n_events)/(float)(tp.tv_sec - initial_sec));
    exit(1);
}
