/*************************************
 This file is modified for collector1
      (original: user_event.c)
           on 6 Dec.1994
           by M.Tairadate.
**************************************/
/*************************************************
 * user_event.c,v
 * Revision 1.3  1993/09/27  19:49:14  stipe
 * eventsource decision making is now done at this level
 *
 * Revision 1.2  1993/07/30  15:20:26  ball
 * modify for UNIDAQ v2.2 changes
 *
 * Revision 1.1.1.1  1993/07/14  19:43:31  dbrenner
 * UNIDAQ 2.1 distribution - July 14,1993
 *
*************************************************/

/************************************************************************
*                                                                       *
* Copyright (c) 1993                                                    *
* The Regents of The University of Michigan                             *
* Universities Research Association, Inc.                               *
* All Rights Reserved                                                   *
*                                                                       *
*                Authors:                                               *
*                        R. Ball, C. Timmermans, UofM                   *
*                        Y. Takeuchi, T.I.T.                            *
*                        M. Nomachi, KEK                                *
*                        A. Fry, C. Erbas, D. Brenner, SSCL             *
*                                                                       *
************************************************************************/

/*
 * this is the code that a user would typically modify
 *   for their own purposes.
 *
 * The following routines must be provided.
 *
 * user_event()  : executed for each event
 *     arguments :
 *         int *buffer;        pointer to buffer area
 *         int *length;        available buffer length
 *         int *event_length;  user_event must set the event length
 *         int *event_ID;      user_event must set the "event ID" word
 *         int *max_length;    max length (words) an event buffer may occupy
 *         int *mode;          mode of event taking, for those using it
 *     return    :
 *         ERANGE              buffer length is not enough for the event.
 *         ETIMEDOUT           timed out on waiting an event
 *         0                   normal return   
 *
 * user_init()   : executed once at the beginning 
 *     arguments :
 *         int *max_length;    maximum event length in words.
 * user_begin()  : executed on "begin" command
 *     arguments :
 *         int *runtype;       runtype variable, for those using it.
 *         int *max_length;    maximum event length in words.
 * user_resume() : executed on "resume" command
 *     arguments :
 *         int *mode;          mode of event taking, for those using it
 * user_pause()  : executed on "pause" command
 *     arguments :
 *         int *mode;          mode of event taking, for those using it
 * user_end()    : executed on "end" command
 *     arguments :
 *         int *runtype;       runtype variable, for those using it.
 * user_exit()   : executed once at the end
 *
 */

#include "template.h"
extern int debug_level;

/*Local headers */
#include "camlib.h"

/* define CAMAC station Number */
#define  TDC     20                    /* TDC */
#define  INTREG   5                    /* Interrupt Register */
#define  OUTREG  15                    /* Output Register */

/* Macros */
#define  ENABLE_INTREG (1<<(INTREG-1)) /* Mask value for LAM */
#define  TIMEOUT 60                    /* Timeout(sec.) for Wait LAM */
#define  MAX_EVENT_SIZE 100            /* Maximum event size */

/* Handles for event source */
extern char eventsource[20];
int eventSrcType;
#define PSEUDO 1
#define OTHER  0

/* Functions */
error_exit(status)
int status; {
    perror(strerror(status));
    exit(status);
}

user_init( max_length ) int *max_length; {
  if (strcmp(eventsource,"PSEUDO") == 0) {
    eventSrcType = PSEUDO;
    *max_length = MAX_EVENT_SIZE;
  }
  else  {
    *max_length = MAX_EVENT_SIZE;
    eventSrcType = OTHER;
  }
  return;
}

user_event(buffer,length,event_length,event_ID,max_length,mode)
     int *buffer;       /* pointer to buffer                            */
     int *length;       /* buffer length                                */
     int *event_length; /* user_event must set the event length         */
     int *event_ID;     /* user_event must set the event ID word        */
     int *max_length;   /* max words an event should occupy             */
     int *mode;         /* mode of event taking, for those who use it   */
{
   int status, a, i, q, x;
   short out=3;                         /* For OUTREG */

  *event_length=0;

  if (eventSrcType == PSEUDO)  {                         /* PSEUDO event */
    pseudo_event ( max_length, event_length, buffer );
    *event_ID = EVENT_RECORD;
    return 0;
  }
  else   {                                               /* OTHER event */

    while(*event_length == 0){
        status = CAMAC(NAF(TDC,   0, 9), buffer, &q, &x );      /* Clear TDC */
        status = CAMAC(NAF(INTREG,0, 9),  &i, &q, &x ); /* Clear LAM request */ 
	/* Enable Interrupt */
	status = CAMAC(NAF(INTREG,0,26),  &i, &q, &x );
	/* Trigger Start */
        status = CAMAC(NAF(OUTREG,0,17), &out, &q, &x ); /* Write to OUTREG */
	/* Wait Interrupt */
	status = CAM_WaitLAM(TIMEOUT);       /* Timeout */
	if( status != 0 ){   
	    printf("Timeout!   status = %d\n", status);
	}

	/* Read Interrupt register */
	status = CAMAC(NAF(INTREG,0,0), &i, &q, &x );
	/* Read Data from TDC*/
	if (i != 0) {
	    for(a=0; a<=3; a++){
	        status = CAMAC(NAF(TDC,a,0), &(buffer[a]), &q, &x );
	        if( status ) error_exit(status);
	    }
	    printf(" buf= %d  %d  %d  %d\n",
		   buffer[0], buffer[1], buffer[2], buffer[3]);

	    *event_length=4;                     /* Number of words */
	}
    }
  
  /*
   * flush_buffer("","") force to forward the nova buffer to the
   * processes on the down-stream.
   * event_length and event_ID must be ready on calling "flush_buffer()"
   */
    if (*mode == 0)  {
      *event_ID = EVENT_RECORD;
      *event_length=10;
    }
    else  {
      *event_ID = ENVIRONMENT_RECORD;
      *event_length=100;
      flush_buffer("","");
    }

  /*
   * end of one event 
   */

    return 0;
  }                                /*  End of OTHER event source switch  */
}

user_begin(run_type,max_length) int *run_type,*max_length; {
  int status;
  if (strcmp(eventsource,"PSEUDO") == 0)  eventSrcType = PSEUDO;
  else                                    eventSrcType = OTHER;

  if (eventSrcType == PSEUDO)  {
    *max_length = 150;
    pseudo_begin();
    return 0;
  }
  else {
   /*
    * Open and Initialize CAMAC 
    */
    status = CAM_Open();
    if( status ) error_exit(status);
    printf("CAM_Open : OK\n");
    status = CSETBR(0);
    if( status ) error_exit(status);
    printf("CSETBR   : OK\n");
    status = CSETCR(0);
    if( status ) error_exit(status);
    printf("CSETCR   : OK\n");
    status = CGENZ();
    if( status ) error_exit(status);
    printf("CGENZ    : OK\n");
    status = CGENC();
    if( status ) error_exit(status);
    printf("CGENC    : OK\n");
    status = CSETI();
    if( status ) error_exit(status);
    printf("CSETI    : OK\n");
    status = CREMI();
    if( status ) error_exit(status);
    printf("CREMI    : OK\n");
    status = CAM_DisableLAM();
    if( status ) error_exit(status);
    printf("CAM_DisableLAM : OK\n");

    return;
  }
}

user_pause( mode ) int *mode; {
  int status;
  
  if (eventSrcType == PSEUDO)  {
    pseudo_pause();
    return 0;
  }
  else {
    status = CAM_DisableLAM();
    if( status ) error_exit(status);
    printf("CAM_DisableLAM : OK\n");

    return;
  }
}

user_resume( mode ) int *mode; {   
  int status, i, q, x;
  short out=3;                         /* For OUTREG */

  if (eventSrcType == PSEUDO)  {
    pseudo_resume();
    return 0;
  }
  else {

   /*
    * Initialize CAMAC 
    */
    status = CSETBR(0);
    if( status ) error_exit(status);
    printf("CSETBR   : OK\n");
    status = CSETCR(0);
    if( status ) error_exit(status);
    printf("CSETCR   : OK\n");
    status = CGENZ();
    if( status ) error_exit(status);
    printf("CGENZ    : OK\n");
    status = CGENC();
    if( status ) error_exit(status);
    printf("CGENC    : OK\n");
    status = CREMI();
    if( status ) error_exit(status);
    printf("CREMI    : OK\n");

    /* Initialize Interrupt Register */
    i = 0;
    status = CAMAC(NAF(INTREG,0, 9), &i, &q, &x );   /* Clear LAM */
    status = CAMAC(NAF(INTREG,0,24), &i, &q, &x );   /* Disable LAM */
    status = CAMAC(NAF(INTREG,0,11), &i, &q, &x );   /* Clear LAM Mask */
    
    /* Enable LAM */
    status = CAM_EnableLAM(ENABLE_INTREG);
    if( status ) error_exit(status);
    printf("CAM_EnableLAM  : OK\n");

    return;
  }
}

user_end( run_type ) int *run_type; {
  int status;

  if (eventSrcType == PSEUDO)  {
    pseudo_end();
    return 0;
  }
  else {
    /*
     * Close CAMAC
     */
    status = CAM_DisableLAM();
    if( status ) error_exit(status);
    printf("CAM_DisableLAM : OK\n");
    status = CAM_Close();
    if( status ) error_exit(status);
    printf("CAM_Close: OK\n");

    return;
  }
}

user_exit() {
   return;
}



