/*---------------------------------------------------------------*/
/*                                                               */
/*      receiver.c                                               */
/*      Donald BRENNER                                           */
/*      Superconducting Super Collider Laboratory                */
/*      July 20, 1992                                            */
/*                                                               */
/*---------------------------------------------------------------*/
#ifdef VXWORKS
#include <stdioLib.h>
#include <msgQLib.h>
#else
#include <stdio.h>
#endif
#include "receiver.h"
#include "template.h"

#include "nova.h"
#define NOVA_HOSTNAME_MAX  32    /*  extracted from novanet.h   */
struct NOVAD *net_nova_open(  ); /*  extracted from novanet.h   */
#define DEFAULT_NAME     "/tmp/nova"
#define DEFAULT_PRI      10 
#define DEFAULT_SLEEP    1

/* for the collector side of nova */
#define DEFAULT_SIGNIFICANCE  70
#define PRI_COLLECTOR         -1 

#define  NTIMEOUT       0
static int n_buffer =0;

int Process_Event();
extern void check_buffer_hibytes();

#ifndef VXWORKS
extern long send_err_mess();
#endif
/* for the receiver part from VxWorks and elsewhere */

struct   NOVAD *nd;

/* for the collector part in UNIX workstation */

struct   NOVAD *ndl;
int *datal, lengthl;

/*(a+out) ------------------------------------------------- */
#ifdef VXWORKS
receiver(argv1,argv2,argv3,argv4,argv5,argv6,argv7,
          argv8,argv9,argv10) 
char *argv1,*argv2,*argv3,*argv4,*argv5,*argv6,*argv7,*argv8,*argv9,*argv10;
{
  char *argv[] = {NULL,argv1,argv2,argv3,argv4,argv5,argv6,argv7,argv8,
		    argv9,argv10}; 
  int argc = 11;
#else
void main (argc, argv)
int argc;
char *argv[];
{
#endif
/*(b+out) -------------------------------------------------
Procedure Name
Main - main procedure of recorder
Synopsis
Description
main(argc, argv) is the main procedure of the dump. It
calls the template with the addresses of the Process_Event()
procedures. It also calls the Template_Init.                           

Superconducting Super Collider Laboratory 1.0
(c+out) --------------------------------------------------- */

   char local_name[NV_MAXNAME + 1];
   char remote_name[NV_MAXNAME + NOVA_HOSTNAME_MAX + 1];

#ifdef VXWORKS
   argv[0] = "receiver";
#endif
   if (debug_level >= 10) printf("------------------------------\n");
   if(argc < 2) {
   	printf(" receiver exiting, you must use receiver -m machine \n");
#ifndef VXWORKS
	send_err_mess(0,"Receiver exiting, you must use receiver -m machine");
#endif
   	exit(1);}
   	
   if(Template_Init(get_name_from_args(argc,argv)) < 0){
     printf("Process of name %s already exists\n",
            get_name_from_args(argc,argv));
     printf("Please use -n to set a different name\n");
     exit(0);
   }

   UNI_nova_parms(DEFAULT_NAME, DEFAULT_PRI, remote_name, local_name);
   if ((int)(ndl = nova_open(local_name, PRI_COLLECTOR, 1)) == -1) {
       if (debug_level >= 10) printf("receiver cannot open %s", local_name);
       exit(1);
   }
   if (debug_level >= 10) printf("ndl opened ----------------------------\n");

   if ((int)(nd = (struct NOVAD *)UNI_nova_open(remote_name, nova_prio, 1)) 
                == -1) {
       printf("Error in opening nova...\n");
       Send_Message ( ownpname, "EXIT" );
   }
   if (debug_level >= 10) printf("nd opened ------------------------------\n");

   Template_Main((int (*)())Process_Event, NULL, NULL);
   if (debug_level >= 10) printf("Template_Main --------------------------\n");
}





/*(a+out) ------------------------------------------------- */
int Process_Event()
/*(b+out) -------------------------------------------------
Procedure Name
Process_Event - is used to dump the incoming data
Synopsis
Description
Process_Event() receives the buffer and display all or part of
the data. 

Superconducting Super Collider Laboratory 1.0
(c+out) --------------------------------------------------- */

{  int len, r_len, r_sig;
#ifndef VXWORKS
   alarm(10);           /* This alarm call makes network nova happy  */
#endif
   if (debug_level >= 10) printf("A ------------------------------\n");
   if(n_buffer == 0) {
    while (nova_get(ndl, &datal, &lengthl) != 0) {
          if (debug_level >= 10) printf(" nova_get failure (a)\n"); 
    } 
   n_buffer=1;
   }
 
   if (debug_level >= 10) printf("B ------------------------------\n");
   while ( (len = net_nova_read_with_sig(nd, nd->base, nd->max_length,
					 &r_len, &r_sig)) <= 0) {
   			  if (len <= 0) return (FALSE);
   			  
   }

   lengthl = len;
   move_them(datal, nd->base, &len);

   check_buffer_hibytes(datal);

   if (debug_level >= 10) printf("C --------------------------len= %d\n",len);
   nova_put(ndl, datal, len, r_sig);
   n_buffer = 0; 

   if (debug_level >= 10) printf("D ------------------------------\n");
   return (TRUE);
}

move_them ( output, input, bytes )
char *output, *input;
int  *bytes;
/*
 *  move remote buffer contents from 'input' to local buffer 'output'.  Move
 *  'bytes' bytes.
 */
{
  int j;
  for (j=0; j<*bytes; j++) *output++ = *input++;
}

/*(a+out) ------------------------------------------------- */
int ExitR(header, arg_string)
char *header, *arg_string;
/*(b+out) -------------------------------------------------
Procedure Name
Exit - processes an end command
Synopsis
Description
End(header, arg_string) processes an exit command.  Calls end to
close the file and then calls nova_close.

Superconducting Super Collider Laboratory 1.0
(c+out) --------------------------------------------------- */

{
   UNI_nova_close(nd);
   nova_close(ndl);
   return(Exit(ownpname, " "));
}
