#!/bin/csh -f
# --------------------------------------------
# user_install.csh
#
# Revision 1.0  1994/02/15  masa
# Revision 1.1  1994/03/30  R.Ball
# Revision 1.2  1994/11/17  R.Ball
#
# --------------------------------------------
#--------------------------------------------------------------------------#
#	This is a c-shell script.  It should be executed once, when        #
#	the user starts use UNIDAQ, to set up the user specific variables  #
#       needed by the UNIDAQ processes to correctly run.                   #
#	Default values are supplied in the prompts.  Output of this script #
#	is the file setup.csh in the user_UNIDAQ tree which can be sourced #
#       from the user's .cshrc file in order to set up the needed UNIDAQ   #
#	environment plus a few other files.  See the settings of the       #
#       "fname" variable for other files which are set up.                 #
#									   #
#	This setup.csh is a super-set of the setup.csh of TOPUNIDAQ, and   #
#       the $TOPUNIDAQ/setup.csh file need not be executed before this     #
#	setup.csh.							   #
#                                                                          #
#       $TOPUNIDAQ/setup.csh must be executed before making user           #
#       installation.                                                      #
#                                                                          #
#--------------------------------------------------------------------------#

echo "-----------------------------------------------------------------------"
echo "             ==> Installation setup script for UNIDAQ <=="
echo "             ==> This is the user-tree installation.  <=="

if ( ! $?TOPUNIDAQ ) then
  echo ""
  echo "Sorry, TOPUNIDAQ is not defined.  Set this environmental variable"
  echo "                 and then try again."
  exit 1
endif

if ( ! -e $TOPUNIDAQ/conf/setup.csh ) then
  echo ""
  echo "Sorry, TOPUNIDAQ is set, but either it is wrong, or the directory"
  echo "was not properly installed (setup.csh does not exist)."
  exit 1
endif

#	Set up the local variables.  Start with USERUNIDAQ.

echo "Variables will now be set.  Defaults are inside [ ]."
echo "Enter a <CR> to accept the default."
echo ""

baddir:
set pd=`pwd`
set pd2=$pd
echo "Setting UNIDAQ user directory."
echo -n "USERUNIDAQ = [$pd] "
set ans=$<
if ( $ans == "" ) then
else
   set pd=$ans
   set pd2="`echo $pd`"
endif
echo ""
if ( ! -e $pd2/user_install.csh ) then
   echo "$pd is not an acceptable directory.  Please try again."
   goto baddir
endif

#	With USERUNIDAQ now known, cd to it and begin to write out the
#	setup files.  Make sure that g+w is available on all files in the
#	tree.

cd $pd2
chmod -R g+w . >&/dev/null

set fname=conf/setup.csh
\rm -f $fname
\rm -f setup.csh
ln -s $fname setup.csh

#	Save this first file name so that the MURMUR_SERVER_ADD can be
#	added to its end.

set savename=$fname

#	Write out the file header

cat >$fname <<ZZZ
#!/bin/csh
#---------------------------------------------------------------#
#          Setup file for UNIDAQ environment variables.         #
#          This file is named "setup.csh" and is                #
#          located in the conf directory of the USERUNIDAQ      #
#	   system.  A link to this file by the same name is     #
#	   provided in the top level directory.                 #
#          Source this file once at login, or from your .cshrc  #
#          file.  Sourcing of the \$TOPUNIDAQ/setup.csh file     #
#	   is not needed, but if done, \$TOPUNIDAQ should be     #
#	   first.                                               #
#---------------------------------------------------------------#

ZZZ

echo '# Setting aliases'	>>$fname
echo 'alias @env-var-rm\'	>>$fname
echo  \''if (${?\\!:1} == 0)  setenv \\!:1 "";'\'\\ 	>>$fname
echo  \''setenv \\!:1 `echo :${\\!:1} | sed -e s~:\\!:2~~g -e s~^:~~`;'\'\\ >>$fname
echo  \''if ("${\\!:1}" == "")  unsetenv \\!:1;'\'\\ >>$fname
echo "" >>$fname

echo 'alias @env-var-prepend\'			>>$fname
echo  \''@env-var-rm \\!:1 \\!:2;'\'\\	>>$fname
echo  \''if (${?\\!:1} == 0)  setenv \\!:1 "";'\'\\		>>$fname
echo  \''if ("${\\!:1}" != "")  setenv \\!:1 :"$\\!:1";'\'\\	>>$fname
echo  \''setenv \\!:1 "\\!:2$\\!:1"'\'				>>$fname
echo ""	>>$fname
    
echo 'alias @env-var-append\'			>>$fname
echo \''@env-var-rm \\!:1 \\!:2;'\'\\		>>$fname
echo \''if (${?\\!:1} == 0)  setenv \\!:1 "";'\'\\		>>$fname
echo \''if ("${\\!:1}" != "")  setenv \\!:1 "$\\!:1": ;'\'\\	>>$fname
echo \''setenv \\!:1 "${\\!:1}"\\!:2'\'\\			>>$fname
echo "" >>$fname

echo 'alias @path-rm'  \''set path = (`echo " $path " | sed "s~ \\!:1 ~ ~g"`)'\'>>$fname
echo 'alias @path-prepend'  \''@path-rm \\!:1; set path = (\\!:1 $path)'\'	>>$fname
echo 'alias @path-append'   \''@path-rm \\!:1; set path = ($path \\!:1)'\'	>>$fname

echo ""	>>$fname

echo '# Remove old UNIDAQ PATH and MANPATH traces, if any' >>$fname
echo 'if ( ${?TOPUNIDAQ} ) then' >>$fname
echo '    @env-var-rm MANPATH $TOPUNIDAQ/man' >>$fname
echo '    @path-rm $TOPUNIDAQ/bin' >>$fname
echo '    @path-rm $TOPUNIDAQ/NOVA/src' >>$fname
echo 'endif' >>$fname
echo ""	>>$fname

echo '# Remove old user UNIDAQ PATH, if any' >>$fname
echo 'if ( ${?USERUNIDAQ} ) then' >>$fname
echo '    @path-rm $USERUNIDAQ/bin' >>$fname
echo 'endif' >>$fname
echo ""	>>$fname

echo '# Now set the variables for this version of UNIDAQ' >>$fname
#echo "setenv TOPUNIDAQ          $TOPUNIDAQ" >>$fname
cat $TOPUNIDAQ/conf/setup.csh | grep 'setenv TOPUNIDAQ' >>$fname
echo "setenv USERUNIDAQ         $pd" >>$fname
echo "" >>$fname
echo "setenv UNIDAQversion      2.3" >>$fname
echo "" >>$fname
set pdsave=$pd
unset pd

#	Define the machine type.  Propagate it from the setting in
#	$TOPUNIDAQ/setup.csh.

set pppname=`cat $TOPUNIDAQ/setup.csh | grep MACHINE`
set pppnm2=`cat $TOPUNIDAQ/setup.csh | grep MACFLAV`
setenv MACHINE $pppname[3]
setenv MACFLAV $pppnm2[3]
echo "setenv MACHINE            $pppname[3]" >>$fname
echo "setenv MACFLAV            $pppnm2[3]" >>$fname
unset pppname
unset pppnm2

#	Set up the CERN_ROOT variable.

if ( $MACHINE == VXWORKS || $MACHINE == HPRT ) then
  set tclused=1
else
  set dcerntst=`printenv|grep CERN_ROOT`
  if ( $#dcerntst == 0 ) then
     set cernroot=""
  else
     set cernroot=`echo $CERN_ROOT`
  endif
  unset dcerntst

  echo "Setting CERN library root."
  echo -n "CERN_ROOT = [$cernroot] "
  set ans=$<
  if ( $ans == "" ) then
  else
     set cernroot=$ans
  endif

  echo "setenv CERN_ROOT          $cernroot" >>$fname
  set crnlop=1
  while ($crnlop == 1)
    set crntyp=`echo $cernroot|grep 93`
    if ($#crntyp == 0) then
      set crntyp=`echo $cernroot|grep 94`
      if ($#crntyp == 0) then
        echo -n "Is this CERN_ROOT from 93 (or earlier) or is it 94 (or later) [93|94]? "
        set ans=$<
        if ($ans == "93") then
          set crnlop=0
          set crntyp=93
        else 
          if ($ans == "94") then
            set crnlop=0
            set crntyp=94
          endif
        endif
      else
        set crnlop=0
        set crntyp=94
      endif
    else
      set crnlop=0
      set crntyp=93
    endif
  end

  unset cernroot
  unset crnlop
  echo ""

#	Some 'fixed' variables.

  echo 'setenv MURMUR_CLIENT_DIR  $TOPUNIDAQ/murmur_client/current' >>$fname
#  echo 'setenv MURMUR_DIR         $TOPUNIDAQ/murmur_dir' >>$fname
  echo "" >>$fname

  echo "If TCL/TK is installed on this machine, and you wish to use it"
  echo "   enter the topdirectory of the TCL/TK installation here, e.g."
  echo "   /usr/local/tcltk    Otherwise, just hit return"
tclans:
  echo -n "top TCL/TK directory [] "
  set ans=$<
  if ( $ans == "" ) then
    echo 'if ( ${?TOPTCLTK} ) then' >>$fname
    echo "   unsetenv TOPTCLTK" >>$fname
    echo "endif" >>$fname
    set tclused=1
  else
    if ( ! -d $ans ) then
	echo ""
	echo "$ans does not exist.  Please choose again."
	goto tclans
    endif

    if ( ! -d $ans/lib ) then
	echo ""
	echo "$ans/lib does not exist.  Please choose again."
	goto tclans
    endif

    if ( ! -e $ans/lib/libtcl.a ) then
	echo ""
	echo "$ans/lib/libtcl.a does not exist.  Please choose again."
	goto tclans
    endif

    set tcllis=`find $ans -name tkInt.h -print|sed -e sx/x\ xg`
    if ( $#tcllis == 0 ) then
        echo "Could not find tk include files, try again please."
        goto tclans
    endif
    set tclused=0

    echo 'if ( ${?TOPTCLTK} ) then' >>$fname
    echo '    @path-rm $TOPTCLTK/bin' >>$fname
    echo 'endif' >>$fname

#	Extract the tk subdirectory relative to TOPTCLTK

    set tclorg=`echo $ans | sed -e sx/x\ xg`
    @ ntclw=$#tclorg
    while ($ntclw > 0)
       shift tcllis
       @ ntclw-=1
    end

    @ ntclw=$#tcllis
    set tclfin=""
    while ($ntclw > 1)
       if ($tclfin != "") then
          set tclfin=$tclfin/
       endif
       set tclfin=$tclfin$tcllis[1]
       @ ntclw-=1
       shift tcllis
    end

    echo "setenv TOPTCLTK           $ans" >>$fname
    echo "setenv TKDIR              $tclfin" >>$fname
    echo ""
    echo "setting TK subdirectory of TCL to $tclfin"
    unset tcllis
    unset ntclw
    unset tclfin
    unset tclorg
    echo "" >>$fname
  endif

endif

#	put out the machine specific compilation variables.

if ( $MACHINE == SUN ) then 
   if ( $MACFLAV == SOLARIS2 ) then
#     echo "setenv CC                 /usr/ucb/cc" >>$fname
#     echo "setenv X11LIB             /opt/NSUG93/X11R5/lib/libX11.a" >>$fname
#     echo "setenv CX11FLAGS          -I/opt/NSUG93/X11R5/include " >>$fname
     echo "setenv CC                 gcc" >>$fname
     echo 'setenv X11LIB             "-L/usr/openwin/lib -lX11"' >>$fname
     echo "setenv CX11FLAGS          -I/usr/openwin/include " >>$fname
   else
     echo "setenv X11LIB             /usr/lib/X11/libX11.a" >>$fname
     echo "setenv CX11FLAGS          -I/usr/include/X11 " >>$fname
   endif
   echo 'setenv LDFLAGS            ""' >>$fname
   echo 'setenv F77FLAGS           ""' >>$fname
#   echo "setenv FF                 f77" >>$fname
   echo "setenv FF                 g77" >>$fname
   if ( $MACFLAV == SOLARIS2 ) then
#     echo 'setenv SUN_EXTLIB         "-L/usr/ucblib -lsocket -lucb -lnsl -ldl" ' >>$fname
     echo 'setenv SUN_EXTLIB         "-L/usr/ucblib -lsocket -lucb -lnsl -ldl -R/usr/lib:/usr/ucblib" ' >>$fname
   else
     echo 'setenv SUN_EXTLIB         "" ' >>$fname
   endif
else if ( $MACHINE == HP ) then
   echo 'setenv X11LIB             "-L/usr/lib/X11R5 -lX11"' >>$fname
   echo "setenv CX11FLAGS          -I/usr/include/X11R5 " >>$fname
   echo "setenv LDFLAGS            -lBSD  " >>$fname
   echo 'setenv F77FLAGS           "+ppu"' >>$fname
   echo "setenv FF                 fort77" >>$fname
   echo 'setenv SUN_EXTLIB         "" ' >>$fname
else if ( $MACHINE == DEC ) then
   echo "setenv X11LIB             /usr/lib/libX11.a" >>$fname
   echo "setenv CX11FLAGS          -I/usr/include/X11" >>$fname
   echo 'setenv LDFLAGS            ""' >>$fname
   echo 'setenv F77FLAGS           ""' >>$fname
   echo "setenv FF                 f77 " >>$fname
   echo 'setenv SUN_EXTLIB         "" ' >>$fname
else if ( $MACHINE == SGI ) then
   echo 'setenv X11LIB             "-lX11 -lXt -lXmu"' >>$fname
   echo "setenv CX11FLAGS          -I/usr/include/X11" >>$fname
   echo 'setenv LDFLAGS            "-lc"' >>$fname
   echo 'setenv F77FLAGS           ""' >>$fname
   echo "setenv FF                 f77 " >>$fname
   echo "setenv SUN_EXTLIB         -lsun " >>$fname
else if ( $MACHINE == Linux ) then
   echo "setenv X11LIB             /usr/X11R6/lib/libX11.a" >>$fname
   echo "setenv CX11FLAGS          -I/usr/X11R6/include/X11" >>$fname
   echo 'setenv LDFLAGS            ""' >>$fname
   echo 'setenv F77FLAGS           ""' >>$fname
   echo "setenv FF                 g77 " >>$fname
   echo 'setenv SUN_EXTLIB         "" ' >>$fname
else if ( $MACHINE == HPRT ) then
   echo "setenv CC                 ccrt" >>$fname
   echo 'setenv X11LIB             "-L$HPRTroot/usr/lib/X11R5 -lX11"' >>$fname
   echo 'setenv CX11FLAGS          "-DXLIB_ILLEGAL_ACCESS -I$HPRTroot/usr/include"' >>$fname
   echo 'setenv LDFLAGS            "-L$HPRTroot/lib -lbsd -lrpc"' >>$fname
   echo 'setenv F77FLAGS           "+ppu"' >>$fname
   echo 'setenv FF                 "f77rt"' >>$fname
   echo 'setenv SUN_EXTLIB         ""' >>$fname
else if ( $MACHINE == VXWORKS ) then
   echo "setenv CC                 cc68k" >>$fname
   echo "setenv LD                 ld68k" >>$fname
   echo "" >>$fname

   cat $TOPUNIDAQ/conf/setup.csh | grep VWROOT >>$fname
   cat $TOPUNIDAQ/conf/setup.csh | grep CPUFLAG >>$fname

   set vxename=conf/vxenvvar.h

   \rm -f $vxename
   touch $vxename
   echo "/* This file contains the VXWORKS env. variables */" >>$vxename
   echo "int n_vx_env_var = 2;"           >>$vxename
   echo " "                               >>$vxename
   echo "struct envvar {"                 >>$vxename
   echo "char *name;"                     >>$vxename
   echo "char *value;"                    >>$vxename
   echo "};"                              >>$vxename
   echo " "                               >>$vxename
   echo "struct envvar vx_env_var[] = { " >>$vxename
   echo -n '"TOPUNIDAQ","'                >>$vxename
   echo -n "$TOPUNIDAQ"                   >>$vxename
   echo '",'                              >>$vxename
   echo -n '"USERUNIDAQ","'               >>$vxename
   echo -n "$pdsave"                      >>$vxename
   echo '",'                              >>$vxename
   echo "};"                              >>$vxename
   touch $vxename

endif

#	Add in the odd variable, not specifically set here.

if ( $MACHINE != VXWORKS ) then
  cat $TOPUNIDAQ/conf/setup.csh | grep BVMELIB >>$fname
  echo "" >>$fname
  cat $TOPUNIDAQ/conf/setup.csh | grep MURMUR_SERVER_ADD >>$fname
  echo "" >>$fname
  echo ""

  echo '# Set the PATH correctly for this UNIDAQ version' >>$fname
  if ( $tclused != "1" ) then
     echo '@path-prepend $TOPTCLTK/bin'    >>$fname
  endif
  unset tclused
  echo '@path-prepend $TOPUNIDAQ/bin'	>>$fname
  echo '@path-prepend $TOPUNIDAQ/NOVA/src' >>$fname
  echo '@path-prepend $USERUNIDAQ/bin'	>>$fname
  echo 'set dmantst=`printenv|grep MANPATH`' >>$fname
  echo 'if ( $#dmantst > 0 ) then' >>$fname
  echo '  @env-var-prepend MANPATH $TOPUNIDAQ/man' >>$fname
  echo 'else' >>$fname
  echo '  setenv MANPATH          $TOPUNIDAQ/man:/usr/man' >>$fname
  echo 'endif' >>$fname
  echo 'unset dmantst' >>$fname
  echo 'rehash' >>$fname
  echo ""	>>$fname


#	End of setup.csh section.
#------------------------------------------------------------------------

#	Find out where to write the log book file.  Save it as an
#	environmental variable.  If this variable is not found, the
#	default from the system installation ($TOPUNIDAQ/log/log.book)
#	will be used by all processes needing access, with very interesting
#	run-time results.

  echo "Setting the location of the log book file."
  echo -n '"Locate log.bookYYMMDD files in [$USERUNIDAQ/log]? "'
  set ans=$<
  if ( $ans == "" ) then
     set logb='"$USERUNIDAQ/log/log.book"'
  else
     set lzz='"'
     set logb="$lzz$ans/log.book$lzz"
     unset lzz
  endif

  echo "setenv LGBK_LOC           $logb"   >> $savename
  unset logb
  echo ""
endif

unset fname


#------------------------------------------------------------------------
#	Write out the node_file

set fname=conf/nodefile
\rm -f $fname

if ( $MACHINE == VXWORKS) then
   set tn=`cat $TOPUNIDAQ/conf/nodefile | grep VXWORKS`
   if ( $#tn == 0 ) then
     echo -n "Enter the VxWorks machine name (name.network format) [] "
     set tnode=$<
     echo $tnode VXWORKS >&fname
   else
     set tnode=$tn[1]
     echo $tn >$fname
   endif
else
   set tnode=`hostname`
   echo $tnode UNIX >$fname
endif

unset fname

#------------------------------------------------------------------------
#	Write out the bin/Start_hostname script

set fname=conf/Start_$tnode
\rm -f $fname

	if ( $MACHINE == VXWORKS ) then
cat >$fname <<ZZZ
cd "$pdsave/conf"
ld <vxshm.o
cd "$TOPUNIDAQ/NOVA/src"
ld <vxnovad
cd "../netsrc"
ld <vxnovand
cd "../.."
sp novad_main, "dummy"
sp novand_main
cd "bin"
ld <msg_server
sp msg_server
ld <status
ld <repair
ld <ask
cd "$pdsave/bin"
ld <collector
ld <recorder
ld <receiver
ZZZ
ln -s $fname Start_$tnode
unset fname
unset tnode
unset pdsave

	else
cat >$fname <<ZZZ
#!/bin/csh
onintr -
source \$USERUNIDAQ/setup.csh
msg_server >& /dev/null &
xpc >& /dev/null  &
if ( ! \$?LOCAL_NOVA ) then
  setenv LOCAL_NOVA /tmp/nova
endif
if(-e \$LOCAL_NOVA) rm \$LOCAL_NOVA
novad \$LOCAL_NOVA  >& /dev/null  &
sleep 1
novand >& /dev/null &
ask xpc START collector
ask xpc START recorder
ask xpc START YES runco
ask xpc START YES logbook
ask xpc START YES xpc_checker
ZZZ

chmod g+wx $fname
chmod u+x $fname
chmod o+x $fname

unset fname
unset tnode

#------------------------------------------------------------------------
#	install the correct analyzer version by creating a link to it.
#	remove any existing link.

        if ( $MACHINE != HPRT ) then
echo " "
echo "Installing (Non-)Dynamic offline analyzer main programs"
if ($MACHINE == DEC) then
  echo "GLobal memory and Non-Dynamic analyzers will be built"
else if ($MACHINE == SUN) then
  echo "GLobal memory, Dynamic, and Non-Dynamic analyzers will be built"
else if ($MACHINE == SGI) then
  echo "Dynamic and Non-Dynamic analyzers will be built"
else if ($MACHINE == HP) then
  echo "Dynamic and Non-Dynamic analyzers will be built"
endif
cd anal/ana.d
\rm pawmain_tool.f
cp pawmain_tool_$crntyp.f pawmain_tool.f
cd ../ana.nd
\rm pawmain_tool.f
cp pawmain_tool_$crntyp.f pawmain_tool.f
cd ../..
sleep 2

	endif # HPRT
        endif # VXWORKS
#------------------------------------------------------------------------
#	Finish with the wind-up message.

cat <<ZZZ
                               ===========
Finished with the user installation setup.  Now you should put into either 
your .login or .cshrc file, the line 

	source $cwd/setup.csh

So you will always be set to use the software.  It is not necessary to
source $TOPUNIDAQ/setup.csh, but if you do, please 
be sure you do it BEFORE $cwd/setup.csh .

To complete the installation, source the above file now, and enter the 
following command:

	make

This will take about 5 minutes depending on the machine type
and create all executables from the user portion of the UNIDAQ system.

-----------------------------------------------------------------------
ZZZ

